'''
This file provides utility functions to 
manipulate DRT tasks

by Morteza Mohaqeqi 2017
'''


import xml.etree.ElementTree as ET
import drt.model


def maxD(task_list):
    maxD = max([j._d for t in task_list 
                        for j in t.get_jobs()]);
    return maxD


def make_graphs_connected(task_list, period):    
    for t in task_list:
        jobs = t._jobs
        for j in jobs:
            out_e = [(v,u) for (v,u) in t._edges if v == j]
            if len(out_e) == 0:
                t._edges[(j,j)] = period



def parse_xml(path):
    tree = ET.parse(path)
    root = tree.getroot()
    
    # parse xml
    tasks = [];
    for tsk in root.iter('task'):
        verts = dict()
        for j in tsk.iter('jobType'):
            jname = j.find('name').text
            WCET = j.find('WCET').text
            dead = j.find('deadline').text
            verts[jname] = drt.model.Job(int(WCET), int(dead), name=jname)
        edges = dict()
        for j in tsk.iter('dependency'):
            src = j.find('src').text
            dst = j.find('dst').text
            sep = j.find('interRelease').text
            edges[(verts[src], verts[dst])] = int(sep)
             
        T = drt.model.DRTTask(verts.values(), edges, name = tsk.get('name'))
        prio = tsk.get('priority')
        if prio:
            T._priority = int(prio)
        
        tasks.append(T)
    return tasks    