# This file is part of libdrt.
#
# Written 2011-2014 by Martin Stigge, Pontus Ekberg.
#
# GPLv3 applies.


from drt.util.logging import ERROR, INFO, DEBUG, DEBUG2, DEBUG3, log, indent_log

class Path(object):
    
    """Basic template for *all* path abstraction classes. Both dbf and
    utilization path abstractions need to implement these methods.
    """

    def __init__(self, jobs, name="p"):
        """Create a Path from the non-empty sequence of jobs."""
        self.jobs = jobs
        self._name = name

    def get_jobs(self):
        return self.jobs

    def get_name(self):
        return self._name

    def set_name(self, name):
        self._name = name

    def get_start(self):
        """Return the first job in the path."""
        raise NotImplementedError

    def get_end(self):
        """Return the last job in the path."""
        raise NotImplementedError

    def get_extended(self, job):
        """Return a list of new paths that are the possible extensions
        of this path with the node <job>.
        """
        raise NotImplementedError

    def __ge__(self, o):
        """Domination relation"""
        raise NotImplementedError

    def __gt__(self, o):
        """Strict domination relation"""
        raise NotImplementedError

    def __str__(self):
        raise NotImplementedError


# vim:textwidth=79:shiftwidth=4:tabstop=4:cc=80:expandtab:autoindent

