'''
 This file is called from java GUI to analyze a task set
 The task set has been saved in an XML file
'''

import drt.model
import drt.analysis.sp
import drt.analysis.edf
import drt.analysis.timesstub.spstub
import xml.etree.ElementTree as ET
import sys
#from __builtin__ import Node

def parse_xml(path):

    tree = ET.parse(path)
    root = tree.getroot()
    
    # parse xml
    tasks = [];
    for tsk in root:
        verts = dict()
        for j in tsk.iter('jobType'):
            name = j.find('name').text
            WCET = j.find('WCET').text
            dead = j.find('deadline').text
            verts[name] = drt.model.Job(int(WCET), int(dead))
        edges = dict()
        for j in tsk.iter('dependency'):
            src = j.find('src').text
            dst = j.find('dst').text
            sep = j.find('interRelease').text
            edges[(verts[src], verts[dst])] = int(sep)
             
        T = drt.model.DRTTask(verts.values(), edges, name = tsk.get('name'))
        
        tasks.append(T)
    return tasks    

# make task set and run analysis
# task_list = parse_xml('C:\\cse561\\times\\3tasks.xml') # sys.argv[1]) # 'DRT.xml') ('new_DRT.xml') #
try:
    '''
    tasks = parse_xml('DRT.xml') # ('new_DRT.xml') # sys.argv[1]) # 
    algorithm = 'SP' #sys.argv[2]   # SP or EDF
    maxD = 100 #sys.argv[3]   # maximum time for which calculate the rf
    '''
    tasks = parse_xml(sys.argv[1]) # 
    algorithm = sys.argv[2]   # SP or EDF
    maxD = int(sys.argv[3])   # maximum time for which calculate the rf
    
    
    task_list = [t for t in tasks if len(t.get_jobs()) > 0]     # exclude empty tasks because the drtlib cannot handle this case
    empty_tasks = [t for t in tasks if len(t.get_jobs()) == 0]    
    TS = drt.model.TaskSet(task_list)
    res =  drt.analysis.sp.is_sp_feasible(TS)
    mrfs = drt.analysis.timesstub.spstub.get_mrf(TS, maxD)
    
    for rf in mrfs:
        print(rf)
    
    
    '''
    res = True    # By default, an empty task set is feasible
    task_order = []
    if (len(TS) > 0):
        if(algorithm == 'SP'):
            task_order = drt.analysis.sp.is_sp_feasible(TS)     # a schedulable order
            if task_order == None:
                res = False  # 'Task set is non-schedulable'                     
        elif(algorithm == 'EDF'):
            res = drt.analysis.edf.is_feasible(TS)

    print (str(res))  
    if(res == True and algorithm == 'SP'):
        task_order = task_order + empty_tasks    
        print (map(str, task_order))
       ''' 
except IndexError as e:
    z = e
    print('Analysis encountered an error in DRTLIB')
    print(z)
#except:
  #  e = sys.exc_info()[2]
 #   print('Analysis encountered an error in DRTLIB')
#    print("Error: %s" % str(e))
