'''
 This file is called from java GUI to generate random task
     set with the given utilization  
'''

# import drt.model
import xml.etree.ElementTree as ET
import task
import random
import sys

def make_xml(TS):
    root = ET.Element("taskSet")
    MINP = 10
    MAX_X = 500
    MAX_Y = 400
    PAD = 50 
    xs = [PAD, MAX_X-PAD, MAX_X-PAD, PAD, 3*PAD, MAX_X-3*PAD, MAX_X-3*PAD, 3*PAD];
    ys = [PAD, PAD, MAX_Y-PAD, MAX_Y-PAD, 3*PAD, 3*PAD, MAX_Y-3*PAD, MAX_Y-3*PAD];
    for task in TS:
        xTask = ET.SubElement(root, "task", name=task.name, utilization=str(task.get_utilization()) )
        ind = 0    
        for j in task._jobs:
            if ind < len(xs):
                rx = xs[ind]
                ry = ys[ind]
                ind = ind + 1
            else:
                rx = random.randint(MINP, MAX_X)
                ry = random.randint(MINP, MAX_Y)
            xJob = ET.SubElement(xTask, "jobType", x=str(rx), y=str(ry))
            ET.SubElement(xJob, "name").text = j.name
            ET.SubElement(xJob, "WCET").text = str(j.E)
            ET.SubElement(xJob, "deadline").text = str(j.D)
                    
        for j in task._jobs:
            for succ in j.succs.keys():
                xEdge = ET.SubElement(xTask, "dependency")
                ET.SubElement(xEdge, "src").text = j.name
                ET.SubElement(xEdge, "dst").text = succ.name
                ET.SubElement(xEdge, "interRelease").text = str(j.succs[succ])
            
                        

    tree = ET.ElementTree(root)
    
    #ET.dump(root)
    #xmlstr = tree.tostring(tree, encoding='utf8', method='xml')
    #rough_string = ElementTree.tostring(root, 'utf-8')
    # xml.etree.ElementTree.tostring(root)
    #xmlstr = ET.tostring(root, encoding=None, method=None)
    
    #print('------------')
    #print(xmlstr)
    #ET.dump(tree)
    # tree.write("RanTS.xml")
    return root    

class Job(object):

    """A basic representation of job types. Each job type has an
    execution time requirement, a deadline and any number of successor
    job types. Each successor is associated with a separation time
    (or period).
    """
    
    def __init__(self, E, D, name='J', task=None):
        self.E = E
        self.D = D
        self.name = name
        self.succs = {}
        self.mindelay = float('Inf')
    
    def add_succ(self, job, separation):
        self.succs[job] = separation
        self.mindelay = min(self.mindelay, separation)

    def get_e(self):
        return self.E

    def get_d(self):
        return self.D

    def get_name(self):
        return str(self.name)

    def get_mindelay(self):
        return self.mindelay

    def set_task(self, task):
        self.task = task

    def get_task(self):
        return self.task

    def __str__(self):
#        return str("%(name)s" % self.__dict__)
        return str("%(name)s(%(E)d, %(D)d)" % self.__dict__)

    def dominates(self, j2, strict=False):
        # Let's not get into the business of overloading < and <=
        # or rely on default behavior
        return (((self.E >= j2.E) and (self.D <= j2.D)) if not strict else
            self.dominates(j2) and ((self.E != j2.E) or (self.D != j2.D)))

class RandomTask(task.Task):

    def __init__(self, size_min, size_max, fanout_min, fanout_max,
            p_min, p_max, e_min, e_max, d_min, d_max, d_implicit=False, 
            d_arbitrary=False, connected=True, name='RandomTask'):
        size = random.randint(size_min, size_max)
        """Strategy:
        (1) Create all jobs with uniformly random E (leave D at None for now!)
        (2) Create all edges with uniformly random P
        (2a) First a circle 1->2->3->...->1 if connected
        (2b) Then all remaining edges
        (3) Assign D to all jobs in accordance with d_arbitrary and d_implicit,
                within these bounds do uniform choice.
        """    

 
        # Create all job types with random parameters.
        jobs = []
        for i in xrange(size):
            e = random.randint(e_min, e_max)
            d = None
            jobs.append(task.Job(e, d, 'v%d' % (i + 1)))
        
        # Connect job i to job i + 1 (to ensure fully connected graph).
        if connected:
            for i in xrange(size):
                p = random.randint(p_min, p_max)
                jobs[i].add_succ(jobs[(i + 1) % size], p)

        # Pick the rest of the successors randomly.
        for job in jobs:
            fanout = random.randint(fanout_min, fanout_max) - 1
            while fanout > 0 and len(jobs) > len(job.succs) + 1:
                succ = random.choice(jobs)
                if succ in job.succs:
                    continue
                p = random.randint(p_min, p_max)
                job.add_succ(succ, p)
                fanout -= 1

        # Set deadlines.
        for job in jobs:
            if d_arbitrary:
                job.D = random.randint(max(job.E, d_min), d_max)
            elif d_implicit:
                # Use d_max here also in case job does not have successors
                job.D = min([d_max] + job.succs.values())
            else:
                job.D = random.randint(max(job.E, d_min), min([d_max] + job.succs.values()))
        self._jobs = jobs
        super(RandomTask, self).__init__(jobs, name)


def boundPassed(criterion, bound, U, cnt):
    passed = False;
    if(criterion == 'U'):
        passed = False if U < bound else True
    else: # (criterion == 'N'):
        passed = False if cnt < bound else True
    return passed
        
        
def randomTaskSet(criterion, bound, taskType):
    U = 0.0;
    tList = [];
    cnt = 1
    done = False    
    while not done:
        # taskType = random.choice(['S','M','L']) 
        '''       
        if taskType == 'S':
            pTask = RandomTask(3, 5, fanout_min=1, fanout_max=3,
                p_min=50, p_max=100, e_min=1, e_max=1, d_min=25, d_max=100, d_implicit=False, 
                d_arbitrary=False, connected=True, name='RandomTask'+str(cnt))
        elif taskType == 'M':
            pTask = RandomTask(5, 9, fanout_min=1, fanout_max=4,
                p_min=100, p_max=200, e_min=1, e_max=1, d_min=50, d_max=200, d_implicit=False, 
                d_arbitrary=False, connected=True, name='RandomTask'+str(cnt))
        elif taskType == 'L':
            pTask = RandomTask(7, 13, fanout_min=1, fanout_max=5,
                p_min=200, p_max=400, e_min=1, e_max=1, d_min=100, d_max=400, d_implicit=False, 
                d_arbitrary=False, connected=True, name='RandomTask'+str(cnt))
        '''
        if taskType == 'S':
            pTask = RandomTask(3, 5, fanout_min=1, fanout_max=3,
                p_min=50, p_max=100, e_min=1, e_max=2, d_min=25, d_max=100, d_implicit=False, 
                d_arbitrary=False, connected=True, name='RandomTask'+str(cnt))
        elif taskType == 'M':
            pTask = RandomTask(5, 9, fanout_min=1, fanout_max=4,
                p_min=100, p_max=200, e_min=1, e_max=4, d_min=50, d_max=200, d_implicit=False, 
                d_arbitrary=False, connected=True, name='RandomTask'+str(cnt))
        elif taskType == 'L':
            pTask = RandomTask(7, 13, fanout_min=1, fanout_max=5,
                p_min=200, p_max=400, e_min=1, e_max=8, d_min=100, d_max=400, d_implicit=False, 
                d_arbitrary=False, connected=True, name='RandomTask'+str(cnt))
        else:
            print('Error: Unknown Task Type:' + taskType)
            break;
                   
        if(pTask.constrained == False):
            print("Error : non constraint task")
            return None
        
        tList.append(pTask)
        U = U + pTask.get_utilization()
        done = boundPassed(criterion, bound, U, cnt);
        cnt = cnt+1
        
    return tList

def customRandomTaskSet(criterion, bound, param):
    U = 0.0;
    tList = [];
    cnt = 1
    done = False
    while not done:
        vMin = param[0]
        vMax = param[1]
        pTask = RandomTask(vMin, vMax, fanout_min=param[2], fanout_max=param[3],
                p_min=param[4], p_max=param[5], e_min=param[6], e_max=param[7], d_min=param[8], d_max=param[9], d_implicit=False, 
                d_arbitrary=False, connected=True, name='RandomTask'+str(cnt))
                   
        if(pTask.constrained == False):
            print("Error : non constraint task")
            return None
        
        tList.append(pTask)
        U = U + pTask.get_utilization()
        done = boundPassed(criterion, bound, U, cnt);
        cnt = cnt+1        
        
    return tList
# make task set 
# task_list = parse_xml('C:\\cse561\\times\\3tasks.xml') # sys.argv[1]) # 'DRT.xml') ('new_DRT.xml') #
#U = float(sys.argv[1])

try:
    # U = .6
    # taskType = 'S'
    args = sys.argv
    # args = ['', '0.5', 'S', 'U']
    bound = float(args[1])  # task set utilization or number of tasks
    taskType = args[2]
    criterion = args[3]
    
    if(taskType == 'C'):   # customized parameters
        param = args[4]
        pList = param.split(',')
        pList = map(int, pList)
        TS = customRandomTaskSet(criterion, bound, pList)
    else:
        TS = randomTaskSet(criterion, bound, taskType)

    root = make_xml(TS)
    xmlstr = ET.tostring(root)    
    print(xmlstr)
except:
    e = sys.exc_info()[2]
    print('Task set generation encountered an error in DRTLIB \r\n')
    print("Error: %s" % str(e))
