'''
 This file is called from java GUI for partitioning a 
 given DRT task set among a number of cores
 The parameters include: scheduling policy; bin packing algorithm (BF, WF);
 Ordering criteria (util. vs. density)    
 The task set is given through an XML file
'''

import drt.model
import drt.analysis.sp
import drt.analysis.edf
import util
import task
import sys

import random
# from responsetime import make_connected

#from test.badsyntax_future3 import result


class Core(object):
    """
    A core with unit speed running a set of tasks
    """
    def __init__(self, maxD):
        self._mrf = [0] * maxD
        self._tasks = []
        self._maxD = maxD
        self._prio_order = []
        
    
    # unused
    '''
    def sp_accept_test_fixed_pr(self, task):
        for j in task.get_jobs():
            e = j.get_e()
            d = j.get_d()
            t = 0
            idle_instant = -1
            while t < d:
                workload = self._mrf[t] + e
                if workload <= t + 1:
                    idle_instant = t
                    break
                else:
                    t = workload - 1
            
            if idle_instant == -1:
                return False
        
        return True
    '''
        
    def sp_accept_test(self, task):
        cList = list(self._tasks) # make a copy of the task list
        cList.append(task)
        TS = drt.model.SPTaskSet(cList)
        prio_order = drt.analysis.sp.is_sp_feasible(TS)
        res = False if (prio_order == 0 or prio_order == None) else True
        if res == True:
            self._prio_order = prio_order        
        return res
    
    def edf_accept_test(self, task):
        cList = list(self._tasks) # make a copy of the task list
        cList.append(task)
        TS = drt.model.SPTaskSet(cList)
        res = drt.analysis.edf.is_edf_schedulable(TS)
        return res
    
    def accept_test(self, task):
        if policy == 'SP':
            return self.sp_accept_test(task)
        elif policy == 'EDF':
            return self.edf_accept_test(task)
        
        print ('Warning: Unknown scheduling policy: ' + str(policy))
        return False
    
    
    def get_utilization(self):
        u = 0;
        for t in self._tasks:
            u = u + t._utilization
        return u
    
    # TO DO: update mrf
    
    def add_task_simple(self, task):    # just adds the new task to the list with no update on mrf function
        if task in self._tasks:
            print('Warning: task already exists in the core task list')
        self._tasks.append(task)
    
    def get_prio_order(self):
        return self._prio_order
    
    def get_mrf(self):
        return self._mrf
      
       
def calc_utilization(T):
    jobs = []
    for j in T.get_jobs():
        jobs.append(task.Job(j.get_e(), j.get_d(), j._name))
    
    for (u,v) in T.get_edges():
        src = [j for j in jobs if j.get_name() == u._name][0]
        dst = [j for j in jobs if j.get_name() == v._name][0]
        src.add_succ(dst, T.get_delay((u,v)) )

    rTask = task.Task(jobs)
    return rTask.get_utilization()
    #self._jobs = jobs
    #super(RandomTask, self).__init__(jobs, name)       
           
       
def init(coreNo, tList):
    drt.analysis.sp.syn = False
    drt.analysis.sp.Abstraction = True                         
    
    maxD = max([j.get_d() for t in tList 
                for j in t.get_jobs()])                         
    
    cores = [0] * coreNo   # list of the cores
    for i in range(coreNo):
        cores[i] = Core(maxD)        
    
    return cores

# Converts a task represented in old format (RTSS2013) to a drtlib task
def convert(pTask):
    pJobs = pTask._sorted_jobs # list(pTask.jobs);
    jobs = [drt.model.Job(j.E, j.D) for j in pJobs]
    i = 0;
    for pj in pJobs:
        pj.my_dual = jobs[i]
        i = i + 1
    edges = dict();
    sorted_edges = list(); # used for deterministic addition of s-edges
    for pj in pJobs:
        for suc in pj.succs.keys():
            edges[(pj.my_dual, suc.my_dual)] = pj.succs[suc]
            sorted_edges.append((pj.my_dual, suc.my_dual));
             
    chann = dict()  
    task = drt.model.DRTTask(jobs, edges, chann, pTask.name)
    task._sorted_jobs = jobs
    task._sorted_edges = sorted_edges
    return task    




def reset_stats():
    drt.analysis.sp.stats_dic_tot = {"total": 0, "tested": 0, "pos": 0, "neg": 0, "time": 0}
    

     

def FFitRand(tList, cores):
    drt.analysis.sp.syn = False
    drt.analysis.sp.Abstraction = True                         
    
    core_t_list = [0] * cores   # list of the tasks assigned to each core
    for i in range(cores):
        core_t_list[i] = []
    
    for T in tList:
        allocated = False
        for i in range(cores):
            cList = list(core_t_list[i])
            cList.append(T)
            reset_stats() 
            TS = drt.model.SPTaskSet(cList)
            res = drt.analysis.sp.is_sp_feasible(TS)
            if res == 1:
                core_t_list[i].append(T)
                allocated = True
                break
            
        if allocated == False:
            return False
    
    return True

def FFitIncr(tList, cores):
    tList = sorted(tList, key=utiliz)
    return FFitRand(tList, cores)


'''
Best Fit Decreasing bin packing method
'''
def BFitDecr(tList, coreNo):
    cores = init(coreNo, tList)
    tList = sorted(tList, key=task_size)  # sorting the tasks with respect to their utilization in a descending order 
    for T in tList:
        allocated = False
        cores = sorted(cores, key=core_util2)
        # cores = sorted(cores, key=core_free_times)
        for i in range(coreNo):
            can_accommodate = cores[i].accept_test(T)
            if can_accommodate == True:
                allocated = True
                cores[i].add_task_simple(T)
                break
            
        if allocated == False:
            return cores
    
    return cores

  
    
'''
Worst Fit Decreasing bin packing method
'''
def WFitDecr(tList, coreNo):
    
    cores = init(coreNo, tList)
    tList = sorted(tList, key=task_size)  # sorting the tasks with respect to their utilization in a descending order 
    for T in tList:
        allocated = False
        cores = sorted(cores, key=core_util)
        # cores = sorted(cores, key=core_free_times)
        for i in range(coreNo):
            can_accommodate = cores[i].accept_test(T)
            if can_accommodate == True:
                allocated = True
                cores[i].add_task_simple(T)
                break
            
        if allocated == False:
            return cores
    
    return cores    


'''
First Fit Decreasing bin packing method
'''
def FFitDecr(tList, coreNo):
    
    cores = init(coreNo, tList)
    tList = sorted(tList, key=task_size)  # sorting the tasks with respect to their utilization in a descending order 
    for T in tList:
        allocated = False
        for i in range(coreNo):
            can_accommodate = cores[i].accept_test(T)
            if can_accommodate == True:
                allocated = True
                cores[i].add_task_simple(T)
                break
            
        if allocated == False:
            return False
    
    return True                


def utiliz(task):   # used in sorted() leads to increasing sort
    return task._utilization

def utiliz2(task):   # used in sorted() leads to decreasing sort
    return -1 * task._utilization

def util_sum(task_list):   # used in sorted() leads to increasing sort
    u_list = [T._utilization for T in task_list]
    return sum(u_list)


def core_util(core):   # used in sorted() leads to increasing sort
    u_list = [task_size(t) for t in core._tasks]
    return sum(u_list)

def core_util2(core):   # used in sorted() leads to a decreasing sort
    u_list = [task_size(t) for t in core._tasks]
    return -1 * sum(u_list)


def task_dens(task):
    max_dens = 0;
    for j in task.get_jobs():
        dns = (j.get_e()+0.0) / j.get_d()
        if dns > max_dens:
            max_dens = dns
    return max_dens

def task_size(task):
    sgn = 1 if decr_sort == True else -1
    
    if task_size_measure == 'utilization':
        return sgn * task._utilization
    elif task_size_measure == 'density':
        max_dens = task_dens(task)
        return sgn * max_dens
    else:
        print('Warning : Unknown task size measure: ' + task_size_measure)
        return 0  
   

def core_free_times(core):   # used in sorted() leads to decreasing sort w.r.t. free space
    mrf = core.get_mrf()
    tot_free = 0
    for t in range(core._maxD):
        # fr = max(0, t + 1 - mrf[t])
        fr = 1 if (t + 1 - mrf[t] > 0) else 0 
        tot_free = tot_free + fr
        
    return -1 * tot_free

# paritioning
def p_run(core_no, tList, BinP):
    if BinP == 'BF':
        res = BFitDecr(tList, core_no)
    elif BinP == 'WF':
        res = WFitDecr(tList, core_no)
        
    return res

# single core
# here, the tasks are not added gradually
# instead, the schedulability is checked in one step  
def u_run(tList):
    coreNo = 1
    cores = init(coreNo, tList)
    
    for i in range(len(tList) - 1):
        T = tList[i];
        cores[0].add_task_simple(T)
    
    T = tList[-1]
    can_accommodate = cores[0].accept_test(T)
    if can_accommodate == True:
        cores[0].add_task_simple(T)
    else:
        cores = init(coreNo, tList) # empty core
        
    return cores

def make_graphs_connected(task_list):
    maxD = max([j._d for t in task_list 
                        for j in t.get_jobs()])
    for t in task_list:
        jobs = t._jobs
        for j in jobs:
            out_e = [(v,u) for (v,u) in t._edges if v == j]
            if len(out_e) == 0:
                t._edges[(j,j)] = maxD + 1


def print_res(res_dic, pop, title, save_to_file):
    result = [(round(u,2), res_dic[u]) for u in sorted(res_dic.keys())]
    result = [map(str,(u,r)) for (u,r) in result]
    result = ["\t".join([u,r]) for [u,r] in result]
    # result = map(str, result)
    print('res: ')
    print('\n'.join(result))
    
    if save_to_file == True:
        res_file = open('Result for ' + title + '.txt', 'w')
        res_file.write("Population = " + str(pop) + "\r\n")
        res_file.write("-------------------------------------\r\n")
        res_file.write('util\t Accept Ratio \r\n') 
        res_file.write('\r\n'.join(result))
            

def tasks_info(tList):
    tasks_info = [t._name + ':' + str(t._utilization) + ':'
                    + str(task_dens(t)) for t in tList]
    tasks_info_str = ','.join(tasks_info)
    return tasks_info_str

random.seed(101)    
rng = random
task_size_measure = 'utilization'    # 'density'
policy = 'EDF'  # 'EDF'    
decr_sort = True 

def main():
    global task_size_measure
    global policy
 
    '''
    core_no = 1
    task_size_measure = 'utilization'
    BinP = 'BF'
    '''
    
    '''  
    tList = parse_xml('100TasksSchedulable.xml') # ('new_DRT.xml') # sys.argv[1]) #  '51tasks.xml')
    core_no = 1
    policy = 'EDF'
    BinP = 'BF'
    task_size_measure = 'utilization'
    analysis_type = 'm'    
    '''
    
    tList = util.parse_xml(sys.argv[1]) # ('new_DRT.xml') # sys.argv[1]) #  '51tasks.xml')
    for T in tList:
        T._utilization = calc_utilization(T)
    
    core_no = int(sys.argv[2])
    policy = sys.argv[3]
    BinP = sys.argv[4]
    task_size_measure = sys.argv[5].lower()
    analysis_type = sys.argv[6].lower()
    
    make_graphs_connected(tList)    # DRTlib works only with strongly connected graphs
    
    if analysis_type == 'u': # uni processor
        cores = u_run(tList)
    else:
        cores = p_run(core_no, tList, BinP)
    
    i = 1;
    for c in cores:
        if policy == 'SP': 
            tasks_info_str = tasks_info(c.get_prio_order())
        else:
            tasks_info_str = tasks_info(c._tasks)
        print('Core ' + str(i) + ': ' + tasks_info_str )
        i = i + 1
    
    
    unalloc = [t for t in tList if 
               len([c for c in cores if t in c._tasks]) == 0]
    tasks_info_str = tasks_info(unalloc)
    print('Unallocated : ' + tasks_info_str )        
  
  
  
if __name__ == "__main__":
    main()
    