from debug import debug_print as dp
import debug


class ExplorationEngine(object):

    """The ExplorationEngine generates all paths in a Task (using the
    abstracton of a given Path class) and stores them in a given
    path store.
    """

    def __init__(self, task, store, path_creator):
        """path_creator is a function taking a list of jobs and returning
        a path object. This can be a class (so the function is implicitly the 
        constructor) or a wrapper function.
        """
        self.task = task
        self.store = store
        self.path_creator = path_creator

    def run(self):
        """Run the ExplorationEngine until completion."""
        dp(3, 0, 'ExplorationEngine starting...')

        # Add all vertices as paths of length 0.
        dp(3, 1, 'Adding 0-paths to path store...')
        paths = [self.path_creator([job]) for job in self.task.jobs]
        debug.down(2)
        self.store.add_paths(paths)
        debug.up(2)
        dp(3, 2, 'Added %d 0-paths to store.' % self.store.get_fresh_num())

        # Build the rest of the paths for the path store one by one.
        dp(3, 1, 'Generating k-paths...')
        while True:
            dp(3, 2, 'Currently %d stored, %d fresh.' %
                (self.store.get_stored_num(), self.store.get_fresh_num()))
            path = self.store.pop_fresh()
            if path is None:
                break
            dp(4, 2, 'Extending %s...' % path)
            u = path.get_end()
            for v in u.succs:
                dp(4, 3, '... with %s. Candidates:' % v)
                debug.down(4)
                ext_paths = path.get_extended(v)
                self.store.add_paths(ext_paths)
                debug.up(4)

        dp(3, 1, 'Done generating paths, %d paths stored in total' %
            self.store.get_stored_num())

