package abstraction;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.LinkedList;

import javax.swing.JPanel;
import javax.swing.JSplitPane;

import services.RunResult;
import util.Dialogs;

import main.Manager;
import main.Util;


public class AbstractionTab extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private	AbstractionView 	view;
	private	AbstractionToolbar 	toolbar;
	private	Manager 			manager;	
	private	int [][]			mrfs;
	private	String []			taskNames;
	private	Color []			colors;
	
	public AbstractionTab(Manager mgr) {
		this.manager = mgr;
		setLayout( new BorderLayout());
		toolbar = new AbstractionToolbar(this);
		view    = new AbstractionView();
		
		JSplitPane splitPaneH = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		splitPaneH.setLeftComponent( toolbar );
		splitPaneH.setRightComponent( view.getScroll() );
		add(splitPaneH);		
		Util.decorateSplit(splitPaneH, toolbar.myPrefWidth());
	}
	

	/*
	 * This is called by AbstractionToolbar
	 */
	protected void MrfDbfButtonPressed(final String metric, final int maxT) {
		if (manager.hasNonEmptyTask() == false) {
			Dialogs.showInfoMsg("There is no non-empty task set to analyze.");
			return;
		}
		
		LinkedList<String> errors = manager.analysisSyntaxErrors();
		if (errors.size() > 0) {
			Dialogs.showDefaultSyntaxErrDialog(errors);
			return;
		}
		
		try {
			manager.showProgressBar();
			new Thread(new Runnable() {
				public void run() {
					runMrfDBFAnalysis(metric, maxT);
					manager.hideProgressBar();
					view.moveToEnd();
				}
			}).start();
		}
		catch(Exception e) {
			manager.hideProgressBar();			
		}
	}
	
	/*
	 * This is called by rfButtonPressed()
	 */
	public void runMrfDBFAnalysis(String metric, int maxT) {
		mrfs = null;
		RunResult res = manager.runMrfDBFAnalysis(metric, maxT);
		if (res.succeeded() == false) {
			clearData();
			return;
		}
		
		try {
			int taskNo = res.linesNo();				
			mrfs = new int[taskNo][maxT];
			for (int i = 0; i < taskNo; i++) {
				String curve = res.getLine(i);
				curve = curve.substring(1, curve.length()-1);	// remove left and right braces
				curve = curve.replaceAll(" ", "");			// remove spaces
				String []vals = curve.split(",");
				int len = Math.min(vals.length, maxT);
				for (int j = 0; j < len; j++) {
					mrfs[i][j] = Integer.parseInt(vals[j]);
				}
			}
		}  catch (Exception e) {
			Util.error(e);
		}
		
		taskNames = Util.extractNames(manager.taskList());
		if (mrfs.length < taskNames.length) {
			clearData();
			return;
		}
		
		colors = Util.randColors(taskNames.length);		
		view.setData(mrfs, taskNames, colors);
		toolbar.setData(taskNames, colors);
		repaint();
	}

	private void clearData() {
		toolbar.setData(new String[]{}, new Color[]{});		
	}


	/* 
	 * Called by AbstractionToolbar.selectedTasksChanged()
	 */
	public void updateSelectedTasks(Integer[] selectedTasks) {
		int len = selectedTasks.length;
		int[][] selected_mrfs = new int[len][];
		String[] selected_task_names = new String[len];
		Color[] selectedColors = new Color[len];
		for (int i = 0; i < len; i++) {
			int task_id = selectedTasks[i];
			selected_mrfs[i] = mrfs[task_id];
			selectedColors[i] = colors[task_id];
			selected_task_names[i] = taskNames[task_id];
		}
		
		view.setData(selected_mrfs, selected_task_names, selectedColors);
		repaint();
	}

	public void updateColor(int row, Color color) {
		if (row < colors.length) {
			colors[row] = color;
			// view.setColor(row, color);
			repaint();
		}		
	}
	
	public void selected() {
		int t = Util.maxD(manager.taskList());
		toolbar.setMaxTime(t);		
	}
}
