package abstraction;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;


import util.Dialogs;
import util.GUIUtil;


class AbstractionToolbar extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private	AbstractionTab 	parent;
	private	TasksTable 		table;
	private	JTextField 		timeField;
	private	JButton 		dbfButton;
	private	int				myPrefWidth;
	
	public AbstractionToolbar(AbstractionTab abstTab) {
		this.parent = abstTab;
		setLayout(new BorderLayout(0, 5));		
		
		table = new TasksTable(this);		
		JPanel header = createHeaderPanel();
		
		JScrollPane scrollPane = new JScrollPane(table);				
		header.setBorder( GUIUtil.getBlackTitleBorder("") );
		scrollPane.setBorder( GUIUtil.getBlackTitleBorder("") );
		
		add(header, BorderLayout.PAGE_START);
		add(scrollPane, BorderLayout.CENTER);
	}

	private JPanel createHeaderPanel() {
		dbfButton = new JButton("  DBF  ");
		JButton mrfButton = new JButton("  MRF  ");
		JLabel	label = new JLabel("Max. Time:");
		timeField = new JTextField();
		label.setLabelFor(timeField);
		
		mrfButton.addActionListener(createRfListner());
		dbfButton.addActionListener(createDBFListner());
		timeField.addActionListener(createTextListener());		
		
		mrfButton.setToolTipText("The Most Abstract Request Function");
		dbfButton.setToolTipText("Demand Bound Function");
		
		JPanel jp = new JPanel(new GridLayout(2, 2, 5, 5));
		jp.add(label);
		jp.add(timeField);
		jp.add(dbfButton);
		jp.add(mrfButton);		
		
		JPanel pp = new JPanel();
		pp.add(jp);
		
		JPanel header = new JPanel();
		header.add(pp);
		
		int w1 = table.getPreferredSize().width;
		int w2 = 2 * dbfButton.getPreferredSize().width + 20;
		myPrefWidth = Math.max(w1, w2);
		return header;
	}
	
	private ActionListener createRfListner() {
		class MRFListner implements ActionListener {
			public void actionPerformed(ActionEvent evt) {
				if (validateTimeValue())
					parent.MrfDbfButtonPressed("MRF", getMaxTime());				
			}
		}
		return new MRFListner();
	}

	private ActionListener createDBFListner() {
		class DBFListner implements ActionListener {
			public void actionPerformed(ActionEvent evt) {
				if (validateTimeValue())
					parent.MrfDbfButtonPressed("DBF", getMaxTime());
			}
		}
		return new DBFListner();
	}

	private ActionListener createTextListener() {
		return new ActionListener() {
		    public void actionPerformed(java.awt.event.ActionEvent e) {
		    	validateTimeValue();
		    }
		};
	}
	
	protected boolean validateTimeValue() {
    	try {
    		int val = Integer.parseInt(timeField.getText());
    		if (val <= 0) {
	        	Dialogs.showErrorMsg("Error: Please enter a positive integer value for \"Max. Time\".");
	        	timeField.selectAll();
	        	timeField.requestFocusInWindow();
	        	return false;
	        } else {
	        	dbfButton.requestFocusInWindow();
	        }
    	} catch (Exception ex) {
    		Dialogs.showErrorMsg("Error: Please enter an integer value for \"Max. Time\".");
    		timeField.selectAll();
        	timeField.requestFocusInWindow();
    		return false;
    	}
    	return true;
	}

	public int myPrefWidth() {
		return myPrefWidth;
	}

	public void selectedTasksChanged(Integer[] selectedRows) {
		parent.updateSelectedTasks(selectedRows);
		
	}

	public void colorChanged(int row, Color color) {
		parent.updateColor(row, color);
	}	
	
	public void setData(String[] names, Color[] colors) {
		table.setData(names, colors);		
	}

	public void setMaxTime(int t) {
		timeField.setText("" + t);
		dbfButton.requestFocusInWindow();
	}
	
	public int getMaxTime() {
		try {
			return Integer.parseInt(timeField.getText());
		} catch (Exception e) {
			return 100;	// a default
		}
	}
}
