package abstraction;

import javax.swing.*;

import main.Util;

import simulator.SimUtil;
import util.GUIUtil;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.geom.Line2D;


/**
 * For showing the simulation
 */
public class AbstractionView extends JPanel implements MouseWheelListener, MouseListener, MouseMotionListener  {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Constants
	 */
	private		static		final 		int 	PAD = 30;
	private		static		final		int 	xPAD = 70;		
	private		static		final 		int 	YMAX = 100;
	private		static		final 		int 	MIN_SCALE = 1;
	private		static		final 		int 	MAX_SCALE = 50;
	
	private		static		final 		String	MAJOR_GRID = "Major Gridlines";
	private		static		final 		String	MINOR_GRID = "Minor Gridlines";
	
	/*
	 * Variables
	 */
	private		JScrollPane scroll 	= null;
	private		int[][]		curves 	= null;
	private		int[]		tot_crv	= null;
	private		String[]	names	= null;
	
	private		int 		panelWidth  = 1000;	// sim panel default width, when new task set is loaded the height will be change accordingly   
	private		int 		panelHeight = 500;	// sim panel default height, when new task set is loaded the height will be change accordingly   
	private		int 		scale = 10;
		
	private 	boolean		show_major_gline = true;
	private 	boolean		show_minor_gline = true;
	private 	int 		mouseStartX;
	private 	int 		mouseStartY;
	private 	Color[] 	colors;	
	
	public AbstractionView () {
		setBackground(Color.WHITE);
		setLayout( new BorderLayout() );
		setPreferredSize(new Dimension(panelWidth, panelHeight));  // TO DO: make the size adjusted according to the number of tasks
		
		scroll = GUIUtil.customScroll(this);
		setAutoscrolls(true); //enable synthetic drag events
		
		curves = new int[0][0];
		names = new String[0];
		
		addMouseListener(this);
		addMouseWheelListener(this);
		addMouseMotionListener(this); //handle mouse drags
		moveToEnd();		
	}
	

	public JScrollPane getScroll() 	{
        return scroll;
	}
	
    protected void paintComponent(Graphics g) {
    	super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        
        int labelSep = (int)Math.ceil(10.0 / scale) * 5;  // 5; //     	
        g2.setPaint(Color.black);
        
        int w = getWidth();
        int h = getHeight();
        
        //  refreshHeader(xPAD);
        drawHTimeLine(g2, xPAD, w-PAD, h-PAD, scale, labelSep);
        drawYAxis(g2, xPAD, h-PAD, PAD, scale, labelSep);
        drawCurves(g2);
    }

	private void drawCurves(Graphics2D g2) {
		if (curves_loaded() == false)
			return;
		
		float thickNess = 1.2f;
		for (int i = 0; i < curves.length; i++) {
			int [] curve = curves[i];
			String name = names[i];
			drawCurve(g2, curve, name, colors[i], thickNess);
		}
		drawCurve(g2, tot_crv, "Total", Color.BLACK, 1.6f /* thickness */);
		int max_t = tot_crv.length * scale;
		drawIdentityCurve(g2, max_t);		// draw the curve f(t) = t		
	}
	
	
	private boolean curves_loaded() {
		if(curves == null || tot_crv == null)
			return false;
		if(curves.length == 0)
			return false;
		return true;
	}


	private void drawCurve(Graphics2D g2, int[] curve, String name, Color cl, float thickness) {
		if (curve.length == 0)
			return;
		//final int r = (int)(1 * scale / 3.0);	// radius
		int h = getHeight();		
		for (int j = 0; j < curve.length - 1; j++) {
			int x1 = xPAD + j * scale;
			int x2 = xPAD + (j+1) * scale;
			int y1 = h - PAD - curve[j] * scale;
			int y2 = h - PAD - curve[j+1] * scale;
			
			if (y1 == y2)
				SimUtil.drawLine(g2, x1, y1, x2, y2, cl, thickness);
			else {
				SimUtil.drawLine(g2, x1, y1, x2, y1, cl, thickness);					
				SimUtil.drawLine(g2, x2, y1, x2, y2, cl, thickness);
				//g2.draw(new Ellipse2D.Float(x2 - r/2, y1 - r/2, r, r));
				//g2.fill(new Ellipse2D.Float(x2 - r/2, y2 - r/2, r, r));					
			}				
		}
		int n = curve.length - 1;
		int xn = xPAD + (n+1) * scale;
		int yn = h - PAD - (last(curve) + 1) * scale;
		if (name.equalsIgnoreCase("Total"))
			SimUtil.drawString(g2, name, xn, yn);
	}
	
	
	public void drawIdentityCurve(Graphics2D g2, int max_t) {
		max_t = Math.min(max_t, getHeight() - 2*PAD);
		max_t = Math.min(max_t, getWidth() - 2*xPAD);
		
		int x0 = xPAD + 0;
		int y0 = getHeight() - PAD - 0;
		int x1 = xPAD + max_t;
		int y1 = getHeight() - PAD - max_t;
		SimUtil.drawLine(g2, x0, y0, x1, y1, Color.BLACK, .1f /* thickness */);
		SimUtil.drawString(g2, "f(t) = t", x1 + scale, y1 - scale);
	}
		
	private void setSizeTotal(int w, int h) {
		setPreferredSize(new Dimension(w, h));  
    	setSize(new Dimension(w, h));      	
	}

    /*
     * Draw and elaborate horizontal time line 
     */
    private void drawHTimeLine(Graphics2D g2, int x1, int x2, int y, double sep, int labelSep) {
    	drawHLine(g2, x1, x2, y, sep, labelSep);
    	int darkC = 200;
    	int lightC = 240;
    	int t = 0;
    	for (float x = x1; x <= x2 - sep; x += labelSep*sep) {
    		String s = "" + t;
    		SimUtil.drawString(g2, s, x, y + PAD / 2);
    		if (show_major_gline) {
    			Color color = new Color(darkC, darkC, darkC, 150);
    			SimUtil.drawLine(g2, x, y, x, PAD, color);
    		}
    		t += labelSep;
    	}    
    	if (show_minor_gline)
    		for (float x = x1; x <= x2 - sep; x += sep) {
    			Color color = new Color(lightC, lightC, lightC, 150);
    			SimUtil.drawLine(g2, x, y, x, PAD, color);
    		}    	
    }
    

    /*
     * Draw horizontal time line
     */
    private void drawHLine(Graphics2D g2, int x1, int x2, int y, double sep, int labelSep) {
    	SimUtil.drawArrow(g2, x1, y, x2, y);
    	for (double x = x1; x <= x2 - sep; x += sep)
    		g2.draw(new Line2D.Double(x, y, x, y+2));
    	
    	for (float x = x1; x <= x2 - sep; x += labelSep*sep)
    		g2.draw(new Line2D.Double(x, y, x, y+4));    	
    }
    
    /*
     * Draw vertical time line
     */
    private void drawYAxis(Graphics2D g2, int x, int y1, int y2, double sep, int labelSep) {
    	SimUtil.drawArrow(g2, x, y1, x, y2-PAD / 2);
    	int t = labelSep;
    	for (float y = (float) (y1-labelSep*sep); y > y2 - sep; y -= labelSep*sep) {
    		String s = "" + t;
    		SimUtil.drawString(g2, s, x - PAD / 2, y);
    		if (show_major_gline) {
    			Color color = new Color(150, 150, 150, 150);
    			SimUtil.drawLine(g2, x, y, getWidth() - PAD, y, color);
    		}
    		t += labelSep;
    	}    
    	if (show_minor_gline)
    		for (float y = (float) (y1 - sep); y > y2 - sep; y -= sep) {
    			Color color = new Color(200, 200, 200, 150);
    			SimUtil.drawLine(g2, x, y, getWidth() - PAD, y, color);
    		}    
    }

    /*
     * handle clicks on the pane (for grid lines)
     */
    public void mousePressed(MouseEvent e) {
    	if (!GUIUtil.rightClick(e))		// right click is used to show/hide gridliens
    		setCursor(new Cursor(Cursor.MOVE_CURSOR));
    	
        mouseStartX = e.getX();
        mouseStartY = e.getY();
    }       
    
    private JPopupMenu createPopupMenu() {
    	JPopupMenu popupMenu = new JPopupMenu();
    	final JCheckBoxMenuItem majorLineMenuItem = new JCheckBoxMenuItem(MAJOR_GRID, show_major_gline);
    	final JCheckBoxMenuItem minorLineMenuItem = new JCheckBoxMenuItem(MINOR_GRID, show_minor_gline);
    	majorLineMenuItem.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (majorLineMenuItem.isSelected())
                	show_major_gline = true;
                else
                	show_major_gline = false;            	
            	repaint();
                }});
    	minorLineMenuItem.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (minorLineMenuItem.isSelected())
                	show_minor_gline = true;
                else
                	show_minor_gline = false;            	
            	repaint();
                }});
    	popupMenu.add(majorLineMenuItem);
    	popupMenu.add(minorLineMenuItem);
    	return popupMenu;
	}

	public void mouseClicked(MouseEvent arg0) {
		
	}


	public void mouseEntered(MouseEvent arg0) {
		
	}


	public void mouseExited(MouseEvent arg0) {
		setCursor(new Cursor(Cursor.DEFAULT_CURSOR));    	
	}


	public void mouseReleased(MouseEvent e) {
		setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		if (GUIUtil.rightClick(e)) {
    		Point p = e.getPoint();
        	int menuX = p.x;
            int menuY = p.y;
        
            JComponent c = (JComponent) e.getSource();
        	JPopupMenu menu = createPopupMenu();
        	menu.show(c, menuX, menuY);                
        }
	}
    
	/*
	 * Called by manager
	 */
	public void setData(int[][] reqFunc, String[] taskNames, Color[] colors) {
		this.curves = reqFunc;
		this.names  = taskNames;
		this.colors = colors;
		
		if (reqFunc == null)
			return;
		
		tot_crv = sum_curves(reqFunc);
		resizeIfNeeded(tot_crv);
	}


	private void resizeIfNeeded(int[] tot_crv) {
		if (tot_crv == null)
			return;
		if (tot_crv.length == 0)
			return;
		
		int w = 2 * xPAD + tot_crv.length * scale;
		int h = 2 * PAD + last(tot_crv) * scale;
		if(w > getWidth() || h > getHeight())
			expand(w, h);
	}
	
	private int last(int []arr) {
		return arr[arr.length - 1];
	}


	private int[] sum_curves(int[][] reqFunc) {
		if (reqFunc.length == 0)
			return new int []{};
		
		int curveLen = reqFunc[0].length;
		int[] res = new int[curveLen];
		for (int j = 0; j < curveLen; j++) {
			int tot = 0;
			for(int i = 0; i < reqFunc.length; i++)
				tot += reqFunc[i][j];
			res[j] = tot;
		}		
		return res;
	}

	
	public void expand(int newW, int newH) {
		setSizeTotal(newW, newH);
	}

	public void moveToEnd() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					JScrollBar horizBar = scroll.getVerticalScrollBar();
					horizBar.setValue(horizBar.getMaximum());
				} catch (Exception e) {
					System.out.println("An error occured.");
					Util.error(e);
				}				
			}		
		});		  
	}
	
	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {    
	    if (e.isControlDown()) {
	        if (e.getWheelRotation() < 0) {            
	            System.out.println("Zoom-in when scrolling up");
	            scale = Math.min(scale+1, MAX_SCALE);
	            resizeIfNeeded(tot_crv);
	        } else {
	            System.out.println("Zoom-out when scrolling down");
	            scale = Math.max(scale-1, MIN_SCALE);
	        }
	        repaint();
	    }
	    else
	    	scroll.dispatchEvent(e);
	}


	@Override
	public void mouseDragged(MouseEvent e) {
		/*Rectangle r = new Rectangle(e.getX(), e.getY(), 1, 1);
        scroll.scrollRectToVisible(r);
        scrollRectToVisible(r);*/
        JViewport viewPort = scroll.getViewport();
        Point vpp = viewPort.getViewPosition();
        vpp.translate(mouseStartX-e.getX(), mouseStartY-e.getY());
        scrollRectToVisible(new Rectangle(vpp, viewPort.getSize()));
	}

	@Override
	public void mouseMoved(MouseEvent arg0) {
		// TODO Auto-generated method stub		
	}


	
	public void setColor(int row, Color color) {
		colors[row] = color;
		repaint();
		System.out.println("collor set " + row + " " + color);
	}

}   
