package analysis;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JProgressBar;
import javax.swing.JToolBar;

import main.Util;

public class AnalysisToolBar extends JToolBar implements ActionListener {

	private JButton	 	 runBut;
	private JButton 	 stopBut;
	private JProgressBar progressBar;	
	
	private AnalysisTab  analysisTab;
	
	public AnalysisToolBar(AnalysisTab at) {
		analysisTab = at;
		
		createButtons();
		createProggressBar(150, 15);

		add(stopBut);
		add(runBut);
		add(Box.createHorizontalStrut(80));
		add(Box.createHorizontalStrut(40));
		add(progressBar);		
		add(Box.createHorizontalStrut(200));		
		add(Box.createHorizontalGlue());
		
		finished();		// initially not running
	}

	private void createButtons() {
		ImageIcon iconStop = Util.getIcon("/pause.gif");
		ImageIcon iconExec = Util.getIcon("/exec.gif");
		
		stopBut = new JButton(iconStop);
		runBut = new JButton(iconExec);
		
		stopBut.setToolTipText("Cancel");
		runBut .setToolTipText("Run");
				
		stopBut.addActionListener(this);
		runBut.addActionListener(this);		
	}

	private void createProggressBar(int w, int h) {
		Dimension d = new Dimension(w, h);
		progressBar = new JProgressBar();
		progressBar.setIndeterminate(true);
		progressBar.setPreferredSize(d);
		progressBar.setMaximumSize(d);
		progressBar.setVisible(false);
	}
	
	private void stop() {
		finished();
		analysisTab.stop();
	}
	
	protected void finished()	{
		runBut.setEnabled(true);
		stopBut.setEnabled(false);
		hideProgressBar();		
	}
	
	private void start() {
		runBut.setEnabled(false);
		stopBut.setEnabled(true);
		showProgressBar(); 
		
		try {
			analysisTab.run();
		} catch (Exception e) {
			e.printStackTrace();			
			finished();
		}
	}

	public void showProgressBar() {
		progressBar.setVisible(true);		
	}
	
	public void hideProgressBar() {
		progressBar.setVisible(false);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == runBut)
			start();
		else if (e.getSource() == stopBut)
			stop();
		else 
			System.out.println("Warning: Unimplemented action button");				
	}
}
