package analysis;

import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.SwingConstants;

import analysis.model.enums.BinPackPolicy;
import analysis.model.enums.SortCriteria;

import enums.TaskModel;

import util.AbstractPanel;
import util.GUIUtil;
import util.SchedPolicy;

public class SettingsPanel extends AbstractPanel implements ActionListener {
	private		JLabel cNoJL;
	private		JLabel binJL;
	private		JLabel srtJL;
	
	private		JSpinner 		 			coreNoJSp;  
	private		JComboBox<TaskModel>		modelCB;
	private		JComboBox<SchedPolicy>		schedulCB;
	private		JComboBox<BinPackPolicy>	binPackCB;
	private		JComboBox<SortCriteria>		sortCriCB;
	private		JRadioButton 				UButton;
	private		JRadioButton 				MButton;
	private		boolean						UniProcAnalysis = true;

	public SettingsPanel() {
		setCustomBorder("Setting");	
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		constructTextFields();	// default number of cores = 1
		constructComboBoxs();
		
		JPanel basicSetting = createBasicPanel(5, 5); 
		JPanel MCsetting = createMCPanel(5, 5);
		modelCB.addActionListener(this);
		
		add(basicSetting); 
		add(Box.createVerticalStrut(5));
		add(new JSeparator(SwingConstants.HORIZONTAL));
		add(Box.createVerticalStrut(5));
		add(MCsetting); 
	}

	
	private JPanel createBasicPanel(int hgap, int vgap) {
		JLabel analJL = new JLabel("Task Model:");
		JLabel schJL = new JLabel("Sched. Policy:  ");
		JLabel corJL = new JLabel("Platform: ");
		JLabel dummy = new JLabel("");
		JRadioButton[] radios = createRadioButtons();
		
		JPanel p = new JPanel(new GridLayout(4, 2, hgap, vgap));
		p.add(analJL);
		p.add(modelCB);
		p.add(schJL);
		p.add(schedulCB);
		p.add(corJL);
		p.add(radios[0]);
		p.add(dummy);
		p.add(radios[1]);
		
		JPanel jp = new JPanel();	// basic setting 
		jp.setLayout(new FlowLayout());
		jp.add(p);
		
		return jp;
	}

	private JPanel createMCPanel(int hgap, int vgap) {
		// add to a vertical panel
		JPanel panel = new JPanel(new GridLayout(3, 2, hgap, vgap));
		panel.add(cNoJL);
		panel.add(coreNoJSp);
		
		panel.add(binJL);
		panel.add(binPackCB);
		
		panel.add(srtJL);
		panel.add(sortCriCB);
				
		JPanel MCPanel= new JPanel();	// multicore setting 
		MCPanel.setLayout(new FlowLayout());		
		MCPanel.add(panel);		
		return MCPanel;
	}

	// called by the createCombos()
	private void constructComboBoxs() {
		modelCB    = new JComboBox<TaskModel>();
		schedulCB  = new JComboBox<SchedPolicy>();
		binPackCB  = new JComboBox<BinPackPolicy>();
		sortCriCB  = new JComboBox<SortCriteria>();
		
		modelCB.addItem(TaskModel.DRT);
		modelCB.addItem(TaskModel.SDRT);
		schedulCB.addItem(SchedPolicy.EDF);
		schedulCB.addItem(SchedPolicy.SP);		
		binPackCB.addItem(BinPackPolicy.BF);
		binPackCB.addItem(BinPackPolicy.WF);
		sortCriCB.addItem(SortCriteria.utilization);
		sortCriCB.addItem(SortCriteria.density);
	}

	public int getCoreNo() {
		return (Integer) coreNoJSp.getValue();
	}

	public SchedPolicy getSchPolicy() {
		return (SchedPolicy)schedulCB.getSelectedItem();
	}

	public BinPackPolicy getBinPMethod() {
		return (BinPackPolicy)binPackCB.getSelectedItem();
	}

	public SortCriteria getSortCriteria() {
		return (SortCriteria)sortCriCB.getSelectedItem();
	}
	
	public TaskModel getTaskModel() {
		return (TaskModel)modelCB.getSelectedItem();		
	}

	protected void constructTextFields() {
		coreNoJSp = GUIUtil.cearteSpinner(1, 1, 999);
		// create labels
		cNoJL = new JLabel("Cores: ");
		binJL = new JLabel("Bin-Packing: ");
		srtJL = new JLabel("Criterion: ");		
	}
	
	private JRadioButton[] createRadioButtons() {
		UButton = new JRadioButton("Single-core");
		MButton = new JRadioButton("Multicore");

		UButton.setActionCommand("U");
		MButton.setActionCommand("M");
		UButton.setSelected(true);
		disable_Multicore();
		
		UButton.addActionListener(this);
		MButton.addActionListener(this); 

		ButtonGroup grp = new ButtonGroup();
		grp.add(UButton);
		grp.add(MButton);
		
		return new JRadioButton[]{UButton, MButton};
	}

	protected void disable_Multicore() {		
		UniProcAnalysis = true;	// disable buttons
		GUIUtil.disable(cNoJL);
		GUIUtil.disable(binJL);
		GUIUtil.disable(srtJL);
		GUIUtil.disable(coreNoJSp);
		GUIUtil.disable(binPackCB);
		GUIUtil.disable(sortCriCB);
	}
	
	protected void enable_Multicore() {		
		UniProcAnalysis = false;	// enable buttons
		GUIUtil.enable(cNoJL);
		GUIUtil.enable(binJL);
		GUIUtil.enable(srtJL);
		GUIUtil.enable(coreNoJSp);
		GUIUtil.enable(binPackCB);
		GUIUtil.enable(sortCriCB);
	}

	public boolean uniProc() {
		return UniProcAnalysis;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == modelCB) {
			schedulCB.removeAllItems();
			schedulCB.addItem(SchedPolicy.SP);
			
			switch ((TaskModel)modelCB.getSelectedItem()) {
			case DRT:
				schedulCB.addItem(SchedPolicy.EDF);
				MButton.setEnabled(true);
				break;

			case SDRT:
				disable_Multicore();
				UButton.setSelected(true);
				MButton.setEnabled(false);
				break;
			}
		} else {
			String cmd = e.getActionCommand();
        	if (cmd.equalsIgnoreCase("U"))
				disable_Multicore();
			else if (cmd.equalsIgnoreCase("M"))
    	    	enable_Multicore();
		}
	}

}
