package analysis.view;

import util.GUIUtil;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import main.Util;

/* 
 * Representing a single core information
 */
public class CoreTable extends JPanel {
	private final String[] columnNames = {"Task Name", "Util."};
	private final int width = 180;
	private final int height = 380;
	private final int first_col_width = 150;

	
	public CoreTable(Object[][] data, String name, double load, double density) {
		setLayout(new BorderLayout(0, 5)); //  BoxLayout(this, BoxLayout.Y_AXIS));
		
		JLabel title_jp = createTitlePanel(name);          
        JPanel table_jp = createTablePanel(data); 
        JPanel load_jp  = createLoadPanel(load, density);
        
        add(title_jp, BorderLayout.NORTH);
    	add(table_jp, BorderLayout.CENTER);
    	add(load_jp,  BorderLayout.SOUTH);
    	
    	GUIUtil.setSize(this, width, height);
    	GUIUtil.color(table_jp, Color.WHITE);
    	GUIUtil.color(load_jp, Color.WHITE);
    	setBackground(Color.WHITE);
	}

	private JLabel createTitlePanel(String name) {
		return GUIUtil.createTitle(name);		
	}

	private JPanel createTablePanel(Object[][] data) {

		JPanel tablePanel = new JPanel(new BorderLayout());
        tablePanel.setBorder(GUIUtil.getBlackTitleBorder("Tasks"));

        JTable tasksTable = new JTable(data, columnNames);
        tasksTable.getColumnModel().getColumn(0).setPreferredWidth(first_col_width);
        /*  The following (commented) lines are for having a sortable table 
         * tasksTable.setPreferredScrollableViewportSize(new Dimension(70, 70));
        tasksTable.setFillsViewportHeight(true);
        tasksTable.setAutoCreateRowSorter(true);
        tasksTable.setModel(new TaskSetTableModel());        
        TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(tasksTable.getModel());        
        sorter.setComparator(0, Util.getStrIntComparator());
        sorter.setComparator(1, Util.getIntComparator());
        tasksTable.setRowSorter(sorter);
        
        tasksTable.getColumnModel().getColumn(0).setPreferredWidth(50);
        tasksTable.getColumnModel().getColumn(1).setPreferredWidth(20);*/
                
        //Create the scroll pane and add the table to it.
        
        JScrollPane table_scr = new JScrollPane(tasksTable);
        table_scr.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
        
        tablePanel.add(tasksTable.getTableHeader(), BorderLayout.PAGE_START);
        tablePanel.add(table_scr, BorderLayout.CENTER);
        
        return tablePanel;
	}
	

	private JPanel createLoadPanel(double load, double density) {
		JPanel lp = new JPanel(new GridLayout(2, 1, 0, 5));
		lp.add(new JLabel("Utilization = " + Util.round(load, 3), SwingConstants.CENTER));
		// lp.add(new JLabel("Density    = " + Util.round(density, 3), SwingConstants.CENTER));
    	return lp;
	}
}



class TaskSetTableModel extends DefaultTableModel {
    /*
    public int getColumnCount() {
        return columnNames.length;
    }

    
    public String getColumnName(int col) {
        return columnNames[col];
    }*/


    /*
     * JTable uses this method to determine the default renderer/
     * editor for each cell.  If we didn't implement this method,
     * then the last column would contain text ("true"/"false"),
     * rather than a check box.
     */
    public Class getColumnClass(int c) {
        return getValueAt(0, c).getClass();
    }

    /*
     * Only the last column (priority) is editable 
     */
    public boolean isCellEditable(int row, int col) {
        return false;            
    }

    /*
     * Don't need to implement this method unless your table's
     * data can change.
     */
    public void setValueAt(Object value, int row, int col) {
    	
    }
}