'''
 This file is called from java GUI to analyze a task set
 The task set has been saved in an XML file
'''

import drt
import drt.model
import drt.analysis.sp
import xml.etree.ElementTree as ET
import sys
#from __builtin__ import Node

def parse_xml(path):

    tree = ET.parse(path)
    root = tree.getroot()
    
    # parse xml
    tasks = [];
    for tsk in root:
        verts = dict()
        for j in tsk.iter('jobType'):
            name = j.find('name').text
            WCET = j.find('WCET').text
            dead = j.find('deadline').text
            verts[name] = drt.model.Job(int(WCET), int(dead))
        edges = dict()
        for j in tsk.iter('dependency'):
            src = j.find('src').text
            dst = j.find('dst').text
            sep = j.find('interRelease').text
            edges[(verts[src], verts[dst])] = int(sep)
             
        T = drt.model.DRTTask(verts.values(), edges, name = tsk.get('name'))
        
        tasks.append(T)
    return tasks    

# make task set and run analysis
task_list = parse_xml(sys.argv[1]) # 'DRT.xml') ('new_DRT.xml') #
TS = drt.model.TaskSet(task_list)
res = drt.analysis.sp.is_sp_feasible(TS)
if res == None:
    print('Task set non-schedulable')
else:
    print(map(str, res))
