package main;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.Enumeration;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import services.DRTStub;
import util.GUIUtil;


class ConfigPanel extends JDialog implements ActionListener{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private	Manager 	manager;
	private	JTextField 	pyth_path_text;
	private	JTextField 	drt__path_text;
	private	JCheckBox 	perm_save_box;
	private	String 		selectedLookAndFeel;
	private	String[] 	lookAndFeels = new String[]{"Metal", "Nimbus", "Motif", "Windows"};
	
	public	static	final	int		WIDTH = 630;	// width of the frame
	public	static	final	int		HEIGHT	= 420;
	
	public ConfigPanel( Manager mngr ) {
		super(mngr.frame());
    	this.manager = mngr;
        initUI();
        this.setLocationRelativeTo(mngr.frame());
    }

	public final void initUI() {		
        JPanel basic = new JPanel();
        basic.setLayout(new BoxLayout(basic, BoxLayout.Y_AXIS));
        add(basic);

        /*** Top Panel: Icon ***/
        JPanel topPanel = new JPanel(new BorderLayout(0, 0));
        topPanel.setMaximumSize(new Dimension(WIDTH, 0));
        JLabel hint = new JLabel("Configuration Panel ");
        hint.setFont( new Font("Arial", Font.BOLD, 13) );
        hint.setBorder(BorderFactory.createEmptyBorder(0, 15, 0, 0));
        topPanel.add(hint);

        ImageIcon icon = Util.getIcon("/conf.png");
        JLabel label = new JLabel(icon);
        label.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        topPanel.add(label, BorderLayout.EAST);

        JSeparator separator = new JSeparator();
        separator.setForeground(Color.gray);

        topPanel.add(separator, BorderLayout.SOUTH);

        basic.add(topPanel);

        /*** Main (Tabbed) Panel: Configuration ***/
        basic.add(createTabbedPane());

        /*** Buttom Panel: OK / Cancel ***/        
        JPanel boxPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 20, 0));
        perm_save_box = new JCheckBox("Save permanently");
        perm_save_box.setMnemonic(KeyEvent.VK_S);
        perm_save_box.setSelected(false);
        boxPanel.add(perm_save_box);
        basic.add(boxPanel);

        JPanel bottom = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        JButton ok = new JButton("OK");
        ok.setMnemonic(KeyEvent.VK_N);
        ok.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {	
				String pyth 	  = pyth_path_text.getText();
				String drt  	  = drt__path_text.getText();
				boolean save2file = perm_save_box.isSelected();
				manager.drtlib().saveConf(pyth, drt, save2file);
				manager.frame().setLookAndFeel(selectedLookAndFeel);
				close();
			}
		});
        JButton close = new JButton("Calcel");
        close.setMnemonic(KeyEvent.VK_C);
        close.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent arg0) {				
				close();
			}
		});
        
        bottom.add(ok);
        bottom.add(close);
        basic.add(bottom);

        bottom.setMaximumSize(new Dimension(450, 0));

        /*** Usual Settings ***/
        setTitle("Configuration");
        setSize(new Dimension(WIDTH, HEIGHT));
        setResizable(false);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setLocationRelativeTo(null);
    }

    private JTabbedPane createTabbedPane() {
    	JPanel feeljp = createLookAndFeelPnl();
    	JPanel pathjp = createDeveloperCfgPnl();
		
        /*** Create Tabbed Pane ***/
        JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane .setPreferredSize(new Dimension(450, 200));
		tabbedPane.addTab ("Developer Cfg.", pathjp);			
		// tabbedPane.addTab ("Look and Feel", feeljp);	
 		
		tabbedPane.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
		tabbedPane.setFont(new Font("", Font.PLAIN, 12));
		return tabbedPane;
    }
    
	private JPanel createLookAndFeelPnl() {
		selectedLookAndFeel = manager.frame().getLookAndFeel();
		ActionListener volumeListener = new ActionListener(){
            public void actionPerformed(ActionEvent evt) {
            	selectedLookAndFeel = evt.getActionCommand();
            }};
            
        ButtonGroup grp = new ButtonGroup();
        JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(lookAndFeels.length, 1));		
		for (String laf : lookAndFeels) {
			JRadioButton b = new JRadioButton(laf);
			b.setActionCommand(laf);
			if (selectedLookAndFeel.equalsIgnoreCase(laf))
				b.setSelected(true);
			b.addActionListener(volumeListener);
			grp.add(b);
			panel.add(b);
		}	
        panel.setBorder(GUIUtil.getEmptyBorder(30));
		return panel;
	}

	private JPanel createDeveloperCfgPnl() {

    	DRTStub drt = manager.drtlib();
    	JPanel pathjp = new JPanel();
    	pathjp.setLayout(new BoxLayout(pathjp, BoxLayout.Y_AXIS));     
        
        JPanel up = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        JLabel pyth_path = new JLabel( "Python Path:" ); 
        pyth_path.setFont(new Font("", Font.PLAIN, 12));
        pyth_path_text = new JTextField( 29 );
        pyth_path_text.setPreferredSize(new Dimension(29,25) );
        pyth_path_text.setFont(new Font("", Font.PLAIN, 12));
        pyth_path_text.setText(drt.getPythPath());
        JButton browse = new JButton( "Browse" );
        browse.setFont(new Font("", Font.PLAIN, 12));
        browse.addActionListener(new BrowseActListener(pyth_path_text, drt.getPythPath()));
        up.add(pyth_path);
        up.add(pyth_path_text);
        up.add(browse);
        
        JPanel up2 = new JPanel(new FlowLayout(FlowLayout.RIGHT));      
        JLabel drt_path = new JLabel( "drtlib Path:" );
        drt_path.setFont(new Font("sansserif", Font.PLAIN, 12));
        drt__path_text = new JTextField( 29 );
        drt__path_text.setFont(new Font("", Font.PLAIN, 12));
        drt__path_text.setText(drt.getDrtPath());
        drt__path_text.setPreferredSize(new Dimension(29,25) );
        JButton browse2 = new JButton( "Browse" );     
        browse2.setFont(new Font("", Font.PLAIN, 12));
        browse2.addActionListener(new BrowseActListener(drt__path_text, drt.getDrtPath()));
        up2.add(drt_path);
        up2.add(drt__path_text);
        up2.add(browse2);
                
        pathjp.add(Box.createRigidArea(new Dimension(0, 20)));
        pathjp.add(up);
        pathjp.add(Box.createRigidArea(new Dimension(0, 1)));
        pathjp.add(up2);
        pathjp.add(Box.createRigidArea(new Dimension(0, 75)));

        pathjp.add(Box.createVerticalGlue());
        return pathjp;
	}
	
	public void actionPerformed(ActionEvent e) {
		SwingUtilities.invokeLater(new Runnable() {
            public void run() {
            	ConfigPanel ex = new ConfigPanel( manager );
                ex.setVisible(true);
            }
        });		
	}
			
	class BrowseActListener implements ActionListener
	{
		JTextField related_text;
		String default_path;
		
		public BrowseActListener(JTextField text, String path) {
			related_text = text;
			default_path = path;
		}
		
		public void actionPerformed(ActionEvent e) {
			JFileChooser fileDialog = Util.getFileChooser("Select", default_path, ".*");
            int option = fileDialog.showDialog(null, "OK");
            if (option != JFileChooser.APPROVE_OPTION)
                return;      // User canceled or clicked the dialogue close box.
            File selectedFile = fileDialog.getSelectedFile();
            related_text.setText( selectedFile.getPath() );	
            related_text.setFont(new Font("", Font.PLAIN, 12));            
		}		
	}
	
	/*
	 * Closes the config. panel
	 */
	public void close() {
		setVisible(false);
		dispose();
	}
}

enum TASK_GEN_METHODS{
	UTILIZATION,
	NUM_OF_TASKS
}
