package main;
import javax.swing.*;

import taskeditor.DrawingPane;
import taskeditor.changetrace.JobChange;
import taskeditor.movables.JobIcon;
import taskeditor.movables.JobLabel;
import util.Dialogs;
import util.GUIUtil;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

/**
 * Created by Morteza Mohaqeqi
 * Date: Mar 25, 2015
 */
public class JobType {
	public 	int     	WCET;
	public	int     	deadline;
    private String  	name;
    private String  	codeStr;				// the code segment for this job type
    
    private	JobIcon		icon;
    private	JobLabel  	label;					// Job label: (WCET, Deadline)
    public  int  		x, y;
    private LinkedList<Edge>    branchOrders;	// order of outgoing edges, used for code generation	
    private final 	int 		LABEL_Y_OFFSET = 40;	// vertical  adjustment    
    
	public JobType(String name, int WCET, int DdLn, int x, int y) {
		this.name	  = name;
		this.WCET 	  = WCET;
		this.deadline = DdLn;
		this.x  	  = x;
		this.y  	  = y;
		branchOrders  = new LinkedList<Edge>();
		label		  = new JobLabel(this, x, y - LABEL_Y_OFFSET);
	}

	public JobType(String namn, String wcet, String ddln, int x, int y) {
		this(namn, Util.parseInt(wcet), Util.parseInt(ddln), x, y);		
	}
	
	public JobType(String namn, int wcet, int ddln, String code, int x, int y) {
		this(namn, wcet, ddln, x, y);
		codeStr = code;
	}

	public void updateName(String newName) {
		setName(newName);
        icon.setText(getName());        
	}

	/*
	 * This is called in two ways:
	 * 	1. Through the job table    (see TaskPanel.NewTaskActionListner)
	 *  2. Through clicking on the task graph  (see DrawingPane.NodeMouseListener) 
	 */
	public void updateWCET_DL(int WCET, int deadline) {
		this.WCET = WCET;
		this.deadline = deadline;
		label.setText(WCET, deadline);
	}

	public void addEdge(Edge e) {
		if (branchOrders.contains(e))
			System.out.println("Warning: Edge " + e.toString() 
				+ " already exists in the branch order list of job " + name);
		else
			branchOrders.add(e);
	}

	public void removeEdge(Edge e) {
		branchOrders.remove(e);
	}

	public LinkedList<Edge> getBranchOrders() {
		return branchOrders;
	}	

	public void getBranchCondFromUser() {		
		LinkedList<Edge> outEdges = branchOrders;
		/*if ( branchOrders.containsKey(job) )
			outEdges = branchOrders.get(job);
		else
			outEdges = outGoingEdges(job);
		*/
		if (outEdges.size() == 0) {
			Dialogs.showWarnMsg("There is no outgoing edge for the selected job");
			return;
		}

		GetCodePanel codePanel = new GetCodePanel(outEdges);
		int result = Dialogs.showOkCancelPanel(codePanel, "Branching Conditions");
        
        // apply the changes
        if (result == JOptionPane.OK_OPTION) {
        	LinkedList<Edge> newOrder = new LinkedList<Edge>();
        	for (int i = 0; i < codePanel.bPanels.size(); i++) {
        		BranchCodePanel bp = codePanel.bPanels.get(i);
        		bp.commit();
        		newOrder.add(bp.edge);
        	}
        	branchOrders = newOrder;
        }
	}
	
	public void clearBranchOrders() {
		branchOrders.clear();
	}

	public JobChange getModifyChange() {
		JobChange ch = createChange();
		ch.beModify();
		return ch;
	}

	public JobChange getCreateChange() {
		JobChange ch = createChange();
		ch.beCreate();
		return ch;
	}

	public JobChange getRemoveChange() {
		JobChange ch = createChange();
		ch.beRemove();
		return ch;
	}
	
	private JobChange createChange() {
		JobChange ch = new JobChange();
		ch.setRef(this);
		ch.setOrig(this.cloneMe());		
		return ch;
	}

	public JobType cloneMe() {
		JobType j = new JobType(new String(name), WCET, deadline, codeStr, x, y);
		j.label = label.cloneMe(j);		
		return j;
	}
	
	// called for undoing some previous modification
	public void loadData(JobType j) {
		this.x 	     = j.x;
		this.y 	     = j.y;
		this.name    = j.name;
		this.label.adjustSizeAndLocation(j.label.getXY());
		this.codeStr = j.codeStr;
		updateWCET_DL(j.WCET, j.deadline);
	}

	// Called when the job is going to be shown
	public void createLabel(DrawingPane drawPane, boolean isStartJob) {
		icon = new JobIcon(drawPane, this, isStartJob, x, y);
		if (label.listenersSet() == false) {
			label.setListeners(drawPane, this);
       }     
	}
	
	public void deselect() {
		GUIUtil.color(icon, Color.black);
		GUIUtil.color(label, Color.black);		
	}
	
	public void selected() {
		GUIUtil.color(icon, Color.blue);
		GUIUtil.color(label, Color.blue);		
	}
	
	public void moveLabel(int dx, int dy) {
		label._move(dx, dy);
	}	

	public Point getLabelXY() {		
		return label.getXY();
	}

	public void setLabelXY(int x, int y) {
		label.adjustSizeAndLocation(x, y);
	}
	
	/* 
	 * Getters and Setters:
	 */	
	public int WCET() {
		return WCET;
	}

	public int deadline() {
		return deadline;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public String getCode() {
		return codeStr;
	}

	public void setCode(String s) {
		codeStr = s;
	}	
	
	public JLabel getIcon() {
		return icon;		
	}
	
	public JLabel getLabel() {
		return label;
	}
	
}


class GetCodePanel extends JPanel {

	LinkedList<BranchCodePanel> bPanels;
	
	public GetCodePanel(LinkedList<Edge> outEdges) {		
		bPanels = new LinkedList<BranchCodePanel>();  
		
		setLayout(new GridLayout(outEdges.size(), 0, 10, 10));
		
		for (int i = 0; i < outEdges.size(); i++) {
			Edge e = outEdges.get(i);
			BranchCodePanel codePanel = new BranchCodePanel(e, this);
			bPanels.add( codePanel );	        
		}
		repaintCodePanels();
	}

	private void repaintCodePanels() {
		removeAll();
		setLayout(new GridLayout(bPanels.size(), 0, 10, 10));

		// add to the main panel
		for (int i = 0; i < bPanels.size(); i++)
			add(bPanels.get(i));
		
		revalidate();
		repaint();		
	}

	public void promoteBranch(BranchCodePanel bcPanel) {
		if (bPanels.size() <= 1)
			return;
		int ind = bPanels.indexOf(bcPanel);
		
		if (ind == 0)
			return;
		
		bPanels.remove(bcPanel);
		bPanels.add(ind - 1, bcPanel);
		repaintCodePanels();
	}

	public void depressBranch(BranchCodePanel bcPanel) {
		if (bPanels.size() <= 1)
			return;
		int ind = bPanels.indexOf(bcPanel);
		
		if (ind == bPanels.size() - 1)	// the last element
			return;
		
		bPanels.remove(bcPanel);
		bPanels.add(ind + 1, bcPanel);
		repaintCodePanels();		
	}		
}


class BranchCodePanel extends JPanel implements ActionListener {
	Edge 		 edge;
	JTextArea 	 textArea;
	GetCodePanel parent;
	
	public BranchCodePanel(Edge e, GetCodePanel getCodePanel) {
		this.edge = e;
		parent = getCodePanel;
		setLayout( new BorderLayout(5, 5) ); 
		
		JLabel label = new JLabel( "            Code for the branch to node " + e.dstName() + ":" );
        
		JButton up 	 = new JButton("\u02C4");
		JButton down = new JButton("\u02C5");
		up  .setName("UP");
		down.setName("DOWN");
		up	.addActionListener(this);
		down.addActionListener(this);
		
		JPanel buttonPanel = new JPanel( new GridLayout(4, 1));
		buttonPanel.add( Box.createVerticalGlue() );
		buttonPanel.add(up);
		buttonPanel.add(down);
		buttonPanel.add( Box.createVerticalGlue() );
						
		// Text area for getting the code
		textArea = new JTextArea(e.guard(), 5, 20);
        textArea.setWrapStyleWord(true);
        JScrollPane areaScroll = new JScrollPane(textArea);

        add(buttonPanel, BorderLayout.LINE_START);
        add(label, BorderLayout.PAGE_START);
        add(areaScroll, BorderLayout.CENTER);				
	}
	
	public void commit() {
		edge.setGuard( textArea.getText() );				
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		JButton src = (JButton) ae.getSource();
		if (src.getName().equalsIgnoreCase("UP"))
			parent.promoteBranch(this);
		else if (src.getName().equalsIgnoreCase("DOWN"))
			parent.depressBranch(this);
	}
}