package main;
import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import javax.swing.border.Border;
import javax.swing.event.*;

import enums.ActionType;
import enums.SendSemantic;

import services.RunResult;
import taskeditor.EditorTab;
import util.Dialogs;
import util.GUIUtil;
import util.SchedPolicy;


import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.Random;

/**
 * This class implements some utility functions, which are mostly stateless
 * (static) and possibly are used by several other classes
 */
public class Util {
	
	private static final String 	RESOURCE_PATH = "/resources";
	private static final boolean 	SIM_DEBUG = true;
	private static final boolean 	DEBUG 	  = true;	// when true. exceptions are printed out

	public static ActionListener createUnavailableFeatureListener() {
		ActionListener AL = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				// JDialog.setDefaultLookAndFeelDecorated(true);
				JOptionPane.showMessageDialog(null,
						"This feature is not yet implemented.", "Information",
						JOptionPane.INFORMATION_MESSAGE);
			}
		};
		return AL;
	}

	public static ActionListener createExitActionListener() {
		ActionListener AL = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				// TO DO: kill the python process, if in run
				System.exit(0);
			}
		};
		return AL;
	}

	public static JFileChooser getFileChooser(String title, String default_path, String ext) {
		JDialog.setDefaultLookAndFeelDecorated(true);
		JFileChooser fileDialog = new JFileChooser();
		fileDialog.setDialogTitle(title);
		File defaultF = new File(default_path);
		fileDialog.setSelectedFile(defaultF); // No file is initially selected.
		FileFilter filter = new CustomizedFilter( ext );
		fileDialog.setFileFilter(filter);
		fileDialog.addChoosableFileFilter(filter);

		return fileDialog;
	}

	public static void showRunErrorMsg(String cmd) {
		String msg = "Couldn't run program \"" + cmd + "\". Please check if it is correctly installed.";
		Dialogs.showErrorMsg(msg);
	}
		
	

	public static boolean isWord(String action) {
		if (action.substring(0,1).matches("[a-zA-Z_]") == false)
			return false;
		return action.matches("\\w*") == true;
	}
	
	// returns null if it matches a word
	public static String checkWord(String action) {
		if (action.isEmpty())
			return "The word is empty.";
		if (action.substring(0,1).matches("[a-zA-Z_]") == false)
			return "A word can start with alphabetics (or underscore) only.";
		if (action.matches("\\w*") == false)
			return "A word should contain only word characters.";
		return null;
	}

	public static void show_config() {
		JTextField name_Field = new JTextField(5);
		name_Field.addAncestorListener(new RequestFocusListener()); // to become the focused component in the dialog
		JTextField WCET_Field = new JTextField(5);
		JTextField deadline_Field = new JTextField(5);
		name_Field.setFont(new Font("", 1, 30));
		WCET_Field.setFont(new Font("", 1, 30));
		deadline_Field.setFont(new Font("", 1, 12));

		JPanel myPanel = new JPanel(new GridLayout(2, 3));		
//		myPanel.add(new JLabel("Name:"));
//		myPanel.add(name_Field);
		myPanel.add(new JLabel("Python Path:"));
		myPanel.add(WCET_Field);
		myPanel.add(new JButton( "Browse" ));
		myPanel.add(new JLabel("drtlib Path:  "));
		myPanel.add(deadline_Field);
		myPanel.add(new JButton( "Browse" ));
		
		JPanel myPanel2 = new JPanel(new BorderLayout());
		myPanel2.add(myPanel, BorderLayout.PAGE_START);
		
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane .setPreferredSize(new Dimension(410, 200));
		tabbedPane.addTab ("Analysis Config.", myPanel2);
		
		
		String[] distros = new String[]{"EDF", "SP"};
		JComboBox comboBox = new JComboBox(distros);        
		/*comboBox.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent evt) {
                int index = getSelectedTaskIndex();
                Task task = manager.getTask(index);
                if(task != null)
                {
                    fillInTable(task);
                    manager.newTaskSelected(task);                    
                }
            }});*/

        JPanel panel = new JPanel( new GridBagLayout() );        
        JLabel label = new JLabel("Scheduling Policy:  ");
        label.setFont(new Font("", Font.PLAIN, 12));        
        label.setLabelFor(comboBox);
        panel.add(label);
        panel.add(comboBox);   
        panel.setBorder(GUIUtil.getGrayTitleBorder(""));
		tabbedPane.addTab ("Scheduling", panel);
		JPanel tbPanel = new JPanel(new GridLayout(1, 1));
		tbPanel.add(tabbedPane);
		
		int result = JOptionPane.showConfirmDialog(null, tbPanel, "Configuration", JOptionPane.OK_CANCEL_OPTION);
		
		/*JFrame frame = new JFrame("TabbedPaneDemo");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
         
        //Add content to the window.
        frame.add(content, BorderLayout.CENTER);
         
        //Display the window.
        frame.pack();
        frame.setVisible(true);*/
	}

	public static ActionListener createConfActionListener() {
		ActionListener AL = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				show_config();
			}
		};
		return AL;
	}

	/*
	 * Loads tools configuration from conf.txt file
	 * It is supposed that the first line of the file denotes the python path and
	 *    the second line denotes drtlib path
	 * Lines starting by # are regarded as comment and are ignored
	 */
	public static String[] load_conf() {
		String path = "conf.txt";
		LinkedList<String> result = new LinkedList<String>();  // default
		RandomAccessFile raf;
		try {
			raf = new RandomAccessFile(path, "r");
			
			String line = null;
			while ( (line = raf.readLine()) != null) {
				if (line.startsWith("#"))  // it is a comment
					continue;
				result.add(line);
			}		
			raf.close();
		} catch (FileNotFoundException e) {
			System.out.println("Warning: Could n't find config file for loading. For more information, see the log file.");
			Util.log(e);
		} catch (IOException e) {
			System.out.println("Warning: Could n't load config file appropriately. For more information, see the log file.");
			Util.log(e);
		}
		while (result.size() <= 2)		// add default value to prevent unexpected error
			result.add("");
		return result.toArray(new String[]{});
	}

	
	/*
	 * Saces the configuration entered by user
	 * This is called from ConfigActionListener
	 */
	public static void save_conf(String pythPath, String drtPath) {
		String path = "conf.txt";
		RandomAccessFile raf;
		try {
			File file = new File(path);
			if(file.exists())
				file.delete();
			
			raf = new RandomAccessFile(path, "rw");			
			raf.writeBytes("# Configuration file for MASI \r\n");
			raf.writeBytes("# The first line contains the python 2.7 installation path. The second line indicates the drt lib path\r\n");
			raf.writeBytes(pythPath + "\r\n");
			raf.writeBytes(drtPath + "\r\n");
			raf.close();
		} catch (FileNotFoundException e) {
			System.out.println("Warning: Could n't find config file");
			e.printStackTrace();
			Util.log(e);
		} catch (IOException e) {
			System.out.println("Warning: Could n't write to the config file appropriately");
			e.printStackTrace();
			Util.log(e);
		}
	}
	
	public static boolean exists(String filePath) {
		if (filePath == null)
			return false;
		
		try {
			return new File(filePath).exists();	
		} catch(Exception e) {
			Util.log(e);
			return false;
		}
	}
	
	public static boolean validate_exec_fiels1(String pythPath, String drtPath) {
		String err_msg = null;
		try{
			if (new File(pythPath).exists() == false)
				err_msg = "The path to the Python executable is not set correctly. \r\n Set the correct path using \"Settings\" tab";
			else if(new File(drtPath).exists() == false)
				err_msg = "The path to drtlib is not set correctly. \r\n Correct using Configuration button";
			
			if(err_msg == null)
				return true;
			else {
				Dialogs.showErrorMsg(err_msg);
				return false;	
			}	
		} catch(Exception e) {
			Util.log(e);
			Dialogs.showErrorMsg("An error occured during validation of executable paths.");
			return false;
		}
	}

	public static void decorateSplit(JSplitPane split, int dividerLoc) {
		split.setOneTouchExpandable(true);
		split.setContinuousLayout(true);
		split.setDividerLocation(dividerLoc);
	}

	public static void drawArrow(Graphics g, int x1, int y1, int x2, int y2, int s) {
	    g.drawLine(x1, y1, x2, y2);
	    int dx = x2-x1;
	    int dy = y2-y1;
	    int l = (int)Math.sqrt(Math.pow(dx,2)+Math.pow(dy,2));
	    if (l > 0) {
	    	int[] xp = new int[3];
	    	int[] yp = new int[3];
	    	xp[0] = x2;
	    	yp[0] = y2;
	    	double b = Math.asin((double)dx/(double)l)+0.25;
	    	double b2 = Math.asin((double)dx/(double)l)-0.25;
	    	xp[1] = (int)(x2-s*Math.sin(b));
	    	xp[2] = (int)(x2-s*Math.sin(b2));
	    	yp[1] = (int)(y2-s*Math.cos(b)*((dy<0)?-1:1));
	    	yp[2] = (int)(y2-s*Math.cos(b2)*((dy<0)?-1:1));
	    	g.fillPolygon(xp, yp, 3);
	    }
    }
	
	// p is the point clicked, (x,y) and (x2,y2) define the line, r is the margin (closeness criterion)
	public static boolean dotIsNearLine(Point p, int x, int y, int x2, int y2, int r) {
	    double minx = Math.min(x,x2);
	    double maxx = Math.max(x,x2);
	    double miny = Math.min(y,y2);
	    double maxy = Math.max(y,y2);
	    double dx = maxx-minx;
	    double dy = maxy-miny;
	    double px = p.x;
	    double py = p.y;

	    double ey = 0;

	    if((minx==x && miny==y) || (minx==x2 && miny==y2))
	      ey = dy-py+miny;
	    else
	      ey = dy+py-maxy;

	    double ex = dx-px+minx;
	    double q = Math.sqrt(Math.pow(ex,2)+Math.pow(ey,2));
	    double sina = ey/q;
	    double sing = dy/Math.sqrt(dx*dx+dy*dy);
	    double h = q*(sing*Math.cos(Math.asin(sina))-Math.cos(Math.asin(sing))*sina);

	    // Added to TIMES code
	    // check if the point is between the two points of the line segment (not out of it) 
	    if(px < minx - r || px > maxx + r || py < miny - r || py > maxy + r)
	    	return false;

	    return Math.abs(h)<=r;
	  }

	public static String tabTitle(JTabbedPane sourceTabbedPane) {
		int index = sourceTabbedPane.getSelectedIndex();
		String selectedTitle = sourceTabbedPane.getTitleAt(index);
        return selectedTitle;
	}

	public static String[] getDefaultJobProps(Task selectedTask) {
		String[] props = new String[3];	// will contain job name, job WCET, and job deadline
		int jobCnt = 0;
		while(selectedTask.getJobNo("J" + jobCnt) >= 0)	// means that a job with this name already exists
			jobCnt++;
		
		props[0] = "J" + jobCnt; // a default unique name
		props[1] = "1";			 // default WCET 
		props[2] = "10";		 // default deadline
		return props;
		/* the commented code was for getting job properties from the user, which canceled to be more userfrendly
		JTextField name_Field = new JTextField(5);
		name_Field.addAncestorListener(new RequestFocusListener());  // to become the focused component in the dialog    	        
        JTextField WCET_Field = new JTextField(5);
        JTextField deadline_Field = new JTextField(5);
        name_Field.setFont(new Font("", 1, 12));
        WCET_Field.setFont(new Font("", 1, 12));
        deadline_Field.setFont(new Font("", 1, 12));
        
        JPanel myPanel = new JPanel(new GridLayout(3, 2));
        myPanel.add(new JLabel("Name:"));
        myPanel.add(name_Field);
        myPanel.add(new JLabel("WCET:"));
        myPanel.add(WCET_Field);
        myPanel.add(new JLabel("Deadline:  "));
        myPanel.add(deadline_Field);
        
        int result = JOptionPane.showConfirmDialog(null, myPanel, "Please Enter Job Parameters", JOptionPane.OK_CANCEL_OPTION);
        if (result == JOptionPane.OK_OPTION) {
        	String name = name_Field.getText();
        	String WCET = WCET_Field.getText();
        	String DdLn = deadline_Field.getText();
        	// isNumeric(WCET);   TO DO: implement for test if the value entered b the user is a numeric
        	// isNumeric(DdLn);                	
        	String valid = manager.jobPropValidity(name, WCET, DdLn, -1);
        	if(valid.compareToIgnoreCase("valid") != 0)
        	{
        		Util.showErrorMsg(valid);
        		mousePressed(e);   // recursive call
        		return;
        	}
        }
        return null;
        */		
	}
	
	public static ImageIcon getIcon(String image) {
		ImageIcon icon;
		URL url = Util.class.getResource(RESOURCE_PATH + image);
		if(url == null)
		{
			System.out.println("Wanring: the resource file " + image + " not found");
			icon = new ImageIcon();		    
		}
		else
			icon = new ImageIcon(url);
	    return icon;
	}

	public static Cursor getCustomCursor(String image) 
	{
		Cursor cursor;
		URL url = Util.class.getResource(RESOURCE_PATH + image);
		if(url == null)
		{
			System.out.println("Wanring: the resource file " + image + " not found");
			cursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);	// a default cursor
		}
		else {		
			Toolkit toolkit = Toolkit.getDefaultToolkit();
	        Image img = toolkit.createImage(url);
	    	cursor = toolkit.createCustomCursor(img, new Point(7,7), "");
		}
		return cursor;
	}

	public static JButton createCodeBrowse(final JTextArea textArea) {
		class BrowseListener implements ActionListener
		{
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileDialog = Util.getFileChooser("Select", "", ".*");
	            int option = fileDialog.showDialog(null, "OK");
	            if (option != JFileChooser.APPROVE_OPTION)
	                return;      // User canceled or clicked the dialogue close box.
	            String filePath = fileDialog.getSelectedFile().getPath();
	            textArea.setText( Util.fileContent( filePath ) );
			}		
		}
		JButton browse = new JButton("Browse");
        browse.setFont(new Font("", Font.PLAIN, 12));
        browse.addActionListener( new BrowseListener() );
        
		return browse;
	}
	
	public static Comparator getIntComparator()	{
		class IntComparator implements Comparator {

			public int compare(Object arg0, Object arg1) {
				int res = 0;
				try{
					Integer i0 = Integer.parseInt((String)(arg0));
					Integer i1 = Integer.parseInt((String)(arg1));
					res = i0.compareTo(i1);
				} catch(Exception e) {
					Util.log(e);
					return 0;
				}
				return res;
			}			
		}
		return new IntComparator();
	}
	
	public static Comparator getStrIntComparator()
	{
		class IntComparator implements Comparator {

			public int compare(Object arg0, Object arg1) {
				int res = 0;
				try{
					String s0 = (String)(arg0);
					String s1 = (String)(arg1);
					
					Integer i0 = extractInt(s0);
					Integer i1 = extractInt(s1);
					res = i0.compareTo(i1);
				} catch(Exception e) {
					Util.log(e);
					return 0;
				}
				return res;
			}
			


		    int extractInt(String s) {
		        String num = s.replaceAll("\\D", "0");
		        // return 0 if no digits found
		        return num.isEmpty() ? 0 : Integer.parseInt(num);
		    }
		}
		return new IntComparator();
	}

	public static boolean contains(int[] nums, int r) {
		if(nums == null)
			return false;
		
		for (int n : nums)
			if(r == n)
				return true;
		return false;
	}

	
	public static ActionListener createHelpActListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				JDialog.setDefaultLookAndFeelDecorated(true);
				JOptionPane
						.showMessageDialog(
								null,
								"The MASI tool for modeling, analysis, simulation, and implementation of CPSs.\r\n Developed at Uppsala University, 2015-2017.",
								"About the application",
								JOptionPane.INFORMATION_MESSAGE);
			}
		};
	}

		
	/*
	 * Called by Manager when plotting request curves in RFDBF viewer
	 */
	public static String[] extractNames(Task[] tasks) {
		if(tasks == null)
			return new String[0];
		
		int tNo = tasks.length;
		String []names = new String[tNo];
		for(int i = 0; i < tNo; i++)
			names[i] = tasks[i].getName();
		return names;
	}
	
	/*
	 * Called by TaskSet syntax checker
	 */
	public static String[] getRedundants(String[] names) {
		LinkedList<String> observed   = new LinkedList<String>();
		LinkedList<String> redundants = new LinkedList<String>();
		 for (String n : names) {
			 if(contains(observed, n))
				 if(! contains(redundants, n))
					 redundants.add(n);
			 observed.add(n);
		 }
		 return toStrArr(redundants);
	}

	private static boolean contains(LinkedList<String> list, String s) {
		return contains(toStrArr(list), s);
	}

	private static String[] toStrArr(LinkedList<String> list) {
		if(list == null)
			return null;
		return list.toArray(new String[]{});
	}

	public static double round(double load, int l) {
		return Math.round(load * Math.pow(10, l)) / Math.pow(10, l);
	}

	public static double []round(double []nums, int l) {
		double [] res = new double [nums.length];
		for(int i = 0; i < nums.length; i++)
			res[i] = round(nums[i], l);
		return res;
	}

	public static int getWidth(JComponent c) {
		return c.getPreferredSize().width;
	}

	public static void showRunMsg(RunResult res, NotesPanel notesPane) {
		if (res == null)
			return;
		
		System.out.println(res.message());
		notesPane.println(res.message());
		if (res.succeeded() == false) {
			Dialogs.showWarnMsg("Analysis terminated without appropriate results. See the log file for details.");
			Util.log(res.message());
		}
		else
			Dialogs.showInfoMsg(res.message() + "\r\nAnalysis time = " + res.getRun_time() + " sec.");		
	}

	
	public static String fileContent(String fileName)  {
        FileReader in;
		try {
			in = new FileReader(fileName);
			StringBuilder contents = new StringBuilder();
	        char[] buffer = new char[4096];
	        int read = 0;
	        do {
	            contents.append(buffer, 0, read);
	            read = in.read(buffer);
	        } while (read >= 0);
	        in.close();
	        return contents.toString();
		} catch (FileNotFoundException e) {
			System.out.println("File not found: " + fileName);
			Util.log(e);
			return "";
		} catch (IOException e) {
			System.out.println("Error while reading from file " + fileName);
			Util.log(e);
			return "";
			//e.printStackTrace();
		}
        
    }
	
	public static String showFileBrowser(JFileChooser fileDialog) {
		int option = fileDialog.showSaveDialog(null);
		if (option != JFileChooser.APPROVE_OPTION)
			return null; // User canceled or clicked the dialogue close

		String path = "";
		File selectedFile = fileDialog.getSelectedFile();
		if (selectedFile.exists()) { // the selected path already exists
			JPanel myPanel = new JPanel(new GridLayout(3, 1));
			myPanel.add(new JLabel("The selected file already exists."));
			myPanel.add(new JLabel("Do you you want to relace it?\r\n"));
			myPanel.add(new JLabel(""));
			int result = JOptionPane.showConfirmDialog(null, myPanel,
					"Save File", JOptionPane.YES_NO_OPTION);
			if (result == JOptionPane.OK_OPTION)
				path = selectedFile.getAbsolutePath();
			else 
				return showFileBrowser(fileDialog); // recursively call yourself
		} else
			path = selectedFile.getAbsolutePath();
		
		return path;
	}
	
	public static String extractPath(String str) {
		if(str == null)
			return "./";
		String res = new String(str);
		int last = Math.max(res.lastIndexOf("\\"), res
				.lastIndexOf("/"));
		res = res.substring(0, last + 1); 
		return res;	
	}
	
	public static String extractFileName(String path) {
		if(path == null)
			return "";
		String res = new String(path);
		int last = Math.max(res.lastIndexOf("\\"), res
				.lastIndexOf("/"));
		res = res.substring(last + 1);
		last = res.lastIndexOf(".");	// remove the suffix
		if (last >= 0)
			res = res.substring(0, last); 
		return res;
	}
	
	public static String prepareRunStr(String drt_path, String str) {		
		String res = extractPath(drt_path) + str;
		return res;
	}

	// called by the TaskSet to write the code to a file
	public static void write2file(String path, String str) {
		BufferedWriter bufferedWriter = null;
        try {
            File file = new File(path);
            if (!file.exists()) {
                file.createNewFile();
            }
            FileWriter writer = new FileWriter(file);
            bufferedWriter = new BufferedWriter(writer);
            bufferedWriter.write(str);
        } catch (IOException e) {
            System.out.println("Problem in writing the string in file: " + path);
            log(e);
        } finally {
            try {
                if(bufferedWriter != null) 
                	bufferedWriter.close();
            } catch(Exception ex){  
            	log(ex);
            }
        }		
	}

	// returns the current date in a formatted way
	// used in code generation
	public static String Date() {

		DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
		Date date = new Date();
		return dateFormat.format(date); // 2016/03/17 11:10:18		
	}

	public static ActionListener createGuideActListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				String path = "UserManual.pdf";
				if (Desktop.isDesktopSupported()) {
				    try {
				    	File myFile = new File(path);
				        Desktop.getDesktop().open(myFile);
				    } catch (Exception ex) {
				    	Dialogs.showErrorMsg("The file: " + path + " doesn't exist.");
				        // no application registered for PDFs
				    }
				}
				else
				{
					System.out.println("Desktop not suppoerted");
					Dialogs.showErrorMsg("Desktop component not suppoerted by the system to open the file " + path);
				}
			}
		};
	}
	

	public static int showOpenErrDialog(JFrame parent, Exception ex) {		
		
		JPanel myPanel = new JPanel(new GridLayout(3, 1, 10, 0));
		myPanel.add(new JLabel());
		myPanel.add(new JLabel(
				"      Could not open the specified file successfully.  Do you want to try again?          "));
		
		MyErrDialog dlg = new MyErrDialog(parent, "Error Message", myPanel, ex);
		return dlg.res;		
	}

	public static String getStackTrace(Exception e) {
		StackTraceElement[] elems = e.getStackTrace();
	    String trace = e.getClass().toString() + "\r\n";
	    for(StackTraceElement e1 : elems)
	    	trace += e1 + "\r\n";
		  
		return trace;
	}

	public static void log(String str) {
		try {
    		File file = new File("log.txt");    		
    		file.createNewFile(); // creates a new file only if a file with this name does not exist 
    		
    		//true = append file
    		FileWriter fileWritter = new FileWriter(file.getName(), true);
	        BufferedWriter bufferWritter = new BufferedWriter(fileWritter);
	        bufferWritter.write(Date() + ": " + str + "\r\n");
	        bufferWritter.close();    	    
    	} catch(IOException e){
    		e.printStackTrace();
    	}		
	}

	public static int maxD(Task[] tasks) {
		if (tasks == null)
			return 0;
		if(tasks.length == 0)
			return 0;
		
		int maxD = 0;
		for(Task t : tasks)
			maxD = Math.max(maxD, t.maxD());
		return maxD;
	}

	public static boolean isRClick(MouseEvent e) {
		if( e.getClickCount() == 1 && GUIUtil.rightClick(e) )
			return true;
		return false;
	}
	
	public static ActionListener createRespTimeListner(Manager manager) {
		return new RespTimeListner(manager);
	}

	public static ActionListener createAnalSyntaxListner(Manager manager) {
		return new AnalysisSyntaxActionListner(manager);
	}

	
	public static String pol2str(SchedPolicy policy) {
		 String str = (policy == SchedPolicy.EDF) ? "EDF" : 
			   (policy == SchedPolicy.SP ? "SP" : null);	
		 if (str == null) {
			 String msg = "Error: Unknown policy: " + policy;
			 System.out.println(msg);
			 Dialogs.showErrorMsg(msg);
			 str = "SP"; // default is SP
		 }
		 return str;
	}

	/*
	 * input: the raw output of the schedulability analysis 
	 * (output of python drtlib). Used by runAnalysis()
	 */
	public static String[] derive_names(String raw_str) {
		String task_sequence = raw_str.substring(raw_str.indexOf("[") + 1,
				raw_str.lastIndexOf("]"));
		LinkedList<String> names = new LinkedList<String>();
		String[] tokens = task_sequence.split("'");
		int id = 1; // set to 1 since the first token is empty
		while (id < tokens.length) {
			names.add(tokens[id]);
			id += 2;
		}

		return names.toArray(new String[0]);
	}

	
	public static void initLog() {
		File file = new File("log.txt");
		if(file.exists())
			file.delete();
		
		log("---------- Program Started -----------");	// a line showing new run 		
	}

	public static void log(Exception e) {
		log(Util.getStackTrace(e));		
	}

	public static String getUniqueName(String name, String[] names) {
		if(! contains(names, name) )
			return name;
			
		int ind = 1;
		String uName = "";
		while(true)
		{
			uName = name + "_" + ind;
			if(! contains(names, uName) )
				break;
			ind = ind + 1;
		}
		return uName;
	}

	private static boolean contains(String[] names, String uName) {
		if(names == null)
			return false;
		
		for (String s : names)
			if(s.equals(uName))
				return true;
		
		return false;
	}

	public static int discritize(int val, int module) {
		return ((int)(val / module)) * module;
	}

	public static ActionListener createExitActListener(final Manager manager) {
		class ExitListener implements ActionListener {
			public void actionPerformed(ActionEvent evt) {
				// img2(manager);
				manager.terminate();
			}
		}
		return new ExitListener();
	}
	
	private static void img2(Manager mgr) {
		Dimension size = mgr.frame().getSize();
        BufferedImage image = new BufferedImage(
                    size.width, size.height 
                              , BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = image.createGraphics();
        mgr.frame().paint(g2);
		
        ImageWriter jpgWriter = ImageIO.getImageWritersByFormatName("jpg").next();
        ImageWriteParam jpgWriteParam = jpgWriter.getDefaultWriteParam();
        jpgWriteParam.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
        jpgWriteParam.setCompressionQuality(1f);
        
        File file = new File("snapshot2.jpg");
        
		try {
			ImageOutputStream ios = ImageIO.createImageOutputStream(file);
	        jpgWriter.setOutput(ios);
	         //better to use ImageOutputStream
			jpgWriter.write(null, new IIOImage(image, null, null), jpgWriteParam);
	        System.out.println("Panel saved as Image.");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private static void img1(Manager mgr) {
		Dimension size = mgr.frame().getSize();
        BufferedImage image = new BufferedImage(
                    size.width, size.height 
                              , BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = image.createGraphics();
        mgr.frame().paint(g2);
        try {
            ImageIO.write(image, "gif", new File("snapshot.gif"));
            System.out.println("Panel saved as Image.");
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
	}

	public static boolean isInteger(String s){
        if(s.length() < 1)
            return false;
        for(int i = 0; i < s.length(); i++){
            char c = s.charAt(i);
            if(Character.isDigit(c)){
                
            }else
                return false;
        }
        return true;
    }

	public static ActionListener createStopActListener(final Manager mgr) {
		class StopListener implements ActionListener {
			public void actionPerformed(ActionEvent evt) {
				mgr.stop();
			}
		}
		return new StopListener();
	}

	// checks whether the given string is a non-negative integer
	public static boolean checkIntVal(String val, String field) {
		try {
			int i1 = Integer.parseInt(val);
			if(i1 < 0)
			{
				Dialogs.showErrorMsg(field + " cannot be negative.");
				return false;
			}
			return true;
		} catch(NumberFormatException e) {
			Dialogs.showErrorMsg(field + " must be an integer number.");
			return false;
		}
	}

	public static String[] extractJobsName(Task task) {
		int n = task.jobNo();
		if (n == 0)
			return new String[0];
		
		String []names = new String[n];
		for(int i = 0; i < n; i++)
			names[i] = task.getJob(i).getName();
		return names;
	}
	
	public static void showSyntaxErrWarningDialog(JFrame f, LinkedList<String> errors) {
		String title = "Syntax Errors";
		String details = "";
		for(String s : errors)
			details += s + "\r\n";

		final String Space = "      "; 
		final String Space4 = Space + Space + Space + Space;
		String msg = "The model contains syntax errors, it cannot be simulated.";		
		msg = Space + msg + Space4 + Space4 + Space;
		Dialogs.showSyntaxErrDialog(title, msg, details);
	}

	
	// called by JobType
	public static int parseInt(String s) {
		int val = 1;	// default
		try {
			val = Integer.parseInt(s);
		} catch(NumberFormatException e) {
			System.out.println("Warning: bad formatted number.");
		}
		return val;
	}

	/*
	 *  To resize the label with new data length
	 *  Called by JobType
	 */	
	public static void adjustLabelSize(JLabel jl, int x, int y) {
		Rectangle rec = jl.getBounds();
        rec.width = jl.getPreferredSize().width;
        rec.height = jl.getPreferredSize().height;
        jl.setBounds(rec);
        
        int x1 = x - rec.width/2;
        int y1 = y - rec.height/2;
        jl.setLocation(x1, y1);
	}

	public static String getPathFromUser(String title, String addr, String ext) {
		JFileChooser fileDialog = getFileChooser(title, addr, ext);				
		String path = showFileBrowser(fileDialog);		
		return path;
	}

	public static String readFile(String path) {
		String content = "";
		  try {
			  File f = new File(path);
	          FileInputStream fis = new FileInputStream(f);
	          byte[] bf = new byte[(int)f.length()];
	          fis.read(bf);
	          fis.close();
	          content = new String(bf, "UTF-8");
	      } catch (FileNotFoundException e) {
	          e.printStackTrace();
	      } catch (IOException e) {
	          e.printStackTrace();
	      }
		  return content;
	}

	public static void sleep(long d) {
		try {
		    Thread.sleep(d);
		} catch(InterruptedException ex) {
		    Thread.currentThread().interrupt();
		}
	}

	public static void simDebug(String str) {
		if (SIM_DEBUG)
			System.out.println(str);
	}

	public static void debug(String str) {
		if (DEBUG)
			System.out.println(str);
	}

	public static int str2int(String s) {
		int val = 0;
		try {
			val = Integer.parseInt(s);
		} catch(NumberFormatException e) {
			Util.log(e);
			val = 0; 
		}
		return val;
	}
	
	public static double str2num(String str) {
		double result = 0;
		try {
			result = Double.parseDouble(str);
		} catch(NumberFormatException e) {
			Util.log(e);
			result = 0; 
		}
		return result;
	}

	public static Color[] randColors(int size) {
		Color[] res = new Color[size];
		final Random r = new Random(100);        
		for(int i = 0; i < size; i++)		
			res[i] = new Color(r.nextInt(200), r.nextInt(200), r.nextInt(200), 250);
			
		return res;
	}

	public static void error(Exception e) {
		if (e == null)
			return;
		Util.log(e);
		if (DEBUG)
			e.printStackTrace();		
	}
	
	public static void error(String s) {
		Util.log(s);
		if (DEBUG) 
			System.out.println(s);		
	}

	public static void warning(String msg) {
		debug(msg);
	}

	public static String[] firstCol(String[][] data) {
		String[] res = new String[data.length];
		for (int i = 0; i < res.length; i++)
			res[i] = data[i][0];
		return res;
	}
}


class MyErrDialog extends JDialog implements ActionListener {
	int res = -1;
	boolean detailsShown = false;
	Exception exception;
	
  public MyErrDialog(JFrame parent, String title, JPanel myPanel, Exception ex) {
	  super(parent, title, true);
	  exception = ex;
	  if (parent != null) {
		  Dimension parentSize = parent.getSize(); 
		  Point p = parent.getLocation(); 
		  setLocation(p.x + parentSize.width / 4, p.y + parentSize.height / 4);
	  }
	  JPanel fixedP = new JPanel(new BorderLayout());
	  fixedP.add(myPanel, BorderLayout.NORTH);
    
    JPanel buttonPane = new JPanel();
    JButton button1 = new JButton("Yes"); 
    JButton button2 = new JButton("No");
    JButton button3 = new JButton("Details >>");
    buttonPane.add(button1); 
    buttonPane.add(button2);
    buttonPane.add(button3);
    button1.addActionListener(this);
    button2.addActionListener(this);
    button3.addActionListener(this);    
    fixedP.add(buttonPane, BorderLayout.CENTER);
    
    getContentPane().add(fixedP, BorderLayout.NORTH);
    //getContentPane().add(new JLabel("   "), BorderLayout.CENTER);
    
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    // setMaximumSize(new Dimension(50, 1000));
    pack(); 
    setVisible(true);
  }
  
  public void actionPerformed(ActionEvent e) {
	  String src = ((JButton)e.getSource()).getText();
	  if(src.equalsIgnoreCase("Yes"))
	  {
		  res = JOptionPane.YES_OPTION;
		  setVisible(false);
		  dispose();
	  }
	  else if(src.equalsIgnoreCase("No"))
	  {
		  res = JOptionPane.NO_OPTION;
		  setVisible(false);
		  dispose();
	  }
	  else // do nothing
	  {
		  Dimension sz = getPreferredSize();		  
		  if(detailsShown == false)
		  {			  
			  String trace = Util.getStackTrace(exception);
			  if (exception.getMessage() != null)
				  trace = exception.getMessage() + "\r\n-----------------\r\n" + trace;
			  
			  JTextArea text = new JTextArea(trace);			  	
			  text.setFont(new Font(text.getFont().getName(), Font.PLAIN, 14));
			  JScrollPane sccr =  new JScrollPane(text);
			  sz.height = sz.height + 200;
			  setPreferredSize(sz);
			  getContentPane().add(sccr, BorderLayout.CENTER);
			  ((JButton)e.getSource()).setText("Details <<");
		  }
		  else {
			  getContentPane().remove(1);	// removing the added textArea
			  sz.height = sz.height - 200;
			  setPreferredSize(sz);			  
			  ((JButton)e.getSource()).setText("Details >>");
		  }
		  detailsShown = !detailsShown;
		  pack();	
		  repaint();
		  revalidate();
	  }
  }
}


class CustomizedFilter extends FileFilter {
	String extension = ".*";
	public CustomizedFilter(String ext)
	{
		this.extension = ext;
	}
	
	public boolean accept(File file) {
		if (file.isDirectory())
			return true;
		String filename = file.getName();
		return filename.endsWith( extension ) || extension.equalsIgnoreCase(".*");
	}

	public String getDescription() {
		return "*" + extension; // .xml";
	}
}

/**
 * Note: this class and respective comments has been gained from
 * https://tips4java.wordpress.com/2010/03/14/dialog-focus/ Convenience class to
 * request focus on a component.
 * 
 * When the component is added to a realized Window then component will request
 * focus immediately, since the ancestorAdded event is fired immediately.
 * 
 * When the component is added to a non realized Window, then the focus request
 * will be made once the window is realized, since the ancestorAdded event will
 * not be fired until then.
 * 
 * Using the default constructor will cause the listener to be removed from the
 * component once the AncestorEvent is generated. A second constructor allows
 * you to specify a boolean value of false to prevent the AncestorListener from
 * being removed when the event is generated. This will allow you to reuse the
 * listener each time the event is generated.
 */
class RequestFocusListener implements AncestorListener {
	private boolean removeListener;

	/*
	 * Convenience constructor. The listener is only used once and then it is
	 * removed from the component.
	 */
	public RequestFocusListener() {
		this(true);
	}

	/*
	 * Constructor that controls whether this listen can be used once or
	 * multiple times.
	 * 
	 * @param removeListener when true this listener is only invoked once
	 * otherwise it can be invoked multiple times.
	 */
	public RequestFocusListener(boolean removeListener) {
		this.removeListener = removeListener;
	}

	@Override
	public void ancestorAdded(AncestorEvent e) {
		JComponent component = e.getComponent();
		component.requestFocusInWindow();

		if (removeListener)
			component.removeAncestorListener(this);
	}

	@Override
	public void ancestorMoved(AncestorEvent e) {
	}

	@Override
	public void ancestorRemoved(AncestorEvent e) {
	}

}

/*
 * This is used for running the response time analysis engine (drtlib in python)
 */
class RespTimeListner implements ActionListener {

	Manager mgr;
	
	public RespTimeListner(Manager manager) {
		mgr = manager;
	}

	public void actionPerformed(ActionEvent evt) {
		if (mgr.hasActiveTaskSet() == false) {
			Dialogs.showWarnMsg("Task set is empty.");			
			return;
		}

		if(taskIsEmpty()) {
			Dialogs.showWarnMsg("The selected task is empty.");			
			return;
		}
		
		LinkedList<String> errors = mgr.analysisSyntaxErrors();
		if (errors.size() > 0) {
			Dialogs.showDefaultSyntaxErrDialog(errors);
			return;
		}
			
		if (mgr.tabsActivated()) {
			mgr.showProgressBar();
			try {
				new Thread(new Runnable() {
					public void run() {
						RunResult[] res = mgr.runRespTimeAnalysis();
						mgr.hideProgressBar();
						if (res == null) {
							errMsg();
						} else if(res.length == 0) {
							mgr.printNote("Result is empty.");
						} else {
							String[] names = mgr.getSelectedTasksNames();
							Object[][] data = extractData(res, names);
							showAndSaveResult(data);
						}
					}
				}).start();
			} catch (Exception e) {
				mgr.hideProgressBar();
				Util.log(e);
			}
		}
	}
	
	private boolean taskIsEmpty() {
		Task t = mgr.getSelectedTask();
		if (t.jobNo() == 0)
			return true;
		return false;
	}

	private void err(Exception e, RunResult res) {
		mgr.hideProgressBar();
		
		if (res != null) {
			for (int i = 0; i < res.linesNo(); i++) {
				System.out.println(res.getLine(i));
				Util.log(res.getLine(i));
			}
			if (res.linesNo() == 0) {
				String s = "No line read from the analysis engine";
				System.out.println(s);
				Util.log(s);
			}
		}
		
		e.printStackTrace();		
		Util.log(e);
		errMsg();
	}
	
	private void errMsg() {
		Dialogs.showErrorMsg("Process terminated not properly. " +
				"\r\nSee the log file for more details.");		
	}		

	private String[] derive_names(String raw_str) throws Exception {
		String[] tokens = {};
		try {
			String sequence = raw_str.substring(raw_str.indexOf("[") + 1,
					raw_str.lastIndexOf("]"));
			sequence = sequence.replace(" ", "");
			sequence = sequence.replace("'", "");
			tokens = sequence.split(",");
		} catch(Exception e) {
			parseErr(e, raw_str);
			throw e;
		}
		return tokens;
	}

	private String[] derive_times(String raw_str) throws Exception {
		String[] tokens = {};	
		try {			
			String sequence = raw_str.substring(raw_str.indexOf("[") + 1,
				raw_str.lastIndexOf("]"));
			sequence = sequence.replace(" ", "");
			tokens = sequence.split(",");
			for(int i = 0; i < tokens.length; i++)
			{
				int num = Integer.parseInt(tokens[i]);
				if(num == -1)	// response time has been larger than the maximum value
					tokens[i] = "> Deadline"; // + maxD; 
			}
		} catch(Exception e) {
			parseErr(e, raw_str);
			throw e;
		}
		return tokens;
	}
	
	private static void parseErr(Exception e, String raw_str) {
		Util.log("Problem in parsing the result got from the python code:");
		Util.log(raw_str);
		Util.log(e);		
	}

	private static JPanel createTablePanel(Object [][] data) {
		JTable table = new JTable(){  
		    public boolean isCellEditable(int row,int column){  
		        return false;   		          
		      }  
		    }; 
		    
		    table.setPreferredScrollableViewportSize(new Dimension(70, 70));
		    table.setFillsViewportHeight(true);
		    table.setAutoCreateRowSorter(true);

		    table.setModel(createRTTableModel(data));        
		    
		    TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(table.getModel());;
			sorter.setComparator(0, Util.getStrIntComparator());
			sorter.setComparator(1, Util.getStrIntComparator());
			sorter.setComparator(2, Util.getIntComparator());
			table.setRowSorter(sorter);
					
			table.getColumnModel().getColumn(0).setPreferredWidth(120);
			//table.getColumnModel().getColumn(1).setPreferredWidth(55);
			GUIUtil.alignHeader(table);        
			GUIUtil.alignCells(table, 3);

			JScrollPane table_scr = new JScrollPane(table);
			table_scr.setBorder(GUIUtil.getBlackTitleBorder(""));
	        // table_scr.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
	        
	        table_scr.setPreferredSize(new Dimension(300, 200));
	        
	        JPanel tablePanel = new JPanel(new BorderLayout());			
	        tablePanel.add(table.getTableHeader(), BorderLayout.PAGE_START);
	        tablePanel.add(table_scr, BorderLayout.CENTER);
	        
	        //tablePanel.add(table, BorderLayout.CENTER);
			//tablePanel.add(table_scr/*table.getTableHeader()*/, BorderLayout.NORTH);
			
			return tablePanel;	
	}
	
	private static TableModel createRTTableModel(Object[][] data) {
		class RTTableModel extends AbstractTableModel {
	        private final String[] columnNames = {"Task", "Job", "Resp. Time"};
	        private Object[][] data; 	
	        		
	        public RTTableModel(Object[][] myData) {
	        	data = myData;
	        }
	        
	        public int getColumnCount() {
	            return columnNames.length;
	        }

	        public int getRowCount() {
	            return data.length;
	        }

	        public String getColumnName(int col) {
	            return columnNames[col];
	        }

	        public Object getValueAt(int row, int col) {
	        	return data[row][col];
	        }

	        public Class getColumnClass(int c) {
	            return getValueAt(0, c).getClass();
	        }

	        /*
	         * The first column (job number) is not editable 
	         */
	        public boolean isCellEditable(int row, int col) {             
	            return false;            
	        }

	        public void setValueAt(Object value, int row, int col) {        	
	        }
	    }
		return new RTTableModel(data);
	}

	protected void showAndSaveResult(Object[][] data) // RunResult[] results, String[] selectedTasks) 
	{
		JPanel table = createTablePanel(data);		
		JLabel title = new JLabel("Response Times for the Selected Task(s):");		
		JPanel panel = new JPanel(new BorderLayout(0, 10));
		
		panel.setBorder(GUIUtil.getEmptyBorder(10));
		panel.add(title, BorderLayout.PAGE_START);
		panel.add(table, BorderLayout.CENTER);
		
		String[] options = {"   Save   ", "OK"};
		int choice = Dialogs.showOptionMsg(panel, "Response Time", options);
		if(choice == 0)	// save is selected
		{
			JFileChooser fileDialog = Util.getFileChooser("Save",
										"Resp_Times.csv", ".csv");				
			String path = Util.showFileBrowser(fileDialog);
			if(path != null) // user did not canceled 
				saveRespTimes(path, data);
		}		
	}

	private void saveRespTimes(String path, Object[][] data) {
		String NL = "\r\n";
		String str = "Task name, Job Name, Resp. Time" + NL;
		for(int i = 0; i < data.length; i++)
		{
			int n = data[i].length;
			for(int j = 0; j < n - 1; j++)
				str += data[i][j] + ",";
			if(n > 0)
				str += data[i][n - 1];
			str += NL;
		}
		Util.write2file(path, str);
	}

	private Object[][] extractData(RunResult[] results, String[] tasks) {
		LinkedList<String[]>rows = new LinkedList<String[]>(); 		
		for(int i = 0; i < results.length; i++)
		{
			RunResult res = results[i];
			String task = tasks[i];
			if (res.succeeded() == false) {
				mgr.printNote(res.message());
				Util.log(res.message());
				errMsg();
				continue;
			} 							
			String job_names = res.getLine(0);
			String job_respt = res.getLine(1);			
			try {									
				String[] names = derive_names(job_names);
				String[] times = derive_times(job_respt);
				int rowsNo = Math.min(names.length, times.length);				
				for(int k = 0; k < rowsNo; k++)
					rows.add(new String[]{task, names[k], times[k]});				
			} 
			catch(Exception e) {
				err(e, res);
			}
		}
		Object[][] data = new Object[rows.size()][3];  
		for(int i = 0; i < rows.size(); i++)
			data[i] = rows.get(i);
		
		return data;
	}
}



/*
 * This is used for checking the syntax of the modeled task set 
 */
class AnalysisSyntaxActionListner implements ActionListener {

	Manager mgr;
	final String Space = "      ";
	final String Space4 = Space + Space + Space + Space;
	
	
	public AnalysisSyntaxActionListner(Manager m) {
		this.mgr = m;
	}

	public void actionPerformed(ActionEvent evt) {
		LinkedList<String> err = mgr.analysisSyntaxErrors();
		boolean sound = true;
		if (err != null)
			if(err.size() > 0)
				sound = false;

		if (sound)
			Dialogs.showInfoMsg("The model is syntactically sound."); 
		else
			showErrDialog(err);
	}

	private void showErrDialog(LinkedList<String> err) {
		String details = "";
		for (String s : err)
			details += s + "\r\n";

		String title = "Syntax Errors";
		String msg;
		int n = err.size();
		if(n == 1)
			msg = "There is 1 syntax error in the model.";
		else
			msg = "There are " + n + " syntax errors in the model.";		
		msg = Space + msg + Space4 + Space4 + Space;
		
		Dialogs.showSyntaxErrDialog(title, msg, details);		
	}
}