package main.storage;

import java.awt.Point;
import java.util.Vector;

import main.JobType;
import main.Util;

import org.w3c.dom.Node;

import enums.SendSemantic;

public class StorageUtil {

	// removes the ending file name and returns only the path  
	static String extractPath(String fullPath) {
		int last = Math.max(fullPath.lastIndexOf("\\"), fullPath
				.lastIndexOf("/"));
		String p = fullPath.substring(0, last + 1);
		return p;
	}	

	public static Vector<Point> extractNails(Node node) {
		Vector<Point> nails = new Vector<Point>();
		String xs = XMLUtil.attrVal(node, "xs");
		String ys = XMLUtil.attrVal(node, "ys");
		String[] xarr = xs.split(",");
		String[] yarr = ys.split(",");
		if (xarr.length != yarr.length)
			return null;
		for (int i = 0; i < xarr.length; i++) {
			int x = Util.str2int(xarr[i]);
			int y = Util.str2int(yarr[i]);
			Point p = new Point(x, y);
			nails.add(p);
		}
		return nails;
	}

	public static SendSemantic getSendSem(Node node, SendSemantic deflt) {
		final String BLOCK = String.valueOf(SendSemantic.Blocking);
		final String NONBLOCK = String.valueOf(SendSemantic.Non_Blocking);

		String s = XMLUtil.attrVal(node, "semantics");
		if (s == null)
			return deflt;
		if (!s.isEmpty()) {
			if (s.equalsIgnoreCase(BLOCK))
				return SendSemantic.Blocking;
			else if (s.equalsIgnoreCase(NONBLOCK))
				return SendSemantic.Non_Blocking;
			else
				System.out.println("Warning: Unknwon Send Semantics: " + s);
		}
		return deflt;
	}
	
	public static void setJobTextLabelPosition(Node jNode, JobType job) {
		try {
			String sx = XMLUtil.attrVal(jNode, "labelx");
			String sy = XMLUtil.attrVal(jNode, "labely");
			int x, y;
			if (sx.length() > 0 && sy.length() > 0) {
				job.setLabelXY((int)Util.str2num(sx), (int)Util.str2num(sy));
			}			
		} catch (Exception e) {
			Util.log(e);
		}		
	}
	
	// if position not defined, return null
	public static Point getEdgeLabelP(Node attrNode) {
		Point p = null;
		String x = XMLUtil.attrVal(attrNode, "x"); 
		String y = XMLUtil.attrVal(attrNode, "y"); 
		if (x != null && y != null) 
			if (x.length() > 0 && y.length() > 0) {
				int _x = Util.str2int(x);
				int _y = Util.str2int(y);    		
				p = new Point(_x, _y);    		
			}
		return p;
	}	

}
