package main.storage;
import java.awt.Point;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.StringReader;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Vector;
import java.util.concurrent.atomic.AtomicInteger;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import main.Edge;
import main.EdgeParam;
import main.JobType;
import main.Task;
import main.TaskSet;
import main.Util;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import taskeditor.movables.Nail;

import enums.ActionType;
import enums.SendSemantic;


public class TSLoader {
	
	// input_type can be either "FILE" or "STRING"
	public static TaskSet load(String path, String input_type) throws FileNotFoundException {
		Task.resetPriorityCounter();
		Document document = getXMLDoc(path, input_type);
		
		String globals 		= "";
		String contexClause = "";
		String procedure 	= "";
		LinkedList<Task> tastList = new LinkedList<Task>();
		
		//Iterating through the nodes and extracting the data.
		Element root = document.getDocumentElement();
		String name = XMLUtil.attrVal(root, "name");
		NodeList nodeList = root.getChildNodes();
		for (int i = 0; i < nodeList.getLength(); i++) {
			Node n = nodeList.item(i);
			String node = n.getNodeName();
			// A <task> tag:
			if (node.equalsIgnoreCase("task")) {
				tastList.add(createTask(n, path));				
			} else if (node.equalsIgnoreCase("globals"))
				globals = n.getTextContent();
			else if (node.equalsIgnoreCase("contextclause"))
				contexClause = n.getTextContent();				
			else if (node.equalsIgnoreCase("procedure"))
				procedure = n.getTextContent();
		}
		return new TaskSet(tastList, contexClause, globals, procedure, name);  // To change body of created methods use File | Settings | File Templates.
	}
	
	private static Task createTask(Node n, String p) {
		String path = StorageUtil.extractPath(p);
		Task task = new Task(n, path);
		setMaxEdgeInd(task, n);
		NodeList nodes = n.getChildNodes();
		createJobs(task, nodes, path);
		createDependencies(task, nodes);
		setStartJob(task, n);
		setVariables(task, nodes);		 
		return task;
	}
	
	private static void setMaxEdgeInd(Task task, Node n) {
		//max_edge_index = new AtomicInteger(Integer.parseInt(getAttrVal(taskNode, "max_edge_index")));   // load it from xml 
		String str = attrVal(n, "max_edge_index");
		int index = str.isEmpty() ? 0 : Integer.parseInt(str);
		task.setmaxEdgeInd(new AtomicInteger(index));
	}

	private static void createJobs(Task t, NodeList job_edge_nodes, String path) {
		for (int j = 0; j < job_edge_nodes.getLength(); j++) {
			Node jNode = job_edge_nodes.item(j);
			if (jNode.getNodeName().equalsIgnoreCase("jobType")) {
				int x = getItemValue(jNode, "x");
				int y = getItemValue(jNode, "y");                               
				String e = "";
				String d = "";
				String n = "";	// name
				NodeList jAttList = jNode.getChildNodes();
				for (int k = 0; k < jAttList.getLength(); k++) {
					Node attrNode = jAttList.item(k);
					String attName = attrNode.getNodeName();
					String content = attrNode.getTextContent();

					if (attName.equalsIgnoreCase("name"))
						n = content;
					else if (attName.equalsIgnoreCase("WCET"))
						e = content;
					else if(attName.equalsIgnoreCase("deadline"))
						d = content;
				}
				JobType job = new JobType(n, e, d, x, y);
				StorageUtil.setJobTextLabelPosition(jNode, job);
				Node pn = jNode.getAttributes().getNamedItem("path");
				if (pn != null)
					job.setCode(readFile(path + pn.getNodeValue()) );

				t.addNewJob(job);
			}
		}
	}

	private static void createDependencies(Task t, NodeList nodes) {
		for (int j = 0; j < nodes.getLength(); j++) {
			Node dNode = nodes.item(j);
			if (dNode.getNodeName().equalsIgnoreCase("dependency")) {
				NodeList dAttList = dNode.getChildNodes();
				String src = null, dst = null, guard = null, code = null;
				int inter_release = 0;
				int index = -1;          // new edge indexing
				String act = "";  // new
				ActionType actType = EdgeParam.defaultAction();
				SendSemantic send =  EdgeParam.defaultSendSem();
				Point labelP = null;
				Vector<Point> nails = new Vector<Point>();                 
				for (int k = 0; k < dAttList.getLength(); k++) {
					Node node = dAttList.item(k);
					String attr = node.getNodeName(); 
					if (attr.equalsIgnoreCase("src"))
						src = node.getTextContent();
					else if (attr.equalsIgnoreCase("dst"))
						dst = node.getTextContent();
					else if (attr.equalsIgnoreCase("edge_index")) 
						index = Integer.parseInt( node.getTextContent() );
					else if (attr.equalsIgnoreCase("interRelease")) {
						inter_release = Integer.parseInt( node.getTextContent() );
						labelP = StorageUtil.getEdgeLabelP(node);
					} else if (attr.equalsIgnoreCase("action")) {                             
						actType = getActType(node);
						send = StorageUtil.getSendSem(node, send);
						act = node.getTextContent();                    	
					} else if (attr.equalsIgnoreCase("nails"))     
						nails = StorageUtil.extractNails(node);
					else if (attr.equalsIgnoreCase("guard"))
						guard = node.getTextContent();
					else if (attr.equalsIgnoreCase("code"))
						code = node.getTextContent();
					else
						System.out.println("Warning: unknown attribute in a dependency node: " + node.getNodeName());                    
				}
				if (index == -1) {   // means this attribute is missing in the loaded file
					index = t.get_edge_Index();
				}

				JobType srcJob = findByName(t.jobIterator(), src);
				JobType dstJob = findByName(t.jobIterator(), dst);
				if (srcJob == null || dstJob == null)	// there is an error while loading
					continue;
				if (nails.size() > 0)	// already loaded
					;
				else if (srcJob == dstJob)
					nails = selfNail(srcJob);
				else if (anyEdge(t, dstJob, srcJob))	// already there is an edge in reverse direction
					nails = middleNail(srcJob, dstJob);

				EdgeParam ed = new EdgeParam(inter_release, act, actType, send); 
				Edge e = new Edge(index, srcJob, dstJob, ed, nails);   // extended with index
				if (labelP != null)
					e.setLabelLoc(labelP);
				if (guard != null)
					e.setGuard(guard);
				if (code != null)
					e.setCode(code);
				t.addEdge( e );  // changed              
			}
		}
	}

	private static Vector<Point> selfNail(JobType srcJob) {
		int x0 = (int)srcJob.x;
		int y0 = (int)srcJob.y;

		int x3 = x0 + 100;
		int y3 = y0 + 70;
		int x4 = x0 + 100;
		int y4 = y0 - 30;        	

		Vector<Point> nails = new Vector<Point>();
		nails.add(new Point(x3, y3));
		nails.add(new Point(x4, y4));

		return nails;
	}
	

	public static boolean anyEdge(Task t, JobType from, JobType to) {
		ListIterator<Edge> itr = t.edgeIterator();
		while (itr.hasNext()) {
			Edge e = itr.next();
			if (from == e.source() && to == e.destin())
				return true;
		}
		return false;
	}

	private static Vector<Point> middleNail(JobType srcJob, JobType dstJob) {
		double r = 30;
		double dx = dstJob.x - srcJob.x;
		double dy = dstJob.y - srcJob.y;

		double x = (srcJob.x + 30 + dstJob.x + 30) / 2;
		double y = (srcJob.y + 30 + dstJob.y + 30) / 2;

		if (dx == 0)
			x = x + r * Math.signum(dy);
		else if (dy == 0)
			x = x + r * Math.signum(dx);
		else {
			double m = dy / dx;
			m = - 1.0 / m;
			double theta = Math.atan(m);
			x = x + r * Math.cos(theta); 
			y = y + r * Math.sin(theta);    		
		}
		if (dy == 0)
			y += 15;	
		/*double x = (srcJob.x + dstJob.x) / 2 + Math.random()*30 - 15;
    	double y = (srcJob.y + dstJob.y) / 2 + Math.random()*30 - 15;
		 */

		Point nail = new Point((int)x, (int)y);
		Vector<Point> nails = new Vector<Point>();
		nails.add(nail);    	
		return nails;
	}
	
	private static JobType findByName(ListIterator<JobType> jItr, String src) {
		while (jItr.hasNext()) {
        	JobType job = jItr.next();
        	if (job.getName().equalsIgnoreCase(src))
				return job;
		}
		String errMsg = "Warning: no job found with name " + src + " during XML loading.";
		System.out.println(errMsg);
		Util.log(errMsg);
		return null;
	}

	// if position not defined, return None
	private static ActionType getActType(Node attrNode) {
		final String NONE = String.valueOf(ActionType.None);
		final String SEND = String.valueOf(ActionType.Send);
		final String RECV = String.valueOf(ActionType.Receive);

		ActionType type = ActionType.None;
		Node tn = attrNode.getAttributes().getNamedItem("type");
		if (tn != null) {
			String s = tn.getNodeValue();
			if (s.equalsIgnoreCase(NONE))
				type = ActionType.None;
			else if (s.equalsIgnoreCase(SEND))
				type = ActionType.Send;
			else if (s.equalsIgnoreCase(RECV))
				type = ActionType.Receive;
			else
				System.out.println("Warning: Unknwon Action Type: " + s);
		}
		return type;
	}
	
	private static int getItemValue(Node jNode, String itemName) {
		return (int)Util.str2num(attrVal(jNode, itemName));
	}

	private static String readFile(String path) {		
		return Util.fileContent(path);
	}
	
	private static void setStartJob(Task t, Node taskNode) {
		String name = XMLUtil.attrVal(taskNode, "startJob");
		if (name == null)
			return;
		
		ListIterator<JobType> itr = t.jobIterator();
		while (itr.hasNext()) {
			JobType j = itr.next();
			if (j.getName().equals(name))
				t.setStartJob(j);
		}
	}	

	private static void setVariables(Task t, NodeList nodes) {
		for (int j = 0; j < nodes.getLength(); j++) {
			Node n = nodes.item(j);
			if (n.getNodeName().equalsIgnoreCase("variables")) {
				String content = n.getTextContent();
				t.setLocalVars(content);
			}
		}		
	}

	private static String attrVal(Node node, String attr) {
		return XMLUtil.attrVal(node, attr);
	}
	
	private static boolean fileExists(String path) {
		File f = new File(path);
		if (f.exists() && !f.isDirectory()) { 
		    return true;
		}
		return false;
	}
	
	private static Document getXMLDoc(String path, String input_type) throws FileNotFoundException  {
		//Get the DOM Builder Factory
		DocumentBuilderFactory factory =
			DocumentBuilderFactory.newInstance();

		//Get the DOM Builder
		DocumentBuilder builder = null;
		try {
			builder = factory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
		}

		//Load and Parse the XML document
		//document contains the complete XML as a Tree.
		Document document = null;
		try {
			if (input_type.equalsIgnoreCase("FILE")) {
				FileInputStream fis = new FileInputStream(path);
				document = builder.parse(fis);     // "DRT.xml"		
			} else if (input_type.equalsIgnoreCase("STRING")) {
				InputSource is = new InputSource(new StringReader(path));
				document = builder.parse(is);     // "DRT.xml"
			} else
				System.out.println("Unknown type of XML input: " + input_type);
		} catch (SAXException e) {
			Util.log("SAXException in getXMLDoc()");
			e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.			
				
		} catch (IOException e) {
			Util.log("IOException in getXMLDoc()");
			Util.log(Util.getStackTrace(e));
			e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.			
			if(fileExists(path) == false && input_type.equalsIgnoreCase("FILE"))
				throw new FileNotFoundException("File " + path + " not found.");			
		}
		if (document == null)
			Util.log("XML doc not loaded");
		return document;
	}	
}
