package services;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.LinkedList;

import analysis.model.Settings;
import analysis.model.enums.Processor;

import enums.TaskModel;

import util.Dialogs;
import util.SchedPolicy;

import main.Manager;
import main.NotesPanel;
import main.Task;
import main.TaskSet;
import main.Util;
import main.storage.TSSaver;


public class DRTStub {
	
	private	String 		pyth_path;
	private	String 		drt_path;	
	private	String 		sdrt_path;	
	private Process 	running_pyth_proc = null;
	
	/*
	 * For response time analysis
	 */
	public RunResult run_resp_time(String[] command) {
		// param is the selected task name
		RunResult res = new RunResult(command);
		ProcessBuilder pb = getProcessBuilder(command, drt_path);		
		double time_1 = System.currentTimeMillis();

		try {
			Process p = pb.start();
			setProcess(p);
			
			BufferedReader r = getReader(p);
			String line;
			while (true) {
				line = r.readLine();
				if (line == null)
					break;
				res.addLine(line);
			}
			double elapsed = (System.currentTimeMillis() - time_1) / 1000;
			res.setRunTime(elapsed);
			
			p.waitFor();
			setProcess(null);
			if (p.exitValue() != 0) {
				unsuccess(p);
				res.succeed(false);
			}
		} catch (Exception e) {
			System.out.println(ErrMsg);
			e.printStackTrace();
		}

		return res;
	}
	
	public RunResult runAnalysis(Settings cfg, TaskSet taskSet) throws Exception {


		String coreN = cfg.coreNo() + "";
		String binPc = cfg.binPc()  + "";
		String sortC = cfg.sortC()  + "";
		TaskModel lib= cfg.model();
		String proc  = (cfg.prcsr() == Processor.unicore) ? "u" : "m";
		
		String libPath = analysisEnginePath(lib);
		String temp_file = "temp10112015.xml";		
		save(taskSet, libPath + "/" + temp_file);
				
		String policy = Util.pol2str(cfg.sched());
		if (validatePaths(libPath + "/" + ANALYSIS_PY) == false)
			return null;
		
		String []command = new String[]{pyth_path, ANALYSIS_PY, temp_file, coreN, policy, binPc, sortC, proc}; 
		RunResult res = new RunResult(command);
		res.succeed(false);
		try {
			ProcessBuilder pb = getProcessBuilder(command, libPath);
			double time_1 = System.currentTimeMillis();
			Process p = pb.start();
			setProcess(p);
			
			BufferedReader r = getReader(p);
			String line;
			while (true) {
				line = r.readLine();
				if (line == null) {
					break;
				}
				res.addLine(line);
			}
			double elapsed = (System.currentTimeMillis() - time_1) / 1000;
			res.setRunTime(elapsed);
			p.waitFor();
			setProcess(null);
			if (p.exitValue() != 0) {
				unsuccess(p);
				res.succeed(false);
			} else { // successful analysis
				res.succeed(true);										
			}
			Util.debug("Exiting run ...");
		} catch (Exception e) {
			res.succeed(false);
			Util.error(ErrMsg);
			Util.error(e);
			// throw e;
		}
		return res;
	}
	
	private String analysisEnginePath(TaskModel lib) {
		String path = "";
		if (lib == TaskModel.DRT)
			path = drt_path;
		else if (lib == TaskModel.SDRT)
			path = sdrt_path;
		return path;
	}

	public RunResult runRandomTasks(String type, String crit, String Ubound,
			String TNum, String TSParam, Manager mgr) {

		String[] cmd;
		
		if (type.equalsIgnoreCase("C")) // if it is customized
			cmd = new String[]{pyth_path, RNDTASK_PY, Ubound, TNum, type, crit, TSParam};
		else
			cmd = new String[]{pyth_path, RNDTASK_PY, Ubound, TNum, type, crit};
				
		RunResult res = new RunResult(cmd);
		if (validatePaths(drt_path + "/" + RNDTASK_PY) == false) {
			res.succeed(false);
			res.setMessage("Invalid path.");			
			return res;
		}
		
		ProcessBuilder pb = getProcessBuilder(cmd, drt_path);
		try {
			Process p = pb.start();
			setProcess(p);
			BufferedReader r = getReader(p);
			String line;
			String xml = "";
	
			while (true) {
				line = r.readLine();
				if (line == null) {
					break;
				}
				xml += line;
			}
			p.waitFor();
			if (p.exitValue() != 0) {
				unsuccess(p);
				res.setMessage(errMsg());
				res.setLog(errLog(p));
				res.succeed(false);
				res.setExitValue(p.exitValue());
			} else {
				res.setMessage("Random task set generated.");
			}
			res.addLine(xml);
		} catch (Exception e) {
			String msg = "Random task set generation unsuccessful due to an error in execution, "
					+ "\r\n check the python and drtlib file paths are specified correctly.";
			res.setMessage(msg);
			res.succeed(false);
			res.setException(e);
		}
		setProcess(null);
		return res;
	}
	
	// metric = MRF or DBF
	public RunResult workload(String metric, TaskSet taskSet, int maxD) {
		String temp_file = "temp10112015.xml";
		save(taskSet, drt_path + "/" + temp_file);		
		String[] cmd = new String[]{pyth_path, RFDBF_PY, temp_file, metric, maxD + ""};
			
		RunResult res = new RunResult(cmd);
		ProcessBuilder pb = getProcessBuilder(cmd, drt_path);				
		try {
			Process p = pb.start();
			setProcess(p);

			BufferedReader r = getReader(p);
			String line;
			while (true) {
				line = r.readLine();
				if (line == null) {
					break;
				}
				res.addLine(line);
			}
			p.waitFor();
			setProcess(null);
			if (p.exitValue() != 0) {
				res.succeed(false);
				unsuccess(p);
			} 
		} catch (Exception e) {
			res.succeed(false);
			Util.error(ErrMsg);
			Util.error(e);
		}
		return res;		
	}

	public RunResult[] runRespTime(Task[] selectedTasks, TaskSet taskSet, NotesPanel notes) {		
		String policy = Util.pol2str( SchedPolicy.SP ); 	// it is not implemented yet for EDF
		
		RunResult[] results = new RunResult[selectedTasks.length];
		for (int i = 0; i < selectedTasks.length; i++) {
			Task task = selectedTasks[i];
			Task[] list = getHPtasks(task, taskSet.toArray());
			String temp_file = "temp10112015.xml"; // just a random name
			TSSaver.save(list, taskSet, drt_path + "/" + temp_file); 
			
			if (list.length == 0)
				return null;			
	
			String[] cmd = new String[]{pyth_path, RESP_TIME_PY, temp_file, task.getName(), policy};			
			results[i] = run_resp_time(cmd);
			notes.println("Response times computed for task " + task.getName() + ".");
		}
		return results;
	}

	public Task[] getHPtasks(Task task, Task[] all) {
		LinkedList<Task> list = new LinkedList<Task>();
		int prio = task.getPriority();
		for (Task t : all)
			if (t.getPriority() <= prio)  // smaller number means higher priority
				list.add(t);
		return list.toArray(new Task[]{});
	}
	
	private static BufferedReader getReader(Process p) {
		InputStreamReader isr = new InputStreamReader(p.getInputStream());
		return new BufferedReader(isr);
	}
	
	private void setProcess(Process p) {
		running_pyth_proc = p;
	}

	public void loadPaths() {
		String[] conf = Util.load_conf();
		pyth_path = conf[0];
		drt_path = conf[1];
		sdrt_path = conf[2];	
	}

	public boolean running() {
		return running_pyth_proc != null;
	}

	public void stop() {
		 running_pyth_proc.destroy();		
	}

	private void save(TaskSet taskSet, String file) {
		TSSaver.save(taskSet.toArray(), taskSet, file);		
	}

	public boolean validatePaths() {
		return validatePaths(drt_path + "/" + ANALYSIS_PY);
	}
	
	private boolean validatePaths(String drtPath) {
 		if (Util.exists(pyth_path) == false) {
			if (isValidCommand(pyth_path) == false) {
				Dialogs.showErrorMsg(ErrMsgPyth);
				return false;
			}
		} 
 		if (Util.exists(drtPath) == false) {
			Dialogs.showErrorMsg(ErrMsgDRTlib);
			return false;
		}		
		return true;		
	}

	private boolean isValidCommand(String cmd) {
		ProcessBuilder pb = new ProcessBuilder(new String[]{cmd, "-V"});
		Process p = null;
		try {
			p = pb.start();
			String s = errLog(p);
			p.destroy();
			p.waitFor();			
			// System.out.println(s);			
		} catch (IOException e) {
			return false;
		} catch (InterruptedException e) {
			return true;
		}
		return true;
	}
	
	public static void test() {
		DRTStub drt = new DRTStub();
		drt.isValidCommand("python");
		drt.isValidCommand("python1");
	}

	private ProcessBuilder getProcessBuilder(String[] command, String path) {
		ProcessBuilder pb = new ProcessBuilder(command);
		pb.directory(new File(path));
		return pb;
	}
	
	private static String errLog(Process p) {
		if (p == null)
			return "";
		byte[] buff = new byte[1000];
		try {
			p.getErrorStream().read(buff);
		} catch (IOException e) {
			Util.log(e);
		}		
		String msg = "Python program trace: \r\n" + new String(buff);
		return msg;
	}
	
	private String errMsg() {
		return "Python process terminated in an unexpected way.";
	}
	
	private void unsuccess(Process p) {
		String err = errMsg() + "\r\n";
		err += errLog(p);
		Util.error(err);
		// TO DO: throw exception
		// notesPanel.println(msg);
	}

	public void saveConf(String pyth, String drt, boolean save) {
		pyth_path = pyth;
		drt_path = drt;
		if (save)
			Util.save_conf(pyth, drt);
	}

	public String getPythPath() {
		return pyth_path;
	}

	public String getDrtPath() {
		return drt_path;
	}
	
	private	static 	final	String ANALYSIS_PY = "analysis.py";
	private	static 	final	String RNDTASK_PY 	= "rndtask.py";
	private	static 	final	String RFDBF_PY 	= "rf_dbf.py";
	private	static 	final	String RESP_TIME_PY	= "responsetime.py";
	
	private	static	final	String ErrMsg = "Analysis unsuccessful due to an error in execution, "
			+ "\r\n check the python and drtlib file paths are specified correctly";
	private	static	final	String ErrMsgPyth = "The Python installation path is not set correctly. " +
			"\r\nPlease set the correct path (or command) using \"Settings\" menu.";
	private	static	final	String ErrMsgDRTlib = "The path to drtlib is not set correctly. \r\n Please set the correct path using \"Settings\" menu.";
	
	
}
