package services;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import main.Util;

public class Executor {

	Process p;
	
	public RunResult run(String[] cmd) {
		RunResult res = new RunResult(cmd);
		ProcessBuilder pb = new ProcessBuilder(cmd);
		try {
			double start_time = now();
			p = pb.start();
			
			BufferedReader r = getReader(p);
			String line;
			while (true) {
				line = r.readLine();
				if (line == null)
					break;

				res.addLine(line);
			}
			res.setRunTime((now() - start_time) / 1000.0);
			p.waitFor();
			res.setExitValue(p.exitValue());
			if (p.exitValue() != 0)
				res.setLog(errLog(p));
		} catch (Exception e) {
			// p.destroy();
			res.setLog(errLog(p));
			p = null;
			res.succeed(false);
			res.setException(e);
		}
		p = null;
		return res;		
	}
	
	private static BufferedReader getReader(Process p) {
		InputStream is = p.getInputStream();
		InputStreamReader isr = new InputStreamReader(is);
		return new BufferedReader(isr);
	}
	

	private static String errLog(Process p) {
		if (p == null)
			return "";
		byte[] buff = new byte[1000];
		try {
			InputStream is = p.getErrorStream();
			if (is != null)
				is.read(buff);
		} catch (IOException e) {
			Util.log(e);
		}		
		String msg = "Executed program trace: \r\n" + new String(buff);
		return msg;
	}
	
	private long now() {
		return System.currentTimeMillis();
	}
	
}
