package simulator;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Vector;

import main.Util;

import simulator.engine.Execution;

public class SimUtil {
    
	/*
	 * Puts a string in the specified location 
	 */
    public static void drawString(Graphics2D g2, String s, float x, float y) {
    	Font font = g2.getFont();
        FontRenderContext frc = g2.getFontRenderContext();
        LineMetrics lm = font.getLineMetrics("0", frc);   
        float sh = lm.getAscent() + lm.getDescent();
        float sy = y + (-sh)/2 + lm.getAscent();
        float sw = (float)font.getStringBounds(s, frc).getWidth();
        float sx = x - sw/2;
        g2.drawString(s, sx, sy);
	}

    public static void drawStringAligned(Graphics2D g2, String str, float x, float y, boolean bold) {
    	Font font = g2.getFont();
        FontRenderContext frc = g2.getFontRenderContext();
        LineMetrics lm = font.getLineMetrics("0", frc);   
        float sh = lm.getAscent() + lm.getDescent();
        float sy = y + (-sh)/2 + lm.getAscent();
        
        FontMetrics fm = g2.getFontMetrics();
        Rectangle2D rect = fm.getStringBounds(str, g2);

        Color bgColor = Color.WHITE;
        Color c = g2.getColor();
        g2.setColor(bgColor);
        g2.fillRect((int)x,
        		(int)sy - fm.getAscent(),
                   (int) rect.getWidth(),
                   (int) rect.getHeight());

        g2.setColor(c);
        if (bold)
        	g2.setFont(g2.getFont().deriveFont(Font.BOLD));
        g2.drawString(str, x, sy);
        g2.setFont(font);
	}
    
    protected static void drawRotatedString(Graphics2D g2, String s, float x, float y) {
    	Font font = g2.getFont();
        FontRenderContext frc = g2.getFontRenderContext();
        LineMetrics lm = font.getLineMetrics("0", frc);   
        float sh = lm.getAscent() + lm.getDescent();
        float sy = y + (-sh)/2 + lm.getAscent();
        float sw = (float)font.getStringBounds(s, frc).getWidth();
        float sx = x - sw/2;
        
        AffineTransform defaultAt = g2.getTransform();
        AffineTransform at = new AffineTransform();
        at.rotate(Math.PI / 2);         
        g2.setTransform(at);
        g2.drawString(s, sy, -sx);
        g2.setTransform(defaultAt);        
	}
    
	public static double getMaxNameLen(Graphics2D g2, String[] names) {
		Font font = g2.getFont();
		FontRenderContext frc = g2.getFontRenderContext();
        
		double w = 0;
		double max = 0;
		for (int i = 0; i < names.length; i++) {
			w = font.getStringBounds(names[i], frc).getWidth();
			max = Math.max(max, w);
		}
        	
		return max;
	}
	
	public static double getMaxNameLen(String[] names) {
		if (names == null)
			return 0;
		
		double max = 0;
		for (int i = 0; i < names.length; i++)
			max = Math.max(max, names[i].length());        
		return max;
	}

	public static int lastExec(Vector<Execution> exTrace) {
		if(exTrace == null || exTrace.size() == 0)
			return 0;
		return ((Execution)exTrace.lastElement()).graphicalEndTime();
	}

	public static void drawExecutions(Graphics2D g2, Vector<Execution> exTrace,
			int PAD, int xPAD, int xscale, int taskNo, boolean bold) {
		Color[] colors = Util.randColors(taskNo);
        for (int i = 0; i < exTrace.size(); i++) {
        	Execution ex = exTrace.get(i);
        	int tn = ex.taskNum;
        	int y = 3*PAD + tn*PAD; // h-PAD-i*PAD;
        	try {
        	g2.setPaint(colors[tn]);
        	} catch(Exception e) {
        		System.out.println("------------->> Exception: t no = " + tn + " colos = " + colors.length);
        	}
        	
        	int x1 = xPAD + ex.start*xscale;
        	int x2 = ex.dur*xscale;
        	switch(ex.type) {
	        	case Arrival:
	        		drawArrow(g2, x1, y, x1, y-15);
	        		g2.setPaint(Color.BLACK);
	        		String s = ex.text;
	        		int off = offset(g2, exTrace, ex, i-1, bold);
	        		if (off > 0)
	        			s = " / " + s;
	        		float x = (float)x1 + off - 7;
	        		SimUtil.drawStringAligned(g2, s, x, y+11, bold);	        		
	        		break;
	        	case Execution:
	        		g2.fill(new Rectangle2D.Double(x1, y-6, x2, 6));
	        		break;
	        	case Deadline:
	        		drawArrow(g2, x1, y-15, x1, y);
	        		break;
	        	case DLMiss:
	        		//drawArrow(g2, (int)x1, y, (int)x1, y-10);
	        		// g2.fill(new Ellipse2D.Double(x1-3.5, y, 7, 7));
	        		Stroke strck = g2.getStroke();
	        		g2.setStroke(new BasicStroke(2));
	        		y = y - 6;
	        		drawLine(g2, x1-5, y-5, x1+5, y+5);
	        		drawLine(g2, x1-5, y+5, x1+5, y-5);
	        		g2.setStroke(strck);
	        		break;
        	}        	
        }		
	}

	private static int offset(Graphics2D g2, Vector<Execution> exTrace, Execution ex, int n, boolean bold) {
		int off = 0;
		Font font = g2.getFont();
        FontRenderContext frc = g2.getFontRenderContext();
        
        for (int i = 0; i <= n; i++) {
			Execution p = exTrace.get(i);
			if (p.type == EXEC_TYPE.Arrival)
				if (simultaneous(p, ex)) {
					if (bold)
						font = font.deriveFont(Font.BOLD);
					off += font.getStringBounds(p.text, frc).getWidth();
			}
        }
		return off;
	}

	private static boolean simultaneous(Execution p, Execution ex) {
		return (p.start == ex.start) && (p.taskNum == ex.taskNum); 
	}

	public static void drawArrow(Graphics g1, int x1, int y1, int x2, int y2) {
        final int ARR_SIZE = 4;
        Graphics2D g = (Graphics2D) g1.create();

        double dx = x2 - x1, dy = y2 - y1;
        double angle = Math.atan2(dy, dx);
        int len = (int) Math.sqrt(dx*dx + dy*dy);
        AffineTransform at = AffineTransform.getTranslateInstance(x1, y1);
        at.concatenate(AffineTransform.getRotateInstance(angle));
        g.transform(at);

        // Draw horizontal arrow starting in (0, 0)
        g.drawLine(0, 0, len, 0);
        g.fillPolygon(new int[] {len, len-ARR_SIZE, len-ARR_SIZE, len},
                new int[] {0, -ARR_SIZE, ARR_SIZE, 0}, 4);
    }

	public static void drawLine(Graphics2D g2, float x, float y, float x2, float y2, Color c) {
		Color prevC = g2.getColor();
		g2.setColor(c);
		drawLine(g2, x, y, x2, y2);
		g2.setColor(prevC);		// resume the initial color
	}
	
	public static void drawLine(Graphics2D g2, float x, float y, float x2, float y2, Color c, float thickNess) {
		Stroke st = g2.getStroke();
        g2.setStroke(new BasicStroke(thickNess));
        drawLine(g2, x, y, x2, y2, c);
        g2.setStroke(st); 	// resume the initial thickness
	}
	
	public static void drawLine(Graphics2D g2, float x, float y, float x2, float y2) {
		g2.draw(new Line2D.Double(x, y, x2, y2));                
	}

	public static void printEx(Vector<Execution> execs) {
		for (int i = 0; i < execs.size(); i++)
			execs.get(i).println();
	}
}
