package simulator.engine;

import java.util.LinkedList;

import main.EdgeParam;

import enums.ActionType;
import enums.SendSemantic;

import simulator.engine.SimEngine.EventType;
import simulator.simenums.SimMethod;

/*
 * Discrete Events showing an arrival, completion, or deadline miss
 */
class SEvent implements Comparable<SEvent> {

	private 	STask		owner;
	protected 	int 		time;
	protected 	EventType	type;
	
	public SEvent(EventType eType, STask task, int time) {
		type = eType;
		owner = task;
		this.time = time;
		/* System.out.println("+++++ New event generated: ++++");
		print();
		System.out.println("++++++++++++++++++++++++++++++++"); */
	}
	
	public LinkedList<Execution> handle(int now, Boolean maxProg) {
		return owner.progress(now, maxProg);	// removes me from its list		
	}

	public boolean isNull() {
		return (type == EventType.NONE);
	}
	
	public boolean notNull() {
		return !isNull();
	}
	
	public int compareTo(SEvent o) {
		if (this.time < o.time)
			return -1;
		if (this.time == o.time)
			return 0;
		return 1;
	}

	public void print() {
		System.out.println(toString());		
	}
	
	public String toString() {
		String tname = spacedStr(owner.name(), 12); 
		return tname + "-> Event type: " + type + ", time = " + time + 
				", owner id = " + owner.getId() + ", job = " + owner.arrivalLabel();
	}

	private String spacedStr(String name, int len) {
		String s = new String(name);
		for (int i = s.length(); i <= len; i++)
			s += " ";
		return s;
	}

	public EdgeParam currEdgeProps() {
		return owner.edgeProps();
	}

	public boolean hasCurrJob() {
		return owner.hasCurrJob();
	}

	public boolean hasPrevJob() {
		return owner.hasPrevJob();
	}
}
