package simulator.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Line2D;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;

import simulator.SimUtil;
import util.GUIUtil;

public abstract class AbsView extends JPanel implements MouseListener {
	
	protected	int 		panelWidth;   
	protected	int 		panelHeight;   
	protected	int 		PAD  = 40;
	protected	int 		xPAD = 100;		// default xPAD when there is no task loaded
	protected	int			labelSep;	
	protected 	boolean		show_major_gline;
	protected	boolean		show_minor_gline;
	protected 	JScrollPane scroll;	
		
	protected				final	int 	MAJOR_STEP = 5;	
		
	private		static		final 	String	MAJOR_GRID = "Major Gridlines";
	private		static		final 	String	MINOR_GRID = "Minor Gridlines";

	public AbsView(int w, int h) {
		panelWidth = w;
		panelHeight = h;
		labelSep = MAJOR_STEP;
		show_major_gline = true;
		show_minor_gline = true;
		addMouseListener(this);
		setBackground(Color.WHITE);
		setSizeTotal(panelWidth, panelHeight);
	}

    /*
     * Draw horizontal time line
     */
	protected void drawHLine(Graphics2D g2, int x1, int x2, int y, double sep) {
    	SimUtil.drawArrow(g2, x1, y, x2, y);
    	for (double x = x1; x <= x2 - sep; x += sep)
    		g2.draw(new Line2D.Double(x, y, x, y+2));
    	
    	for (float x = x1; x <= x2 - sep; x += MAJOR_STEP*sep)
       		g2.draw(new Line2D.Double(x, y, x, y+4));    	
    }
    
	protected void drawGrid(Graphics2D g2, int x1, int x2, int y1, int y2, int majorStep, double sep) {
		if (show_major_gline) {
			Color color = new Color(150, 150, 150, 150);		
    		for(float x = x1; x <= x2 - sep; x += majorStep*sep) 
    			SimUtil.drawLine(g2, x, y1, x, y2, color);
		}

    	if (show_minor_gline) {
    		Color color = new Color(200, 200, 200, 150);
    		for(float x = x1; x <= x2 - sep; x += sep)     			
    			SimUtil.drawLine(g2, x, y1, x, y2, color);
		}
	}

	protected void putLabels(Graphics2D g2, int x1, int x2, int y, double sep) {
    	int t = 0;
    	for (float x = x1; x <= x2 - sep; x += labelSep*sep) {
    		String s = "" + t;
    		SimUtil.drawString(g2, s, x, y);
    		t += labelSep;
    	}		
	}
    
	/*
     * handle clicks on the pane (for grid lines)
     */
    public void mousePressed(MouseEvent e) {
    	  
    }
   
	public void mouseClicked(MouseEvent arg0) {
		
	}

	public void mouseEntered(MouseEvent arg0) {
		
	}

	public void mouseExited(MouseEvent arg0) {
		
	}

	public void mouseReleased(MouseEvent e) {
		if (GUIUtil.rightClick(e)) {
			Point p = e.getPoint();
        	int menuX = p.x;
            int menuY = p.y;
        
            JComponent c = (JComponent) e.getSource();
        	JPopupMenu menu = createPopupMenu();
        	menu.show(c, menuX, menuY);                
        }  		
	}
	
    private JPopupMenu createPopupMenu() {
    	JPopupMenu popupMenu = new JPopupMenu();
    	final JCheckBoxMenuItem majorLineMenuItem = new JCheckBoxMenuItem(MAJOR_GRID, show_major_gline);
    	final JCheckBoxMenuItem minorLineMenuItem = new JCheckBoxMenuItem(MINOR_GRID, show_minor_gline);
    	majorLineMenuItem.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if(majorLineMenuItem.isSelected())
                	show_major_gline = true;
                else
                	show_major_gline = false;            	
            	repaint();
                }});
    	minorLineMenuItem.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if(minorLineMenuItem.isSelected())
                	show_minor_gline = true;
                else
                	show_minor_gline = false;            	
            	repaint();
                }});
    	popupMenu.add(majorLineMenuItem);
    	popupMenu.add(minorLineMenuItem);
    	return popupMenu;
	}
    
    protected void setSizeTotal(int w, int h) {
		setPreferredSize(new Dimension(w, h));  
    	setSize(new Dimension(w, h));      	
	}

	public JScrollPane getScroll() {
        return scroll;
	}
}
