package simulator.view;

import javax.swing.*;

import simulator.SimUtil;
import simulator.engine.Execution;
import util.GUIUtil;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.util.LinkedList;
import java.util.Vector;

/**
 * For showing the simulation
 */
public class SimView extends AbsView implements MouseWheelListener {
	
	// public		SimEngine	simEngine;
	private		int 		xscale = 10;	
	private 	boolean		scrBarHeld 	   	= false;		// true if scroll bar is currently held by the mouse
	private 	String[]	tasks;
	private		WorkloadView 		WLView;
	private 	Vector<Execution>	exTrace;
	private		final		boolean	BOLD 	= true; 	
		
	
	public SimView(WorkloadView wlView) {
		super(1000, 800);
		this.WLView = wlView;
		
		scroll = GUIUtil.customScroll(this);
		addScrListener(scroll);			 
		WLView.setXPAD(xPAD);
		
		exTrace = new Vector<Execution>();
		addMouseWheelListener(this);
	}
	
	/* this is called by manager when the window is maximized */
	public void triggerHeaderResize() {
		
	}
	
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        
        // String[] names = getTaskNames();
        int xPAD2 = getXPAD(g2, tasks);        
        
        if (tasks == null || tasks.length == 0)
        	return;        
        
        g2.setPaint(Color.black);
        
        int w = getWidth();
        w = resizeOnDemand(w, xPAD2);
        	
        drawHTimeLine(g2, PAD, w-PAD, PAD + 10, xscale, MAJOR_STEP);
        WLView.setXPAD(xPAD2 + PAD);
        
        // time line for each task
        for (int i = 0; i < tasks.length; i++) {
        	int y = 3*PAD + i*PAD; // h-PAD-i*PAD;
        	drawHLine(g2, PAD, w-PAD, y, xscale);
    	}
        
        /*** plotting all simulation events ***/
        SimUtil.drawExecutions(g2, exTrace, PAD, PAD, xscale, tasks.length, BOLD);
        g2.setPaint(Color.BLACK);	// restore default color        
    }

    /* 
     * The blank space from left of the frame
     */
	private int getXPAD(Graphics2D g2, String[] names) {
		if (names == null || names.length == 0)
			return xPAD;	// return default xPAD
        
		int maxLen = (int)(SimUtil.getMaxNameLen(g2, names));
		return maxLen + PAD / 2;
	}

	/*
	 * Checks if the simulated time exceeds the current dimensions of the panel, extend the panel's width
	 */
	private int resizeOnDemand(int w, int xPAD) {		
		final int GROWTH = 50;
		int lastTime = SimUtil.lastExec(exTrace);
        if (xPAD + lastTime * xscale + (GROWTH/2) * xscale > w) {
        	int newW = w + GROWTH * xscale;
        	setSizeTotal(newW, getHeight() /*panelHeight*/);  
        	WLView.expand(newW);
			w = newW;			
			revalidate();
    	}    	
		return w;
	}
	
	private void scrollOnDemand(final int xPAD, final int lastT) {
		
		if (scrBarHeld)
			return;
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				int lastPoint = lastT * xscale;
				int width = viewWidth();
				  
				JScrollBar horizBar = scroll.getHorizontalScrollBar();
				int val = horizBar.getValue() - xPAD;
				int beginPoint = val;
				int endPoint = beginPoint + width;
				if ( outside(lastPoint, beginPoint, endPoint) ) {
					smoothMove(xPAD + lastPoint - 3*width/4);
					// horizBar.setValue(xPAD + lastPoint - width/2);						
				}
			}
			  
			private void smoothMove(int target) {
				  final int start = scroll.getHorizontalScrollBar().getValue();
				  final int delta = target - start;
				  final int msBetweenIterations = 10;

				  ActionListener lis = new ActionListener() {
				      final long animationTime = 250; // milliseconds
				      final long nsBetweenIterations = msBetweenIterations * 1000000; // nanoseconds
				      final long startTime = System.nanoTime() - nsBetweenIterations; // Make the animation move on the first iteration
				      final long targetCompletionTime = startTime + animationTime * 1000000;
				      final long targetElapsedTime = targetCompletionTime - startTime;


					@Override
					public void actionPerformed(ActionEvent e) {
						  long timeSinceStart = System.nanoTime() - startTime;
				          double percentComplete = Math.min(1.0, (double) timeSinceStart / targetElapsedTime);

				          double factor = getFactor(percentComplete);
				          int val = (int) Math.round(start + delta * factor);
				          scroll.getHorizontalScrollBar().setValue(val);
				          WLView.setBarValue(val);
				          
				          if (timeSinceStart >= targetElapsedTime) {
				              ((javax.swing.Timer) e.getSource()).stop();
				          }						
					}

					private double getFactor(double percent) {
						/*return percent < 0.5
					            ? easeInCubic(percent * 2) / 2
					            : easeInCubic(percent * -2 + 2) / -2 + 1;
						*///return Math.pow(percent, 3);
						return percent;						
					}
				  };
				  
				  javax.swing.Timer scrollTimer = new javax.swing.Timer(msBetweenIterations, lis);

				  scrollTimer.setInitialDelay(0);
				  scrollTimer.start();				
			}

			private boolean outside(int v, int begin, int end) {				
				if (v < begin)
					return true;
				if (v > end)
					return true;
				
				return false;
			}

			int viewWidth() {
				  JViewport view = scroll.getViewport();
				  return view.getSize().width;
			  }
			 }
			);		
	}

    /*
     * Draw horizontal time line 
     */
    private void drawHTimeLine(Graphics2D g2, int x1, int x2, int y, double sep, int majorStep) {
    	drawHLine(g2, x1, x2, y + 10, sep);    	
    	int h = getHeight();    	
    	drawGrid(g2, x1, x2, y + 10, h, majorStep, sep);
    	putLabels(g2, x1, x2, y, sep);
    }
    
	public void reset(String[] names) {
		tasks = names;
		if(exTrace != null)
			exTrace.clear();
		else
			exTrace = new Vector<Execution>();  	// new set of execution traces because new task set is going to be simulated
		
		Graphics2D g2 = (Graphics2D)getGraphics();
		if(g2 == null)	// the graphical view is not yet initialized
			return;
		
		// modify the size of the pane
		int h = 3*PAD + names.length*PAD;
		setSizeTotal(panelWidth, h);
				
        int dist = PAD;
		int offset = 3*PAD;		
		int xPAD2 = getXPAD(g2, names);
        Ruler rowView = new Ruler(Ruler.VERTICAL, dist, offset, names);
        rowView.setPreferredHeight(offset + names.length * dist);
        rowView.setPreferredWidth(xPAD2);
        scroll.setRowHeaderView(rowView);
        
		JScrollBar horizBar = scroll.getHorizontalScrollBar();
		horizBar.setValue(0);
		
		WLView.reset();
	}

	public void refresh() {
		repaint();
		WLView.repaint();
	}

	private void addScrListener(JScrollPane scrl) {
		JScrollBar hBar = scrl.getHorizontalScrollBar();
		hBar.addMouseListener(new MouseListener() {
			public void mousePressed(MouseEvent mevt) {
				scrBarHeld = true;
		    }

			@Override
			public void mouseClicked(MouseEvent arg0) {				
			}

			@Override
			public void mouseEntered(MouseEvent arg0) {				
			}

			@Override
			public void mouseExited(MouseEvent arg0) {				
			}

			@Override
			public void mouseReleased(MouseEvent arg0) {
				scrBarHeld = false;
			}
		});
	}
	
	@Override
	public void mouseWheelMoved(MouseWheelEvent e) {    
	    if (e.isControlDown()) {
	        if (e.getWheelRotation() < 0) {
	            System.out.println("Zoom-in when scrolling up");
	            xscale = Math.min(xscale+1, MAX_SCALE); 
	        } else {
	            System.out.println("Zoom-out when scrolling down");
	            xscale = Math.max(xscale-1, MIN_SCALE);
	        }
	        adjustLabelSep();
	        WLView.setXScale(xscale);
	        repaint();
	        WLView.repaint();
	    }
	    else
	    	scroll.dispatchEvent(e);
	}

	private void adjustLabelSep() {		
    	labelSep = MAJOR_STEP;
    	if (xscale < 8)
    		labelSep = MAJOR_STEP * 2;
    	if (xscale < 5)
    		labelSep = MAJOR_STEP * 4;
    	
    	WLView.setLabelSep(labelSep);
	}

	public int getWLViewMinHeight() {
		return WLView.getMinHeight();
	}

	public void addNewExecs(LinkedList<Execution> new_execs, double wl, int time) {
		exTrace.addAll(new_execs);		
		WLView.addPoint(wl * 10, time);
	    
	    int lastTime = SimUtil.lastExec(exTrace);
	    scrollOnDemand(xPAD, lastTime);
	}	
	
	private		final 		int 		MIN_SCALE = 2;
	private		final 		int 		MAX_SCALE = 15;
}   