package simulator.view;

import javax.swing.*;

import simulator.SimUtil;
import util.GUIUtil;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Line2D;
import java.util.Vector;


/**
 * For showing the simulation instantanuous workload
 */
public class WorkloadView extends AbsView {
	
	private		int 		 xscale = 10;
	private		Vector<Point>points = null;
	private 	boolean		 refresh_header = false;		// set by the manager when the window is maximized
	
	private		final		int 	scrMinHeight;
	private		final		int 	yscale = 10;		
	private		final 		int 	YMAX = 100;

		
	public WorkloadView() {
		super(1000, 170);
		scrMinHeight = panelHeight + 25;	// default height		 
		
		scroll = GUIUtil.customScroll(this);
		scroll.setMinimumSize(new Dimension(panelWidth, scrMinHeight));
		JScrollBar vScrollBar = scroll.getVerticalScrollBar();
		vScrollBar.setValue( panelHeight );
	    
		reset_points();
	}

    protected void paintComponent(Graphics g) {
    	super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        
        if (refresh_header) {
    		refresh_header = false;
    		System.out.println("refresshing header");
    	}
    	
        g2.setPaint(Color.black);
        
        int w = getWidth();    
        int h = getHeight();
        
        //  refreshHeader(xPAD);		
        drawHTimeLine(g2, xPAD, w-PAD, h-PAD, xscale);
        drawYAxis(g2, xPAD, h-PAD, (h-PAD)-YMAX, yscale);
        drawPoints(g2);
    }

	private void drawPoints(Graphics2D g2) {
		for (int i = 1; i < points.size(); i++) {
			Point p1 = points.get(i-1);
			Point p2 = points.get(i);
			int x1 = xPAD + p1.x * xscale;
			int x2 = xPAD + p2.x * xscale;
			int h = getHeight();
			int y1 = h - PAD - p1.y;
			int y2 = h - PAD - p2.y;
			SimUtil.drawLine(g2, x1, y1, x2, y2, Color.RED);
		}
	}

    /*
     * Draw horizontal time line 
     */
    private void drawHTimeLine(Graphics2D g2, int x1, int x2, int y, double sep) {
    	drawHLine(g2, x1, x2, y, sep);
    	drawGrid(g2, x1, x2, y, y-YMAX, MAJOR_STEP, sep);
    	putLabels(g2, x1, x2, y + PAD / 2, sep);    	
    }

	/*
     * Draw vertical time line
     */
    private void drawYAxis(Graphics2D g2, int x, int y1, int y2, double sep) {
    	SimUtil.drawArrow(g2, x, y1, x, y2-PAD / 2);
    	if (show_major_gline)
        	for	(float y = (float) (y1-MAJOR_STEP*sep); y > y2 - sep; y -= MAJOR_STEP*sep) {    		
    			Color color = new Color(150, 150, 150, 150);
    			SimUtil.drawLine(g2, x, y, getWidth() - PAD, y, color);
    		}
    	if	(show_minor_gline)
    		for (float y = (float) (y1 - sep); y > y2 - sep; y -= sep) {
    			Color color = new Color(200, 200, 200, 150);
    			SimUtil.drawLine(g2, x, y, getWidth() - PAD, y, color);
    		} 
    	
    	double t = 50;
    	for	(float y = (float) (y1-MAJOR_STEP*sep); y > y2 - sep; y -= MAJOR_STEP*sep) {
    		String s = "" + t;
    		SimUtil.drawString(g2, s, x - PAD / 2 - 5, y);
    		t += 50;
    	}    
    	SimUtil.drawString(g2, "Workload", x + 20 - PAD / 2, (float) (y2 - sep - 20));    	
    }

	public void reset() {
		reset_points();
		
		Graphics2D g2 = (Graphics2D)getGraphics();
		if (g2 == null)	// the graphical view is not yet inititlaized, this happens when the application is starting
			return;
		
		setSizeTotal(panelWidth, panelHeight);		
		setBarValue(0);
	}
	
	public void reset_points() {
		if (points != null)
			points.clear();
		else
			points = new Vector<Point>();  	// new set of execution traces because new task set is going to be simulated
		points.add(new Point(0, 0));
	}

	/*
	 * Called by simPanel
	 */
	public void triggerHeaderRefresh() {
		refresh_header = true;		
	}

	public void setXPAD(int xpad) {
		xPAD = xpad;		
	}

	// called by SimView.run_one_step()
	public void addPoint(double wl, int currTime) {
		int h = (int)(wl*10);
		
		Point prev = points.lastElement();
		for (int t = prev.x + 1; t < currTime; t++)		// fill in the gap
			points.add(new Point(t, prev.y) );
		
		if (prev.y == 0)
			points.add(new Point(currTime, 0) );
		
		points.add(new Point(currTime, h) );
		if (h > getHeight())
			expand(getWidth(), h + 2*PAD);
	}

	public void expand(int newW) {
		setSizeTotal(newW, getHeight());
	}
	
	public void expand(int newW, int newH) {
		setSizeTotal(newW, newH);
	}

	public void setBarValue(int val) {
		scroll.getHorizontalScrollBar().setValue(val);		
	}

	protected void setXScale(int s) {
		xscale = s;		
	}

	public void setLabelSep(int sep) {
		labelSep = sep;
	}

	public int getMinHeight() {
		return scrMinHeight;
	}
}   