package synthesis;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JToolBar;

import enums.Language;
import main.Util;


public class ToolBar extends JToolBar implements ActionListener {

	private SynthesisTab parent;
	private JButton		 syntBut;
	private JButton 	 openBut;
	private JButton 	 saveBut;
	private JButton 	 chckBut;
	private JButton 	 compBut;
	
	private JButton 	 cntxBut;
	private JButton 	 globBut;
	private JButton 	 mainBut;
	
	private JButton 	 undoBut;
	private JButton 	 redoBut;
	private JButton 	 findBut;
	private JButton 	 plusBut;
	private JButton 	 minsBut;
	private JButton 	 confBut;
		
	public ToolBar(SynthesisTab st) {
		parent = st;		
		createButtons();

		add(syntBut);
		add(openBut);
		add(saveBut);
		addSeparator();
		add(chckBut);
		add(compBut);
		addSeparator();
		add(cntxBut);
		add(globBut);
		add(mainBut);
		addSeparator();
		add(undoBut);
		add(redoBut);
		addSeparator();
		add(findBut);
		addSeparator();
		add(plusBut);
		add(minsBut);
		add(glue());
		add(confBut);
		
		plusBut.setEnabled(false);	// not implemented yet
		minsBut.setEnabled(false);	// not implemented yet
		findBut.setEnabled(false);	// not implemented yet		
	}

	private JButton textButton(String text, String tip) {
		ImageIcon icon = Util.getIcon("/new.gif");
		JButton b = new JButton(text, icon);
		b.setHorizontalTextPosition(JButton.CENTER);
		b.setVerticalTextPosition(JButton.CENTER);
		b.setFont(b.getFont().deriveFont(Font.BOLD));
		b.setToolTipText(tip);
		return b;
	}
	
	private void createButtons() {
		ImageIcon iconExec = Util.getIcon("/synth.png");		
		ImageIcon iconOpen = Util.getIcon("/open.gif");
		ImageIcon iconSave = Util.getIcon("/save.gif");
		ImageIcon iconChck = Util.getIcon("/chck.png");
		ImageIcon iconComp = Util.getIcon("/comp.png");
		ImageIcon iconUndo = Util.getIcon("/undo.gif");
		ImageIcon iconRedo = Util.getIcon("/redo.gif");
		ImageIcon iconFind = Util.getIcon("/find.png");
		ImageIcon iconPlus = Util.getIcon("/plus.png");
		ImageIcon iconMins = Util.getIcon("/mins.png");
		ImageIcon iconConf = Util.getIcon("/conf.png");		
		
		syntBut = new JButton(iconExec);
		openBut = new JButton(iconOpen);
		saveBut = new JButton(iconSave);
		chckBut = new JButton(iconChck);
		compBut = new JButton(iconComp);
		undoBut = new JButton(iconUndo);
		redoBut = new JButton(iconRedo);
		plusBut = new JButton(iconPlus);
		minsBut = new JButton(iconMins);
		findBut = new JButton(iconFind);
		confBut = new JButton(iconConf);
		
		cntxBut = textButton("C", "Context Clause");
		globBut = textButton("G", "Global Declarations");
		mainBut = textButton("M", "Main Procedure");
		
		
		// set up the jbutton
		syntBut.setToolTipText("Generate");				
		openBut.setToolTipText("Open");
		saveBut.setToolTipText("Save");
		chckBut.setToolTipText("Syntax Check");
		compBut.setToolTipText("Compile");
		undoBut.setToolTipText("Undo");
		redoBut.setToolTipText("Redo");
		plusBut.setToolTipText("Zoom In");
		minsBut.setToolTipText("Zoom Out");
		findBut.setToolTipText("Find...");
		confBut.setToolTipText("Configuration");
		
		syntBut.addActionListener(this);
		openBut.addActionListener(this);
		saveBut.addActionListener(this);
		chckBut.addActionListener(this);
		compBut.addActionListener(this);
		undoBut.addActionListener(this);
		redoBut.addActionListener(this);
		plusBut.addActionListener(this);
		minsBut.addActionListener(this);
		findBut.addActionListener(this);
		confBut.addActionListener(this);
		cntxBut.addActionListener(this);
		globBut.addActionListener(this);
		mainBut.addActionListener(this);
	}

	private Component glue() {
		return  Box.createHorizontalGlue();
	}

	public void adapt(Language lang) {
		switch (lang) {
		case Ada:
			chckBut.setEnabled(true);
			compBut.setEnabled(true);
			cntxBut.setText("C");
			cntxBut.setToolTipText("Context Clauses");
			break;

		case C:
			chckBut.setEnabled(false);
			compBut.setEnabled(false);
			cntxBut.setText("H");
			cntxBut.setToolTipText("Headers");			
			break;

		default:
			break;
		}		
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == syntBut)
			parent.synthesisPressed();
		else if (e.getSource() == openBut)
			parent.openPressed();
		else if (e.getSource() == saveBut)
			parent.savePressed();
		else if (e.getSource() == chckBut)
			parent.checkPressed();
		else if (e.getSource() == cntxBut)
			parent.cntxClausPressed();
		else if (e.getSource() == globBut)
			parent.globDeclPressed();
		else if (e.getSource() == mainBut)
			parent.mainProcPressed();
		else if (e.getSource() == compBut)
			parent.compilePressed();
		else if (e.getSource() == undoBut)
			parent.undoPressed();
		else if (e.getSource() == redoBut)
			parent.redoPressed();
		else if (e.getSource() == confBut)
			parent.confPressed();
		else 
			System.out.println("Warning: Unimplemented action button");
	}
}
