package synthesis.model.adacoder;

import synthesis.model.AbsSynthesizer;
import synthesis.model.Settings;

import main.Task;
import main.TaskSet;
import main.Util;

public class AdaSynthesizer extends AbsSynthesizer {

	public AdaSynthesizer() {
		kwords = new String[] { "with", "use", "type", "subtype", "range",
				"package", "new", "procedure", "is", "begin", "body", "task",
				"end", "entry", "constant", "loop", "case", "when", "delay",
				"until", "select", "accept", "goto", "or", "constant", "null",
				"goto", "if", "then", "else", "while", "return", "pragma", "abort" };
		symbs = new String[] { ":=", ":", ";", "<", ">", "=" };
	}

	@Override
	public String synthesisCode(TaskSet ts, Settings setting) {
		String name = ts.getName();
		String globVars = ts.globalVars();
		String ctxClaus = ts.contextClause();
		Task[] tlist = ts.toArray();

		String code = "";
		code += header(ctxClaus, name);
		code += Coder.beginProc(name);
		code += gVarsCode(globVars);

		// code for inserting declaration code
		for (Task t : tlist)
			code += TaskCoder.declCode(t) + NL;
		// code += t.taskDecCode(TAB, NL) + NL;

		for (Task t : tlist)
			code += TaskCoder.synthesCode(t, tlist, setting) + NL;

		// code += t.synthesCode(tlist, setting, TAB, NL) + NL;

		code += procBody(ts);
		code += Coder.endProc(name);

		return code;
	}

	public static String header(String ctxClaus, String name) {
		String h = "";
		h += "-- Ada code generated by MASI" + NL;
		h += "-- for task set " + name + NL;
		h += "-- " + Util.Date() + NL;

		h += "pragma  Task_Dispatching_Policy ( FIFO_Within_Priorities ) ;" + NL + NL; 
		h += ctxClaus + NL;
		h += "with Ada.Real_Time; use Ada.Real_Time;" + NL;
		h += "with Ada.Text_IO;   use Ada.Text_IO;" + NL;
		h += "with System;        use System;" + NL + NL;		

		return h;
	}

	private String gVarsCode(String globVars) {
		// code added for putting the list of global variables
		String code = "";
		code += TAB + "----  Global variables declaration" + "  ----" + NL;
		code += TAB + "Program_Start_Time : Ada.Real_Time.Time:= Ada.Real_Time.Clock;" + NL;
		code += Coder.indent(1, globVars) + NL + NL;
		
		return code;
	}

	private String procBody(TaskSet ts) {
		/*** The body of the main procedure ***/
		String code = "begin" + NL;
		code += Coder.indent(1, ts.mainProcCode());
		code += Coder.indent(1, "null;");
		return code;
	}

	@Override
	public String suffix() {
		return SUFFIX;
	}

	final static String SUFFIX = ".adb";
	final static String NL = "\n"; // new line
	final static String TAB = "    "; // four spaces
}