package synthesis.model.adacoder;

import java.util.LinkedList;

import main.Edge;
import main.Task;
import synthesis.model.Settings;


class EdgeCoder extends Coder {

	public static String synthCode(Edge e, Task t, Task[] tasks, Settings conf) {
		String code = TAB(6) + "if " + e.guard() + " then" + NL;
		if (e.sends()) {
			Task[] receivers = receivers(t, e.action(), tasks);
			for (Task receiver : receivers) {
				code += sendCode(e, receiver, conf);
			}
		}
		code += transitionCode(e, t);
		code += TAB(6) + "end if;" + NL;
		return code;
	}

	private static String sendCode(Edge e, Task receiver, Settings conf) {
		String code = "";
		String sendCode = receiver.getName() + "." + e.action() + ";";

		if (e.blockingSend()) {
			code += TAB(7) + sendCode + NL;
			code += indent(7, "Next_Time := Clock;");
		} else {
			code += TAB(7) + "select" + NL;
			code += TAB(8) + sendCode + NL;
			code += TAB(7) + "else" + NL;
			code += TAB(8) + "null;" + NL;
			code += TAB(7) + "end select;" + NL;
		}
		return code;
	}

	public static String acceptCode(Edge e, Task t) {
		String code = indent(7, "when " + e.guard() + "  =>");
		code += indent(7, "accept " + e.action() + ";");
		code += indent(7, "Next_Time := Clock;");
		code += transitionCode(e, t);
		return code;
	}

	private static String transitionCode(Edge e, Task t) {
		String code = indent(7, e.code());		
		code += TAB(7) + "Current_State := " + e.dstName() + ";" + NL;
		code += TAB(7) + "goto " + t.getName() + "_end;" + NL; 
		return code;
	}
	
	private static Task[] receivers(Task t, String action, Task[] tasks) {
		LinkedList<Task> receivers = new LinkedList<Task>();
		for (Task task : tasks) {
			if (task.getName() == t.getName())
				continue;
			if (task.receives(action))
				receivers.add(task);
		}
		return receivers.toArray(new Task[] {});
	}
}