package synthesis.model.adacoder;

import java.util.Collections;
import java.util.LinkedList;

import main.Edge;
import main.JobType;
import main.Task;
import synthesis.model.Settings;


class JobCoder extends Coder {

	public static String procedureCode(JobType j, Task t) {
		String code = "";
		String name = j.getName();
		String body = j.getCode();
		body = (body == null) ? "null;" : body;
		body = (body.length() == 0) ? "null;" : body;
		
		code += TAB(2) + beginProc(name + "_code") + NL;
		code += TAB(2) + "begin" + NL;
		code += TAB(3) + "Put_Line (\" In job " + t.getName() + "." + name + " \" " +
				"& Duration'Image(Ada.Real_Time.To_Duration(Ada.Real_Time.Clock - Program_Start_Time)));" + NL;
		code += indent(3, body); 

		code += TAB(2) + endProc(name + "_code");
		return code;
	}
	
	public static String synthCode(JobType j, Task t, Task[] tasks,
			Settings conf) {

		String code = "";
		code += TAB(6) + j.getName() + "_code;" + NL; // call the associated
														// procedure
		LinkedList<Edge> outEdges = j.getBranchOrders();
		Collections.sort(outEdges); // sorting is done here instead of job level

		if (outEdges.size() == 0) {
			code += TAB(6) + "exit;" + NL; // task is finished
			return code;
		}

		// we have changes here for branch code
		Edge e = outEdges.get(0);
		code += TAB(6) + "Next_Time := Next_Time + " + edge_delay(e) + ";" + NL;
		code += TAB(6) + "delay until Next_Time;" + NL;

		for (int i = 0; i < outEdges.size(); i++) {
			e = outEdges.get(i);
			if (!e.receives())
				code += EdgeCoder.synthCode(e, t, tasks, conf);

			boolean hasNext = (i < outEdges.size() - 1);
			if (hasNext) {
				Edge next = outEdges.get(i + 1);
				code += TAB(6) + "Next_Time := Next_Time + " + edge_delay(next)
						+ " - " + edge_delay(e) + ";" + NL;
			}
			code += selectiveAcceptCode(outEdges, t, i, hasNext);
		}

		// code needed for exit case
		return code;
	}

	static String selectiveAcceptCode(LinkedList<Edge> outEdges, Task t,
			int last, boolean boundedWait) {
		Edge[] edges = acceptingEdges(outEdges, last);
		int len = edges.length;
		if (len == 0) {
			if (boundedWait)
				return TAB(6) + "delay until Next_Time;" + NL;
			else
				return "";
		}

		String code = TAB(6) + "select" + NL;
		for (int k = 0; k < len; k++) {
			code += EdgeCoder.acceptCode(edges[k], t);
			if (k != len - 1) {
				code += TAB(6) + "or" + NL;
			}
		}
		if (boundedWait) {
			code += TAB(6) + "or" + NL;
			code += TAB(7) + "delay until Next_Time;" + NL;
		}
		code += TAB(6) + "end select;" + NL;
		return code;
	}

	private static Edge[] acceptingEdges(LinkedList<Edge> edges, int last) {
		LinkedList<Edge> res = new LinkedList<Edge>();
		for (int k = 0; k <= last; k++) {
			Edge e = edges.get(k);
			if (e.receives())
				res.add(e);
		}
		return res.toArray(new Edge[] {});
	}

}
