package synthesis.model.adacoder;

import java.util.LinkedList;
import java.util.ListIterator;

import main.Edge;
import main.JobType;
import main.Task;
import synthesis.model.Settings;

public class TaskCoder extends Coder {
	public static String declCode(Task t) {
		String cmnt = "Task declaration for " + t.getName();
		String code = TAB + "----  " + cmnt + "  ----" + NL;
		code += TAB    + "task " + t.getName() + " is     -- A singleton task" + NL;
		code += TAB(2) + "pragma Priority (" + t.getPriority() + ");" + NL; // System.Default_Priority +  
		code += entryCode(t);
		code += TAB    + "end " + t.getName() + ";" + NL;
		return code;
	}

	private static String entryCode(Task t) {
		String[] actions = recActions(t);
		if (actions.length > 0) {
			String code = "";
			for (int i = 0; i < actions.length; i++) {
				code += TAB(2) + "entry " + actions[i] + ";" + NL;
			}
			return code;
		} else {
			return "";
		}
	}

	private static String[] recActions(Task t) {
		LinkedList<String> acts = new LinkedList<String>();
		ListIterator<Edge> itr = t.edgeIterator();
		while (itr.hasNext()) {
			Edge e = itr.next();
			if (e.receives()) {
				if (!acts.contains(e.action()))
					acts.add(e.action());
			}
		}
		return acts.toArray(new String[] {});
	}

	public static String synthesCode(Task t, Task[] tlist, Settings setting) {
		ListIterator<JobType> itr = t.jobIterator();

		String code = TAB + "task body " + t.getName() + " is" + NL;
		code += TAB + TAB + "---------- Variable declaration -------" + NL;
		code += varDeclaration(t) + NL;

		code += TAB + TAB + "-- Procedures for the job types of " + t.getName()
				+ ":" + NL;
		while (itr.hasNext()) {
			JobType j = itr.next();
			code += JobCoder.procedureCode(j, t) + NL + NL;
		}

		code += TAB + TAB + "--------------- Task logic --------------" + NL;
		code += TAB + TAB + "begin" + NL;
		code += behaviorCode(t, tlist, setting) + NL;

		code += TAB + "end " + t.getName() + ";" + NL;
		code += TAB + "--------------------------------------------";
		return code;
	}

	private static String varDeclaration(Task t) {
		String code = taskVars(t);
		if (t.isEmpty())
			return code;

		JobType firstJ = t.getStartJob();
		firstJ = (firstJ == null) ? t.getJob(0) : firstJ;
		ListIterator<JobType> jItr = t.jobIterator();
		ListIterator<Edge> eItr = t.edgeIterator();

		String states = jItr.next().getName();
		while (jItr.hasNext()) {
			states += ", " + jItr.next().getName();
		}

		code += TAB(2) + "type State is (" + states + ");" + NL;
		code += TAB(2) + "Current_State : State := " + firstJ.getName() + ";"
				+ NL;
		code += TAB(2) + "Next_Time : Ada.Real_Time.Time := Clock;" + NL; 
		
		// declaring inter-release times:
		while (eItr.hasNext()) {
			Edge e = eItr.next();
			code += TAB(2) + edge_delay(e) + " : constant Time_Span := Milliseconds(" 
					+ e.interRel()	+ ");" + NL;
		}
		return code;
	}

	private static String taskVars(Task t) {
		String code = "";
		String v = t.localVars();
		if (v == null)
			return code;
		
		String[]vars = v.split("\n");
		for (int i = 0; i < vars.length; i++)
			code += TAB(2) + vars[i] + NL;
		return code;
	}

	private static String behaviorCode(Task t, Task[] tasks, Settings cfg) {

		ListIterator<JobType> jItr = t.jobIterator();

		String code = TAB(3) + "Next_Time := Clock;" + NL;
		code += TAB(3) + "loop" + NL;
		if (t.isEmpty()) {
			code += TAB(4) + "null;" + NL;
			code += TAB(3) + "end loop;";
			return code;
		}

		code += TAB(4) + "case Current_State is" + NL;
		while (jItr.hasNext()) {
			JobType j = jItr.next();
			code += TAB(5) + "when " + j.getName() + " =>" + NL;
			code += JobCoder.synthCode(j, t, tasks, cfg);
		}
		code += TAB(4) + "end case;" + NL;
		// here we add case statement to end jump
		// added label for goto
		code += TAB(4) + "<<" + t.getName() + "_end>>" + NL;
		code += TAB(3) + "end loop;";
		return code;
	}
}
