package synthesis.model.ccoder;

import synthesis.model.AbsSynthesizer;
import synthesis.model.Settings;
import synthesis.model.ccoder.CCoder;
import synthesis.model.ccoder.CTaskCoder;
import main.Task;
import main.TaskSet;
import main.Util;

public class CSynthesizer extends AbsSynthesizer{
	
	
	public CSynthesizer() {
		kwords = new String[] { "void", "#for", "switch", "case",
				"break", "if", "else", "while", "do", "return", "for", "int",
				"static", "include"};
		symbs = new String[] { ":", ";", "\\{", "\\}", "=" };
	}

	@Override
	public String synthesisCode(TaskSet ts, Settings setting) {
		String name = ts.getName();
		String globVars = ts.globalVars();
		String ctxClaus = ts.contextClause();
		Task[] tlist = ts.toArray();

		String code = "";
		code += header(ctxClaus, name);
		code += prototypeCode(ts);
		code += gVarsCode(globVars);

		for (Task t : tlist)
			code += CTaskCoder.synthesCode(t, tlist, setting) + NL;

		// code for main function
		code += procBody(ts);
		
		// this is optional based on IDE-Compiler combination
/*		code += addHooks();
		
		code += addSetup();*/
	
		//code += Coder.endProc(name);

		return code;
	}

	public static String header(String ctxClaus, String name) {
		String h = "";
		h += "/* C code generated by MASI for FreeRTOS" + NL;
		h += "for task set " + name + NL;
		h += Util.Date() + "*/" + NL;

		h += ctxClaus + NL;
		h += "/* Kernel includes */" + NL;
		
		h += "#include \"FreeRTOS.h\"" + NL; // adding freertos
		h += "#include \"task.h\"" + NL + NL; // adding task header definitions
		
		// board specific header TODO: make it a configuration
		h += "#include \"stm32f10x.h\"" + NL;
		

		return h;
	}

	// this function is to generate task function prototype code
    private String prototypeCode(TaskSet ts){
    	String code = "";
    	Task[] tlist = ts.toArray();
    	
    	// prototype code for hardware setup function
        code += "static void prvSetupHardware( void );";	
        	
    	for (Task t : tlist)
			code += "static void " + t.getName() +"(void *pvParameters);" + NL;
    	    	
    	return code;
    }
	
	
	
	private String gVarsCode(String globVars) {
		// code added for putting the list of global variables
		String code = "";
		code += TAB + "/***  Global Variables Declaration" + "  ***/" + NL;
		code += CCoder.indent(1, globVars) + NL + NL;
		return code;
	}

	// main function - need to fix indentation
	private String procBody(TaskSet ts) {
		Task[] tlist = ts.toArray();
		/*** The body of the main procedure ***/
		String code = "int main (void) " +NL;
		code +="{"+NL+NL;
		
		code += TAB + TAB + "prvSetupHardware();" + NL;
		
    	for (Task t : tlist)
			code += TAB + TAB + "xTaskCreate(" + t.getName() +",(signed char*)\"" + t.getName() 
			+ "\", configMINIMAL_STACK_SIZE, NULL, "+ t.getPriority() + ", NULL);" + NL;
		
    	code += TAB + TAB + "vTaskStartScheduler();" + NL;
        code += TAB + TAB + "for(;;);" + NL;
        code += "}" + NL;
        
/*		String code = "begin" + NL;
		code += Coder.indent(1, ts.mainProcCode());
		code += Coder.indent(1, "null;");*/
		return code;
	}
	
	private String addHooks(){
		String code = "/* Application tick hook */" + NL + NL;
		
		code += "void vApplicationTickHook( void )" + NL;
		code += "{" + NL;
		
		code += NL + NL;
		
		code += "}" + NL + NL;
		
		
		code += "/* Application Malloc failed hook */" + NL + NL;
		
		code += "void vApplicationMallocFailedHook( void )" + NL;
		
		code += "{" + NL;
		
		code += TAB + TAB + "for( ;; );" + NL;
		
		code += "}" + NL + NL;
		
		
		code += "/* Application Stack overflow hook */" + NL + NL;
		
		code += "void vApplicationStackOverflowHook( xTaskHandle pxTask, signed char *pcTaskName )" + NL;
		
		code += "{" + NL;
		
		code += TAB + TAB + "( void ) pcTaskName;" + NL;
		code += TAB + TAB + "( void ) pxTask;" + NL + NL;
		code += TAB + TAB + "for( ;; );" + NL;
		
		code += "}" + NL + NL;
		
		
        code += "/* Application idle hook */" + NL + NL;
		
		code += "void vApplicationTickHook( void )" + NL;
		code += "{" + NL;
		
		code += NL + NL;
		
		code += "}" + NL + NL;
				
		return code;
	}
	
	
	private String addSetup() {
		String code = "static void prvSetupHardware( void )" + NL;
		
		code += "{" + NL + NL;
		
		code += TAB + TAB + "NVIC_SetPriorityGrouping( 0 );" + NL;
		
		code += "}" + NL + NL;
		
		return code;
	}
	
	

	@Override
	public String suffix() {
		return SUFFIX;
	}

	final static String SUFFIX = ".c";
	final static String NL = "\n"; // new line
	final static String TAB = "    "; // four spaces

}
