package synthesis.model.ccoder;

import java.util.LinkedList;
import java.util.ListIterator;

import main.Edge;
import main.JobType;
import main.Task;
import synthesis.model.Settings;

public class CTaskCoder extends CCoder {
	public static String declCode(Task t) {
		String cmnt = "task declaration for " + t.getName();
		String code = TAB + "----  " + cmnt + "  ----"  + NL;
		code += TAB + "task " + t.getName() + entryCode(t)
				+ ";     -- a singleton task" + NL;
		return code;
	}

	private static String entryCode(Task t) {
		String[] actions = recActions(t);
		if (actions.length > 0) {
			String code = " is" + NL;
			for (int i = 0; i < actions.length; i++) {
				code += TAB(2) + "entry " + actions[i] + ";" + NL;
			}
			code += TAB + "end " + t.getName();
			return code;
		} else {
			return "";
		}
	}

	private static String[] recActions(Task t) {
		LinkedList<String> acts = new LinkedList<String>();
		ListIterator<Edge> itr = t.edgeIterator();
		while (itr.hasNext()) {
			Edge e = itr.next();
			if (e.receives()) {
				if (!acts.contains(e.action()))
					acts.add(e.action());
			}
		}
		return acts.toArray(new String[] {});
	}

	public static String synthesCode(Task t, Task[] tlist, Settings setting) {
				
		ListIterator<JobType> itr = t.jobIterator();
		
		JobType j;
		
		String code = "/* Task function */" +NL; 
		code += CCoder.beginTask(t.getName())+NL;

		//String code = TAB + "task body " + t.getName() + " is" + NL;
		code += TAB + TAB + "/* Local  variable declaration */" + NL;
		code += varDeclaration(t) + NL;

		//code += TAB + TAB + "-- procedures for the job types of " + t.getName() + ":" + NL;

		// main loop for task code
		code += TAB + "for (;;){" + NL;
		
		// switch case for each job code
		code += TAB + TAB + "switch(job_index){" + NL;   
		//  cases
	    while (itr.hasNext()) {
	    	
	    	j = itr.next();
			
			code += TAB + TAB + "case "+ t.getJobIndex(j) + " :" + NL;
			
			code += TAB + TAB + CJobCoder.JobCode(j) + NL;
			
			//code += CJobCoder.synthCode(j, t, tlist, setting) + NL;
			
			code += behaviorCode(t, tlist, setting, j) + NL;		
		}
				
		// end of switch case structure
		code += TAB + TAB + "}"+ NL;
		
		// end of main for loop
		code += TAB + "}" + NL;
			
		code += CCoder.endTask()+NL;
		return code;
	}

	private static String varDeclaration(Task t) {
		String code = taskVars(t);
		
		code += TAB(2) + "portTickType xReleaseTime;"+NL;
		code += TAB(2) + "int job_index = 0;" + NL;

		if (t.isEmpty())
			return code;
		
		JobType firstJ = t.getStartJob();
		firstJ = (firstJ == null) ? t.getJob(0) : firstJ;
		ListIterator<JobType> jItr = t.jobIterator();
		ListIterator<Edge> eItr = t.edgeIterator();

		
		while (eItr.hasNext()) {
			Edge e = eItr.next();
			code += TAB(2) + "portTickType " + edge_delay(e) + " = " 
					+ e.interRel()	+ ";" + NL;
		}
		
		// assigning current job index to start job index
		code += TAB(2) + "job_index = "+ t.getJobIndex(firstJ)+";" + NL;
		// initialize release time of initial job 
		code += TAB(2) + "xReleaseTime = xTaskGetTickCount();" + NL; 
		
		// declaring inter-release times:

		return code;
	}

	private static String taskVars(Task t) {
		String code = "";
		String v = t.localVars();
		if (v == null)
			return code;
		
		String[]vars = v.split("\n");
		for (int i = 0; i < vars.length; i++)
			code += TAB(2) + vars[i] + NL;
		return code;
	}

	private static String behaviorCode(Task t, Task[] tasks, Settings cfg, JobType j) {
		
		String code = "";

        code += CJobCoder.synthCode(j, t, tasks, cfg);

		return code;
	}
}