package synthesis.view;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;

import javax.swing.BorderFactory;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.event.DocumentEvent.EventType;
import javax.swing.text.AbstractDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.PlainDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.TabSet;
import javax.swing.text.TabStop;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;
import javax.swing.undo.UndoableEdit;

import main.Util;

import enums.Language;

import simulator.view.Ruler;
import util.GUIAttr;

public class CodePane extends JScrollPane {

	private	int				linesNo;
	private boolean 		busy = false;
	private ColoredTextPane textPane;
		
	public CodePane() {
		textPane = createTextPane();
		setViewportView(textPane);
		getVerticalScrollBar().setUnitIncrement(19);
	}

	/*
	 * Public methods:
	 */
	public void display(String code) {
		try {
			busy = true;
			textPane.setText(code); 
			busy = false;
			resetHeaderView();
			textPane.setCaretPosition(0);			
		} catch (Exception e) {
			System.out.println("Exception during inserting new code to the code pane");
			e.printStackTrace();
		}
		// undoManager.discardAllEdits();
	}

	public String getText() {
		return textPane.getText();
	}

	public void undo() {
		textPane.undo();
	}

	public void redo() {
		textPane.redo();
	}

	public void stringInserted() {
		resetHeaderView();		
	}

	public boolean isBusy() {
		return busy;
	}
	

	public void setKWrds(String[] kwords) {
		textPane.setKWrds(kwords);
	}

	public void setSymbs(String[] symbs) {
		textPane.setSymbs(symbs);		
	}
	
	public void setLang(Language lang) {
		textPane.setLang(lang);
	}
	
	/*
	 * Private methods:
	 */
	/*
	 * Got from
	 * http://stackoverflow.com/questions/4059198/jtextpane-appending-a-
	 * new-string
	 */
	private ColoredTextPane createTextPane() {
		ColoredTextPane pane = new ColoredTextPane(this); 
		pane.setText("");
		pane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		pane.setFont(new Font("Consolas", Font.PLAIN, 16));
		
		TabStop[] tabs = new TabStop[4];/*
		tabs[0] = new TabStop(60, TabStop.ALIGN_RIGHT, TabStop.LEAD_NONE);
        tabs[1] = new TabStop(100, TabStop.ALIGN_LEFT, TabStop.LEAD_NONE);
        tabs[2] = new TabStop(200, TabStop.ALIGN_CENTER, TabStop.LEAD_NONE);
        tabs[3] = new TabStop(300, TabStop.ALIGN_DECIMAL, TabStop.LEAD_NONE);
        TabSet tabset = new TabSet(tabs);
		StyleContext sc = StyleContext.getDefaultStyleContext();
        AttributeSet aset = sc.addAttribute(SimpleAttributeSet.EMPTY,
        StyleConstants.TabSet, tabset);
		pane.setParagraphAttributes(aset, false);
		*/
        
//      int charWidth = fm.charWidth( 'w' );    
		
		resetHeaderView();
		return pane;
	}
	
	private void resetHeaderView() {
		int lines = getNumOfLines();
		if (linesNo == 0)	// this is the first time
			linesNo = 500;
		else if (lines <= linesNo)
			return;
		
		linesNo = Math.max(lines, 2*linesNo);
		int offset = 16;
		int dist   = 20;
		
		String[] nums = new String[linesNo];
		for (int i = 1; i <= linesNo; i++)
			nums[i - 1] = "" + i;

		Ruler rowView = new Ruler(Ruler.VERTICAL, dist, offset, nums,
				Color.GRAY);
		rowView.setPreferredHeight(offset + linesNo * dist);
		rowView.setPreferredWidth(40 + 10 * (int) Math.max(0,
				Math.log10(linesNo) - 3));
		setRowHeaderView(rowView);
	}

	private int findLastNonWordChar(String text, int index) {
		while (--index >= 0) {
			if (String.valueOf(text.charAt(index)).matches("\\W")) {
				break;
			}
		}
		return index;
	}

	private int findFirstNonWordChar(String text, int index) {
		while (index < text.length()) {
			if (String.valueOf(text.charAt(index)).matches("\\W")) {
				break;
			}
			index++;
		}

		return index;
	}

	private int getNumOfLines() {
		if (textPane == null)
			return 0;
		return textPane.getText().split("\n").length;
	}
	
	
	class ColoredTextPane extends JTextPane {
		
		UndoManagerFix undoManager;
		
		public ColoredTextPane(CodePane cp) {			
			// Has to be marked as transparent so the background is not replaced
			// by super.paintComponent(g);
			//setOpaque(false);
			CodeStyledDocument doc = new CodeStyledDocument();
			doc.setParent(cp);
			//doc.putProperty(PlainDocument.tabSizeAttribute, 1);
			undoManager = new UndoManagerFix();
			undoManager.setLimit(-1); 	// unlimited
			doc.addUndoableEditListener(undoManager);
			doc.putProperty(PlainDocument.tabSizeAttribute, new Integer(2));
			setDocument(doc);
			
			// adjust tab size
	        /*TabStop[] tabs = new TabStop[4];
	        tabs[0] = new TabStop(60, TabStop.ALIGN_RIGHT, TabStop.LEAD_NONE);
	        tabs[1] = new TabStop(100, TabStop.ALIGN_LEFT, TabStop.LEAD_NONE);
	        tabs[2] = new TabStop(200, TabStop.ALIGN_CENTER, TabStop.LEAD_NONE);
	        tabs[3] = new TabStop(300, TabStop.ALIGN_DECIMAL, TabStop.LEAD_NONE);
	        TabSet tabset = new TabSet(tabs);

	        StyleContext sc = StyleContext.getDefaultStyleContext();
	        AttributeSet aset = sc.addAttribute(SimpleAttributeSet.EMPTY,
	        		StyleConstants.TabSet, tabset);
	        setParagraphAttributes(aset, false);*/
		}

		/*//@Override
		protected void paintComponent(Graphics g) {
			System.out.println("text = " + getText());
			if (getText().length() > 0) {
				g.setColor(getBackground());
				g.fillRect(0, 0, getWidth(), getHeight());
				try {
					Rectangle rect = modelToView(getCaretPosition());
					if (rect != null) {
						g.setColor(VERY_LIGHT_BLUE);
						g.fillRect(0, rect.y, getWidth(), rect.height);
					}
				} catch (BadLocationException e) {
					e.printStackTrace();
				}						
			}
			super.paintComponent(g);
		}

		@Override
		public void repaint(long tm, int x, int y, int width, int height) {
			// This forces repaints to repaint the entire TextPane.
			super.repaint(tm, 0, 0, getWidth(), getHeight());
		}*/		
		
		public void setSymbs(String[] symbs) {
			((CodeStyledDocument)getDocument()).setSymbs(symbs);			
		}

		public void setKWrds(String[] kwords) {
			((CodeStyledDocument)getDocument()).setKWrds(kwords);
		}

		public void setLang(Language lang) {
			((CodeStyledDocument)getDocument()).setLang(lang);
		}
		
		public void undo() {
			if (undoManager.canUndo())
				undoManager.undo();	
			else
				System.out.println("cannot undo");
		}

		public void redo() {
			if (undoManager.canRedo())
				undoManager.redo();
			else
				System.out.println("cannot redo");
		}
		
		private class UndoManagerFix extends UndoManager {

		    @Override
		    public synchronized void undo() throws CannotUndoException {
		    	
		    	AbstractDocument.DefaultDocumentEvent event;                
		        do {
		            UndoableEdit edit = editToBeUndone();
		            if (edit instanceof AbstractDocument.DefaultDocumentEvent) {
		                event = (AbstractDocument.DefaultDocumentEvent) edit;
		                if (event.getType() == EventType.CHANGE) {
		                    super.undo();
		                    continue;
		                }
		            }
		            break;
		        } while (canUndo());
		        if (canUndo())
		        	super.undo();
		    }

		    @Override
		    public synchronized void redo() throws CannotRedoException {
		        super.redo();
		        int caretPosition = getCaretPosition();

		        AbstractDocument.DefaultDocumentEvent event;                
		        do {
		            UndoableEdit edit = editToBeRedone();
		            if (edit instanceof AbstractDocument.DefaultDocumentEvent) {
		                event = (AbstractDocument.DefaultDocumentEvent) edit;
		                if (event.getType() == EventType.CHANGE) {
		                    super.redo();
		                    continue;
		                }
		            }
		            break;
		        } while (canRedo());
		        setCaretPosition(caretPosition);
		    }
		}
	}

	class CodeStyledDocument extends DefaultStyledDocument {

		private CodePane 	parent2; 
		private String 		kwRegExp   = "(\\W)*()";
		private String 		symsRegExp = "(\\W)*()";
		private String	 	lineComment;
		private String	 	commentBegin;
		private String	 	commentEnd;
				
		public void insertString(int offset, String str, AttributeSet a)
				throws BadLocationException {

			super.insertString(offset, str, a);

			String text = getText(0, getLength());
			int beg = findBOL(text, offset) + 1;
			int end = findEOL(text, offset + str.length());
			if(end == -1)
				end = text.length();
			
			if (beg < end)	// non empty line
				colorText(text, beg, end);
			
			colorBlockComment(text);
			parent2.stringInserted();
		}

		public void setKWrds(String[] kws) {
			kwRegExp = makeRegExp(kws);
		}

		public void setSymbs(String[] smbs) {
			symsRegExp = makeRegExp(smbs);
		}
		
		public void setLang(Language lang) {
			switch (lang) {
			case Ada:
				lineComment = "--";
				commentBegin = null;
				commentEnd	 = null;
				break;

			case C:
				lineComment = "//";
				commentBegin = "/*";
				commentEnd	 = "*/";
				break;

			default:
				Util.warning("Unknown language: " + lang);
				break;
			}
		}
		
		private String makeRegExp(String[] kws) {
			String re = "(\\W)*(";
			int n = kws.length;
			for (int i = 0; i < n - 1; i++) {
				re += kws[i] + "|";
			}
			if (n > 0) {
				re += kws[n - 1];
			}
			re += ")";
			return re;
		}

		public void remove(int offs, int len) throws BadLocationException {
			super.remove(offs, len);

			String text = getText(0, getLength());
			int begin = findBOL(text, offs) + 1;
			int end = findEOL(text, offs);
			if(end == -1)
				end = text.length();
			
			if(begin < end)	// non-empty line
				colorText(text, begin, end);
			colorBlockComment(text);
		}
		
		public void setParent(CodePane p) {
			parent2 = p;
		}
		
		// Beginning of Line
		private int findBOL(String text, int offs) {
			return text.lastIndexOf("\n", offs);			
		}
		
		// End of Line
		private int findEOL(String text, int offs) {
			return text.indexOf("\n", offs);			
		}

		private void colorText(String text, int from, int to) {
			int before = findLastNonWordChar(text, from);
			if (before < 0)
				before = 0;
			int after = findFirstNonWordChar(text, to);
			int wordL = before;
			int wordR = before;

			while (wordR <= after) {
				if (wordR == after
						|| String.valueOf(text.charAt(wordR)).matches("\\W")) {
					colorTerm(text, wordL, wordR);
					colorComments(text, wordL);
					wordL = wordR;
				}
				wordR++;
			}
		}
		
		/*
		 * colors a single term
		 */
		private void colorTerm(String text, int from, int to) {
			String term = text.substring(from, to); 
			if (isKWord(term)) 
				setAttribute(from, to, GUIAttr.attrBlue);
			else if (isSymbol(term))
				setAttribute(from, to, GUIAttr.attrOrng);			
			else 
				setAttribute(from, to, GUIAttr.attrBlack);
		}

		private boolean isKWord(String s) {
			return s.matches(kwRegExp);			
		}

		private boolean isSymbol(String s) {
			return s.matches(symsRegExp);			
		}
		
		private void colorComments(String text, int off) {
			int begin = findBOL(text, off) + 1;
			int end = findEOL(text, off);
			if (end == -1)
				end = text.length();
			
			if (begin > end)
				return;
			
			String line = text.substring(begin, end);
			int commentInd = line.indexOf(lineComment);
			if (commentInd >= 0) 
				setAttribute(begin + commentInd, end, GUIAttr.attrGreen);
		}
		
		private void colorBlockComment(String text) {
			if (commentBegin == null || commentEnd == null)
				return;
			int begin = text.indexOf(commentBegin);
			while (begin >= 0) {
				int end = text.indexOf(commentEnd, begin + 2);
				if(end == -1)
					end = text.length() - 2;
				setAttribute(begin, end + 2, GUIAttr.attrGreen);
				begin = text.indexOf(commentBegin, end + 2);
			}
		}
		
		private void setAttribute(int from, int to, AttributeSet attr) {
			int len = to - from;
			setCharacterAttributes(from, len, attr, false);
		}
	}	
}
