package taskeditor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import main.Util;

import util.Dialogs;
import util.GUIUtil;


public class ConfigPanel extends JDialog {

	private Config				conf;
	
	private JSpinner   			NumOfTaskSpin;
	private JTextField 			UtilizationText;
	private CustomizedPropJP 	customizedTprop;
	private RandTSGenerator		parent;
	
	public	final	int		WIDTH  = 630;	// width of the frame
	public	final	int		HEIGHT = 420;
	
	// parameters for small, medium, and large tasks, derived from [Stigge 2013]
	public static final String[] S_prop = new String[]{"3", "5", "1", "3", "50", "100", "1", "2", "25", "100"};        
	public static final String[] M_prop = new String[]{"5", "9", "1", "4", "100", "200", "1", "4", "50", "200"};
	public static final String[] L_prop = new String[]{"7", "13", "1", "5", "200", "400", "1", "8", "100", "400"}; 

    public ConfigPanel(Config cfg, RandTSGenerator rng, Component c) {
    	conf   = new Config(cfg);
    	parent = rng;
        initUI(c);
    }

    public final void initUI(final Component c) {
        JPanel basic = new JPanel();
        basic.setLayout(new BoxLayout(basic, BoxLayout.Y_AXIS));
        add(basic);

        /*** Top Panel: Icon ***/
        JPanel topPanel = new JPanel(new BorderLayout(0, 0));
        topPanel.setMaximumSize(new Dimension(WIDTH, 0));
        JLabel hint = new JLabel("Configuration Panel ");
        hint.setFont( new Font("Arial", Font.BOLD, 13) );
        hint.setBorder(BorderFactory.createEmptyBorder(0, 15, 0, 0));
        topPanel.add(hint);

        ImageIcon icon = Util.getIcon("/conf.png");
        JLabel label = new JLabel(icon);
        label.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        topPanel.add(label, BorderLayout.EAST);

        JSeparator separator = new JSeparator();
        separator.setForeground(Color.gray);

        topPanel.add(separator, BorderLayout.SOUTH);
        basic.add(topPanel);

        /*** Main (Tabbed) Panel: Configuration ***/
        basic.add(createTabbedPane());

        /*** Buttom Panel: OK / Cancel ***/        
        JPanel bottom = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        JButton ok = new JButton("OK");
        ok.setMnemonic(KeyEvent.VK_N);
        ok.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent arg0) {	
				conf.custom_param = customizedTprop.getParams();				
				if (conf.numOfTasksSelected == false && conf.utilizationSelected == false)
					Dialogs.showErrorMsg("No task set criteria is selected.");
				else {
					try {
						conf.setNumOfTasks( (Integer) NumOfTaskSpin.getValue() );
						conf.utilization = Double.parseDouble(UtilizationText.getText());
					} catch (Exception e) {
						Dialogs.showErrorMsg("Number format error.");
					}
					parent.setCfg(conf);
					close();				
				}				
			}
		});
        JButton close = new JButton("Calcel");
        close.setMnemonic(KeyEvent.VK_C);
        close.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent arg0) {				
				close();
			}
		});
        
        bottom.add(ok);
        bottom.add(close);
        basic.add(bottom);

        bottom.setMaximumSize(new Dimension(450, 0));

        /*** Usual Settings ***/
        setTitle("Configuration");
        setSize(new Dimension(WIDTH, HEIGHT));
        setResizable(false);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setLocationRelativeTo(c);
    }

    private JTabbedPane createTabbedPane()  {
    	/********************** Random Task Settings **************************/
        JPanel rndjp = createRandTSJP();        
        
        /*** Create Tabbed Pane ***/
        JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane .setPreferredSize(new Dimension(450, 200));
		tabbedPane.addTab ("Random Task Generation", rndjp);
		
		tabbedPane.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
		tabbedPane.setFont(new Font("", Font.PLAIN, 12));
		return tabbedPane;
    }
    
	private JPanel createRandTSJP() {
		
        /*** creating customized task set properties ***/
        customizedTprop = new CustomizedPropJP();        
        
        /*** creating radio buttons ***/
		JRadioButton SButton = new JRadioButton("Small Tasks");
	    JRadioButton MButton = new JRadioButton("Medium Tasks");
	    JRadioButton LButton = new JRadioButton("Large Tasks");
	    JRadioButton PButton = new JRadioButton("Periodic");
	    JRadioButton CButton = new JRadioButton("Customized");
		SButton.setActionCommand("S");
	    MButton.setActionCommand("M");
	    LButton.setActionCommand("L");
	    PButton.setActionCommand("P");
	    CButton.setActionCommand("C");

	    if(conf.task_type.equalsIgnoreCase("S"))
			SButton.setSelected(true);
	    else if(conf.task_type.equalsIgnoreCase("M"))
	    	MButton.setSelected(true);
	    else if(conf.task_type.equalsIgnoreCase("L"))
	    	LButton.setSelected(true);
	    else if(conf.task_type.equalsIgnoreCase("P"))
	    	PButton.setSelected(true);
	    else if(conf.task_type.equalsIgnoreCase("C"))
	    	CButton.setSelected(true);
	    
	    if(conf.task_type.equalsIgnoreCase("C"))
	    	customizedTprop.editable(true);
	    else if(conf.task_type.equalsIgnoreCase("P"))
	    	customizedTprop.editable_periodic(true);	    
	    else
	    	customizedTprop.editable(false);	    
	    	
	    //Group the radio buttons.
	    ButtonGroup group = new ButtonGroup();
	    group.add(SButton);
	    group.add(MButton);
	    group.add(LButton);
	    group.add(PButton);
	    group.add(CButton);

	    //Register a listener for the radio buttons.
	    ActionListener taskListener = new ActionListener(){
            public void actionPerformed(ActionEvent evt) {
            	conf.task_type = evt.getActionCommand();
            	if (conf.task_type.equalsIgnoreCase("S"))
            		conf.custom_param = S_prop;
        	    else if(conf.task_type.equalsIgnoreCase("M"))
        	    	conf.custom_param = M_prop;
        	    else if(conf.task_type.equalsIgnoreCase("L"))
        	    	conf.custom_param = L_prop;
            	
            	if (conf.task_type.equalsIgnoreCase("C"))  // Customized Task Set 
            		customizedTprop.editable(true);
            	else if (conf.task_type.equalsIgnoreCase("P"))  // Customized Periodic Task Set 
            		customizedTprop.editable_periodic(true);
            	else {
            		customizedTprop.editable(false);
            		customizedTprop.loadData();
            	}            		
            }};
        SButton.addActionListener(taskListener);
        MButton.addActionListener(taskListener);
        LButton.addActionListener(taskListener);
        PButton.addActionListener(taskListener);
        CButton.addActionListener(taskListener);
    
        // Put the radio buttons in a column in a panel.
        JPanel radioPanel = new JPanel(new GridLayout(5, 1));
        radioPanel.add(SButton);
        radioPanel.add(MButton);
        radioPanel.add(LButton);
        // radioPanel.add(new JSeparator(JSeparator.HORIZONTAL));
        radioPanel.add(PButton);
        radioPanel.add(CButton);
        radioPanel.setBorder(GUIUtil.getBlackTitleBorder("Task Type"));
                 
        JPanel voljp = createCriteriaPanel();
        
        JPanel rndjp = new JPanel();
        rndjp.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;        
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 0.3;
        
        c.ipadx = 25;
        rndjp.add(radioPanel, c);
        
        c.gridx = 1;
        c.gridy = 0;
        c.weightx = 0.5;
        c.ipadx = 0;
        c.insets = new Insets(0, 4, 0, 4);
        rndjp.add(customizedTprop, c);
        
        c.gridx = 2;
        c.gridy = 0;
        c.weightx = 0.5;
        c.ipadx = 0;
        c.insets = new Insets(0, 0, 0, 0);
        rndjp.add(voljp, c);
        
        rndjp.setBorder(BorderFactory.createEmptyBorder(20,10,20,10));
        
        return rndjp;
	}
	
	private JPanel createCriteriaPanel() {
		UtilizationText = new JTextField("" + conf.utilization);
		NumOfTaskSpin = GUIUtil.cearteSpinner(conf.numOfTasks, 1, 999);
		GUIUtil.setSize(NumOfTaskSpin, 45, 0);
		
		final JCheckBox UButton = new JCheckBox("Utilization: ");
		final JCheckBox NButton = new JCheckBox("Tasks:");
		UButton.setSelected(conf.utilizationSelected);
		NButton.setSelected(conf.numOfTasksSelected);
		
		volDecorate();
		
		ItemListener volumeListener = new ItemListener() {
			 /** Listens to the check boxes. */
		    public void itemStateChanged(ItemEvent e) {
		        Object source = e.getItemSelectable();
		 
		        boolean selected = true;
		        if (e.getStateChange() == ItemEvent.DESELECTED) {
		        	selected = false;
		        }
		        
		        if (source == UButton) {
		        	conf.utilizationSelected = selected;
		        } else if (source == NButton) {
		        	conf.numOfTasksSelected = selected;
		        }
		 		        
		        volDecorate();
		    }
        };

		UButton.addItemListener(volumeListener);
		NButton.addItemListener(volumeListener); 
			    
        JPanel jp = new JPanel(new GridLayout(2, 2, 0, 2));
        jp.add(NButton);
        jp.add(NumOfTaskSpin);
        jp.add(UButton);
        jp.add(UtilizationText);
        
        JPanel voljp = new JPanel(new FlowLayout());
        voljp.setBorder(GUIUtil.getBlackTitleBorder("Task Set Criteria")); 	// BorderFactory.createEmptyBorder(0,20,0,0));
        voljp.add(jp);
        
        return voljp;
	}

	protected void volDecorate() {
		GUIUtil.disable(NumOfTaskSpin);
		GUIUtil.disable(UtilizationText);
		
		if (conf.numOfTasksSelected)
			GUIUtil.enable(NumOfTaskSpin);
		
		if (conf.utilizationSelected)
			GUIUtil.enable(UtilizationText);		
	}

	public static Border getGrayTitleBorder(String title) {
		TitledBorder border = BorderFactory.createTitledBorder(title);
		border.setTitleColor(Color.gray);
		return border;
	}

	public void appear() {
		/*int res = Util.showOkCancelPanel(p, title);		
        if (res == JOptionPane.OK_OPTION) 
        	return p.getSettings();
        else
        	return cfg;*/
		// Util.get
		SwingUtilities.invokeLater(new Runnable() {
            public void run() {
            	// EditorConfig ex = new EditorConfig();
                setVisible(true);
            }
        });		
	}
	
	class CustomizedPropJP extends JPanel {
		private		final	String 		TSParam_TITLE = "Task Set Parameters";
		private		final	JTextField 	vMin = new JTextField();
		private		final	JTextField 	vMax = new JTextField();
		private		final	JTextField 	fMin = new JTextField();
		private		final	JTextField 	fMax = new JTextField();
		private		final	JTextField 	pMin = new JTextField();
		private		final	JTextField 	pMax = new JTextField();
		private		final	JTextField 	eMin = new JTextField();
		private		final	JTextField 	eMax = new JTextField();
		private		final	JTextField 	dMin = new JTextField();
		private		final	JTextField 	dMax = new JTextField();
		
		public void editable(boolean enabled) {
			
			Color color = enabled ? Color.BLACK : Color.GRAY;
			setBorder(GUIUtil.getTitleBorder(TSParam_TITLE, color));	
			
			Component[] columns = this.getComponents();
			for (int i = 0; i < columns.length; i++) {
				Component[] components = ((JPanel)columns[i]).getComponents();
				for (int j = 0; j < components.length; j++) {				
			        if(components[j].getClass().getName().toString().equals("javax.swing.JTextField")){
			            components[j].setEnabled(enabled);			            
			        }
			        components[j].setForeground(color);
				}	             
		    }
		}
		
		
		public void editable_periodic(boolean b) {
			editable(true);
			vMin.setText("1");
			vMax.setText("1");
			fMin.setText("1");
			fMax.setText("1");
			vMin.setEnabled(false);
			vMax.setEnabled(false);
			fMin.setEnabled(false);
			fMax.setEnabled(false);
			// pMin
		}


		public String[] getParams() {
			String[] res = new String[]{vMin.getText(), vMax.getText(), fMin.getText(), fMax.getText(), 
					pMin.getText(), pMax.getText(), eMin.getText(), eMax.getText(), dMin.getText(), dMax.getText()};
			return res;
		}


		public CustomizedPropJP() {
			// setLayout(new GridLayout(6, 3, 20, 2));	
			setBorder(GUIUtil.getBlackTitleBorder("Task Set Parameters"));
			setLayout(new GridBagLayout());
			loadData();				
			
			JPanel col1 = new JPanel(new GridLayout(6, 1, 0, 2)); 
			JPanel col2 = new JPanel(new GridLayout(6, 1, 0, 2));
			JPanel col3 = new JPanel(new GridLayout(6, 1, 0, 2));
			col3.setPreferredSize(new Dimension(100,0));
			
			col1.add(new JPanel());
			col1.add(new JLabel(" Vertices"));
			col1.add(new JLabel(" Fan-out"));
			col1.add(new JLabel(" Inter-release"));
			col1.add(new JLabel(" WCET"));
			col1.add(new JLabel(" Deadline"));
				        
			col2.add(new JLabel("Min"));
			col2.add(vMin);
			col2.add(fMin);
			col2.add(pMin);
	        col2.add(eMin);
	        col2.add(dMin);
	        
			col3.add(new JLabel("Max"));
	        col3.add(vMax);
	        // customjp0.add(new JLabel("<html>Branching<br> degree</html>"));
	        col3.add(fMax);
	        col3.add(pMax);
	        col3.add(eMax);
	        col3.add(dMax);
	        
	        GridBagConstraints c = new GridBagConstraints();
	        c.fill = GridBagConstraints.BOTH;
	        c.insets = new Insets(0, 2, 4, 2);
	        
	        c.gridx = 0;
	        c.gridy = 0;
	        add(col1, c);
	        
	        c.ipadx = 20;
	        c.gridx = 1;
	        add(col2, c);
	        
	        c.ipadx = 20;
	        c.gridx = 2;
	        add(col3, c);	        
		}

		private void loadData() {
			vMin.setText(conf.custom_param[0]);
			vMax.setText(conf.custom_param[1]);
			fMin.setText(conf.custom_param[2]);
			fMax.setText(conf.custom_param[3]);
			pMin.setText(conf.custom_param[4]);
			pMax.setText(conf.custom_param[5]);
			eMin.setText(conf.custom_param[6]);
			eMax.setText(conf.custom_param[7]);
			dMin.setText(conf.custom_param[8]);
			dMax.setText(conf.custom_param[9]);					
		}		
	}
		
	class BrowseActListener implements ActionListener {
		JTextField related_text;
		String default_path;
		
		public BrowseActListener(JTextField text, String path) {
			related_text = text;
			default_path = path;
		}
		
		public void actionPerformed(ActionEvent e) {
			JFileChooser fileDialog = Util.getFileChooser("Select", default_path, ".*");
            int option = fileDialog.showDialog(null, "OK");
            if (option != JFileChooser.APPROVE_OPTION)
                return;      // User canceled or clicked the dialogue close box.
            File selectedFile = fileDialog.getSelectedFile();
            related_text.setText( selectedFile.getPath() );	
            related_text.setFont(new Font("", Font.PLAIN, 12));            
		}		
	}
	
	/*
	 * Closes the panel
	 */
	public void close() {
		setVisible(false);
		dispose();
	}
}

class Config {
	public String 		task_type = "S";  // default: Small tasks for random task set generation
	public String[] 	custom_param;	// list of task parameters, including WCET, deadline, and Inter-Release intervals
	public double 		utilization = 0.5;			// default values
	public int 			numOfTasks = 8;
	public boolean		utilizationSelected = false;	
	public boolean		numOfTasksSelected  = true;		
	
	public Config() {
		setParam();
	}
	
	public Config(Config c) {
		task_type 			= c.task_type;
		custom_param 		= c.custom_param;
		utilization 		= c.utilization;
		numOfTasks 			= c.numOfTasks;
		utilizationSelected = c.utilizationSelected;
		numOfTasksSelected  = c.numOfTasksSelected;
	}

	private void setParam() {
    	if(task_type.equalsIgnoreCase("S"))
    		custom_param = ConfigPanel.S_prop;
    	else if(task_type.equalsIgnoreCase("M"))
    		custom_param = ConfigPanel.M_prop;
    	else if(task_type.equalsIgnoreCase("L"))
    		custom_param = ConfigPanel.L_prop;
	}	
	

   	public String getTaskType() {		
		return task_type;
	}
   	
   	public String getTaskCriterion() {
   		String res = "U"; // default is utilization-based
		if (utilizationSelected && numOfTasksSelected)
			res = "B";	// denoting both
		else if (utilizationSelected)
			res = "U";
		else if (numOfTasksSelected)
			res = "N";
		return res;
	}
   	
	public String getParamStr() {
		String res = "";
		int n = custom_param.length;
		for (int i = 0; i < n - 1; i++)
			res += custom_param[i] + ",";
		res += custom_param[n - 1];
		return  res;
	}

	public void setNumOfTasks(int numOfTasks) {
		this.numOfTasks = numOfTasks;
	}
}