package taskeditor;
import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import main.JobType;
import main.Task;
import main.Util;

import taskeditor.changetrace.GroupChange;
import taskeditor.changetrace.TaskChange;
import util.Dialogs;
import util.GUIUtil;


import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.LinkedList;
import java.util.ListIterator;

/**
 * This component is placed on the left side of GUI
 * showing the tasks properties, as well as a feature
 * for adding a new task
 */
public class TaskPanel extends JPanel {

    private	EditorTab	editorTab;
    private JPanel      tablePanel;
    private	JTable		taskSetTable;	// the table which shows task list
    private Object[][] 	taskSetData;	// to be used in tasks table 
    private JButton     newJobBut;
    private JComboBox<String>   taskComboBox;
    private LinkedList<Task> 	clipBoard;
        
    private final		int		NumOfTaskSetTableCol = 2; // contains: task name, task priority 

    public TaskPanel(EditorTab editor){
        this.editorTab = editor;
        
        setLayout(new BorderLayout());
        JPanel tasksPropPanel = createTaskPropertiesPanel(); 
        JPanel taskSetPanel = createTaskSetPanel();
        
        JSplitPane verticalSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        verticalSplit.setLeftComponent(taskSetPanel);		// set left (or above) component
        verticalSplit.setRightComponent(tasksPropPanel);
		Util.decorateSplit(verticalSplit, 200);
		verticalSplit.setResizeWeight(0.5);
        add(verticalSplit, BorderLayout.CENTER);
        clipBoard = new LinkedList<Task>();
    }
    
	private JPanel createTaskPropertiesPanel() {
    	
    	createTablePanel();
        JPanel comboPanel =  createTaskComboPanel();
        Object[][] data = {{"", "", "", ""} };
        addTableData(data);        
        JPanel buttonPanel     = createButtonPanel();   // button for new task 
        JPanel body = new JPanel();
        body.setBorder(GUIUtil.getBlackTitleBorder(""));
    	body.setAlignmentY(1f);
    	body.setLayout(new BorderLayout());        
    	// body.add(comboPanel, BorderLayout.NORTH);
    	body.add(tablePanel, BorderLayout.CENTER);
    	body.add(buttonPanel, BorderLayout.SOUTH);
    	
    	JPanel panel = new JPanel(new BorderLayout());
    	panel.add(comboPanel, BorderLayout.NORTH);
    	panel.add(body, BorderLayout.CENTER);
    	return panel;
	}
	
	private JPanel createTaskSetPanel() {
		final String RENAME       = "Rename";
		final String SET_PRIORITY = "Reprioritize";
		final String COPY_TASK 	  = "Copy";
		final String PASTE_TASK   = "Paste";
		final String COPY_TASKS   = "Copy";
		final String DELETE_TASK  = "Delete";
		final String DECLAR 	  = "Declarations";
		final String []singleItems = new String[]{RENAME, SET_PRIORITY, DECLAR, COPY_TASK, PASTE_TASK, DELETE_TASK};
		final String []multplItems = new String[]{RENAME, SET_PRIORITY, DECLAR, COPY_TASKS, PASTE_TASK, DELETE_TASK};
		
    	JPanel panel = new JPanel(new BorderLayout());
    	
        JLabel title = GUIUtil.createTitle("Tasks");

        if(taskSetData == null)
        	taskSetData = emptyTaskSetData();
        taskSetTable = new JTable();
        taskSetTable.setPreferredScrollableViewportSize(new Dimension(70, 70));
        taskSetTable.setFillsViewportHeight(true);
        taskSetTable.setAutoCreateRowSorter(true);
        taskSetTable.setModel(new TaskSetTableModel());        
        TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(taskSetTable.getModel());
        sorter.setComparator(0, Util.getStrIntComparator());
        sorter.setComparator(1, Util.getIntComparator());
        taskSetTable.setRowSorter(sorter);
        
        taskSetTable.getColumnModel().getColumn(0).setPreferredWidth(50);
        taskSetTable.getColumnModel().getColumn(1).setPreferredWidth(20);
        GUIUtil.alignHeader(taskSetTable);
        GUIUtil.alignCells(taskSetTable, 2);
        
        //Create the scroll pane and add the table to it.
        JScrollPane body = new JScrollPane(taskSetTable);
        
        taskSetTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent event) {
            	if (taskSetTable.getSelectedRow() > -1) {
                	int row = taskSetTable.getSelectedRow();
          	      	String task_name = (String)taskSetTable.getValueAt(row, 0);
          	      	int index = editorTab.getTaskIndex(task_name);
          	      	taskComboBox.setSelectedIndex(index);
    	    	  	newTaskSelected(index);       	
                }
            }
        });
        
        taskSetTable.addMouseListener(new MouseAdapter() {
        	  public void mouseClicked(MouseEvent e) {
        		  JTable target = (JTable)e.getSource();
        	      
        	      if (e.getClickCount() == 1 && GUIUtil.leftClick(e)) {
        	    	  /*String task_name = (String)target.getModel().getValueAt(row, 0);
            	      int index = manager.getTaskIndex(task_name);            	      
        	    	  taskComboBox.setSelectedIndex(index);
        	    	  newTaskSelected(index);*/
        	      } else if ( Util.isRClick(e) ) {
        	    	  int []rows = taskSetTable.getSelectedRows();
        	    	  int r = taskSetTable.rowAtPoint(e.getPoint());
        	    	  JComponent c = (JComponent) e.getSource();
	                  Point p = e.getPoint();
	                  int menuX = p.x;
                      int menuY = p.y;     
                      JPopupMenu menu;
        	          if (r < 0 || r >= taskSetTable.getRowCount()) {
        	        	  taskSetTable.clearSelection();
        	        	  return;
        	          }
    	        	  if (Util.contains(rows, r) && rows.length > 1)
    	        		  menu = createPopupMenu(multplItems);
    	        	  else {
    	        		  taskSetTable.setRowSelectionInterval(r, r);
    	        		  menu = createPopupMenu(singleItems);
    	        	  }
    	        	  menu.show(c, menuX, menuY);    	                          
        	      }
        	  }

			public JPopupMenu createPopupMenu(final String []cmds) {
				class RowListener implements ActionListener {
					public void actionPerformed(ActionEvent e) {
						
			        	String []names = getSelectedTasksNames();
			        	
			        	String cmd = e.getActionCommand();
			        	if (cmd.equals(RENAME))
		        			editCol(0);
			        	else if (cmd.equals(SET_PRIORITY))
			        		editCol(1);
			        	else if (cmd.equals(DELETE_TASK) )
			        		deleteTasks(names);   		
		        		else if (isCopy(cmd) )
		        			copyTasks(names); 
		        		else if (isPaste(cmd) )
		        			pasteTasks(); 
		        		else if (cmd.equals(DECLAR) )
		        			editorTab.getTaskVarFromUser(); 
			        	revalidate();
						repaint();
		    		}

					private void editCol(int col) {
						int row = taskSetTable.getSelectedRow();
					    taskSetTable.editCellAt(row, col);
					    taskSetTable.requestFocusInWindow();
					}
					
					private boolean isCopy(String cmd) {
						if (cmd.equals(COPY_TASK))
							return true;
						if (cmd.equals(COPY_TASKS))
							return true;
						return false;
					}
					
					private boolean isPaste(String cmd) {
						return cmd.equals(PASTE_TASK);
					}
				}
	            JPopupMenu popupMenu = new JPopupMenu();
	            for (int i = 0; i < cmds.length; i++) {
	            	JMenuItem mi = new JMenuItem(cmds[i]);
		            mi.addActionListener(new RowListener());
		            popupMenu.add(mi); 
		            if (i == 1 || i == 2) 
		            	popupMenu.addSeparator();
	            }
	            return popupMenu;            
	          }

        	});
        
        panel.add(title, BorderLayout.NORTH);
    	panel.add(body, BorderLayout.CENTER);
    	return panel;
	}
	
	private void deleteTasks(String[] task_names) {
	    GroupChange ch = new GroupChange();
		for (String t_name : task_names) {
			ch.add(getDeleteChange(t_name));
			editorTab.deleteTask(t_name);
		}
		
		editorTab.trackGlobalChange(ch);
		showTaskSet();
		// taskComboBox.setSelectedIndex( 0 );   
	}
	
	// track deleting a task
	private TaskChange getDeleteChange(String name) {	
		int ind = editorTab.getTaskIndex(name);
		Task t = editorTab.getTask(ind);
    	return t.getDeleteChange();
	}
    
    private void trackCreate(Task t) {
    	TaskChange ch = t.getCreateChange();
		editorTab.trackGlobalChange(ch);    		
	}
    	
	private void copyTasks(String[] task_names) {
		while (!clipBoard.isEmpty()) {
			clipBoard.removeFirst();
	    }
		for (String t_name : task_names) {
			int ind = editorTab.getTaskIndex(t_name);
			Task t  = editorTab.getTask(ind);			
			clipBoard.add(t.cloneMe());
		}
	}
	
	private void pasteTasks() {
		for (Task t : clipBoard) {
			Task copy = t.cloneMe();
			String name = copy.getName() + "_Copy"; 
			name = Util.getUniqueName(name, tasksName());
			copy.setName(name);
			editorTab.addTask(copy);
			trackCreate(copy);
		}
		showTaskSet();
	}
	
	private String[] getTasksNames(int []rows) {
		 String []task_names = new String [rows.length]; 
    	for(int i = 0; i < rows.length; i++)
    		task_names[i] = (String)taskSetTable.getValueAt(rows[i], 0);    	      	
    	return task_names;
	}
	
	private String[] tasksName() {
		return tasksName(editorTab.tasks());
	}

    private String[] tasksName(Task[] tasks) {
    	return Util.extractNames(tasks);
	}
	
	/*
	 * Called by two methods:
	 * 1. When a new index is selected in the comboBox
	 * 2. When a new raw is selected in the task set table (the table at the bottom)
	 */
	private void newTaskSelected(int index) {
		Task task = editorTab.getTask(index);
        if (task != null) {
        	TaskPanel.this.revalidate();
            TaskPanel.this.repaint();
            
            fillInTable(task);
            editorTab.drawTaskGraph(task);
        }
	}
	

    private Object[][] emptyTaskSetData() {
    	Object[][] data = new String[1][NumOfTaskSetTableCol];
    	for (int i = 0; i < NumOfTaskSetTableCol; i++)
    		data[0][i] = "";
		return data;
	}


	private JPanel createTaskComboPanel() {
        String[] names = new String[]{""};
        taskComboBox = new JComboBox<String>(names);        
        taskComboBox.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent evt) {
            	int index = getSelectedTaskIndex();
            	// taskSetTable.setRowSelectionInterval(0, 0);
                newTaskSelected(index);
            }});

        JPanel panel = new JPanel(new BorderLayout());
        JLabel label = new JLabel(" Select Task:  ");
        // label.setFont(new Font("", Font.PLAIN, 11));
        
        panel.setBorder(new TitledBorder(new EtchedBorder(), ""));
        panel.add(label, BorderLayout.WEST);
        panel.add(taskComboBox, BorderLayout.CENTER);
        // add(jp1, BorderLayout.NORTH);
        
        return panel;
    }
    
    public static GridBagConstraints getConstraints(int aY, int aX){
        GridBagConstraints result = new GridBagConstraints();
        result.gridy = aY;
        result.gridx = aX;
        result.anchor = GridBagConstraints.WEST;
        final int ONE_SPACE = 5;
        result.insets = new Insets(0,0,0,ONE_SPACE);
        return result;
      }

    private void createTablePanel() {
    	tablePanel = new JPanel(new BorderLayout());
    	
        // tablePanel.setBorder(GUIUtil.getBlackTitleBorder("Jobs Properties"));
//        tablePanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));	
    }
    
    /*
     * The function creates a new JTable, adds the data to this JTable and adds it to the Table Panel
     */
    private void addTableData(Object[][] data) {
        
        JTable jTable = createJobTable(data);
        // tablePanel.add(jTable.getTableHeader(), BorderLayout.PAGE_START);
        JScrollPane body = new JScrollPane(jTable);
        //jtScroll.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
        JLabel title = GUIUtil.createTitle("Jobs Parameters");
    	tablePanel.add(title, BorderLayout.NORTH);    	
        tablePanel.add(body, BorderLayout.CENTER);
    }
    
    private JTable createJobTable(Object[][] data) {
    	
    	JTable jobTable = new JTable() {  
	    public boolean isCellEditable(int row,int column) {  
	        if (column == 0) return false;   // to make the first column (job number) non-editable
	        return true;  
	      }  
	    }; 
	    
	    jobTable.setPreferredScrollableViewportSize(new Dimension(70, 70));
	    jobTable.setFillsViewportHeight(true);
	    jobTable.setAutoCreateRowSorter(true);

	    jobTable.setModel(new TaskTableModel(data));        
	    
	    TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(jobTable.getModel());;
		sorter.setComparator(0, Util.getIntComparator());
		sorter.setComparator(1, Util.getStrIntComparator());
		sorter.setComparator(2, Util.getIntComparator());
		sorter.setComparator(3, Util.getIntComparator());
		jobTable.setRowSorter(sorter);
				
	    jobTable.getColumnModel().getColumn(0).setPreferredWidth(20);
	    jobTable.getColumnModel().getColumn(1).setPreferredWidth(55);
		GUIUtil.alignHeader(jobTable);        
		GUIUtil.alignCells(jobTable, 4);
		
		/*
	    jobTable.getModel().addTableModelListener( new TaskTableModelListender(this) );*/

		return jobTable;
	}


	private JPanel createButtonPanel() {
        JButton newTaskBut = new JButton("New Task");
        newJobBut = new JButton("New Job");
        newJobBut.setEnabled(false);
        newTaskBut.addActionListener(new NewTaskActionListner());
        newJobBut. addActionListener(Util.createUnavailableFeatureListener());
        newTaskBut.setToolTipText("Create new task");        

        JPanel buttom = new JPanel(new FlowLayout());
        buttom.add(newTaskBut);
        buttom.add(newJobBut);
        return buttom;
    }
	
    /*
     * please note that this function resets all things and the selected task is set to the default
     */
    public void showTaskSet() {
    	String[] names = tasksName();
        taskComboBox.removeAllItems();
        for (int i = 0; i < names.length; i++) {
            taskComboBox.addItem(names[i]);
        }
        LinkedList<Task> taskList = editorTab.taskList();
    	updateTaskTable(taskList);
        refreshTaskSetTable(taskList);
        
        if(names.length == 0)
        	editorTab.emptyPane();
        
        revalidate();
        repaint();
    }
    
	public void refreshTaskSetTable(LinkedList<Task> taskSet) {
    	int taskNo = taskSet.size();
    	
    	String[][] newTaskSetData = new String[taskNo][NumOfTaskSetTableCol];
        for (int i = 0; i < taskNo; i++) {
        	Task task = taskSet.get(i);
        	newTaskSetData[i][0] = task.getName();
        	// newTaskSetData[i][1] = "" + task.getJobs().size();
        	newTaskSetData[i][1] = "" + task.getPriority();
        }
        taskSetData = newTaskSetData;
        taskSetTable.setModel(new TaskSetTableModel());
        
        taskSetTable.getColumnModel().getColumn(0).setPreferredWidth(60);
        taskSetTable.getColumnModel().getColumn(1).setPreferredWidth(20);
        
        taskSetTable.revalidate();
        taskSetTable.repaint();
    }

    
    public void updateTaskTable(LinkedList<Task> taskSet) {
    	int taskInd = getSelectedTaskIndex();
    	Task task = taskInd == -1 ? null : taskSet.get( taskInd );
    	fillInTable(task);
	}
    
	public int getSelectedTaskIndex() {
		return taskComboBox.getSelectedIndex();  // To Do
    }
	
	private String[] getSelectedTasksNames() {
		 int []rows = taskSetTable.getSelectedRows();
		 return getTasksNames(rows);
	}
	
	public String[] getSelectedTasksName() {
		String[] names = getSelectedTasksNames();
		if(names.length > 0)
			return names;
		else
		{
			String name = (String)(taskComboBox.getSelectedItem());  
			return new String[]{name}; 
		}			
    }

    // Fill in the table according to the loaded task properties
    public void fillInTable(Task task) {
    	tablePanel.removeAll();
    	if(task == null)
    	{
    		Object[][] data = {{"", "", "", ""} };            
    		addTableData(data);    	
        	return;
    	}
    	
    	ListIterator<JobType> jItr = task.jobIterator();
    	Object[][] data = new Object[task.jobNo()][4];
    	int i = 0;
    	while(jItr.hasNext())
		{
			JobType job = jItr.next();    
            data[i][0] = i + 1;
            data[i][1] = job.getName();
            data[i][2] = job.WCET + "";   // for consistency, convert to string
            data[i][3] = job.deadline + ""; // for consistency, convert to string
            i++;
        }
        addTableData(data);

//        JTable table = new JTable(data, columnNames);
//        table.setPreferredScrollableViewportSize(new Dimension(500, 70));
//
//        jobTable.removeAll();
//        jobTable.add(table.getTableHeader(), BorderLayout.PAGE_START);
//        jobTable.add(table, BorderLayout.CENTER);
//        jobTable.repaint();
//        jobTable.revalidate();
    }
    
    /*
     * For creating new task
     * This listener is associated to the New Task Button 
     */
    class NewTaskActionListner implements ActionListener{
        public void actionPerformed(ActionEvent evt) {
        	String name = Dialogs.getTextFromUser("Task Name:", ""); 
        	if (editorTab.getTaskIndex(name) >= 0) {	// task with this name already exists        	
        		Dialogs.showErrorMsg("Task with this name already exists.\r\n" +
        				" Tasks must have unique names.");
        		actionPerformed(evt);
        	} else if (name != null) {
        		Task t = editorTab.createNewTask(name);
        		trackCreate(t);
        		refreshPanel();
    		}
        }
        
        private void refreshPanel() {
    		showTaskSet();
    		int ind = taskComboBox.getItemCount()-1; 
    		taskComboBox.setSelectedIndex( ind );
		}
    }
    
    class TaskTableModel extends AbstractTableModel {
        private final String[] columnNames = {"#", "Name", "WCET", "Deadline"};
        private Object[][] data; 	
        		
        public TaskTableModel(Object[][] myData) {
        	data = myData;
        }
        
        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {
            return data.length;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public Object getValueAt(int row, int col) {
        	return data[row][col];
        }

        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        /*
         * The first column (job number) is not editable 
         */
        public boolean isCellEditable(int row, int col) {
            if (col < 1) 
               return false;
            return true;
        }

        public void setValueAt(Object value, int row, int col) {
        	Object old_value = data[row][col]; 
        	data[row][col] = value;
        	boolean success = handleTableDataChange(row, col, this);
        	if(success == false)
        		data[row][col] = old_value;	// revert the change
        }
    }

    class TaskSetTableModel extends AbstractTableModel {
        private final String[] columnNames = {"Task Name",
                                        // "# of Jobs",
                                        "Priority"};

        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {
            return taskSetData.length;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public Object getValueAt(int row, int col) {
        	if(taskSetData == null)
        		return 0;
        	if(taskSetData.length == 0)
        		return 0;
        	return taskSetData[row][col];
        }

        /*
         * JTable uses this method to determine the default renderer/
         * editor for each cell.  If we didn't implement this method,
         * then the last column would contain text ("true"/"false"),
         * rather than a check box.
         */
        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }

        /*
         * Only the last column (priority) is editable 
         */
        public boolean isCellEditable(int row, int col) {
            //Note that the data/cell address is constant,
            //no matter where the cell appears on screen.
            // if (col < NumOfTaskSetTableCol - 1) 
               return true;            
        }

        /*
         * Don't need to implement this method unless your table's
         * data can change.
         */
        public void setValueAt(Object value, int row, int col) {
        	String taskName = (String)taskSetData[row][0];	// if the user is modifying the name, this is the previous name                        	
            if (col == 0) {   // task name changed            
            	String newName = (String)value;
            	String oldName = (String)taskSetData[row][col];
            	String syntaxError = Util.checkWord(newName);
            	if (syntaxError != null) {
            		Dialogs.showErrorMsg(Util.checkWord(newName));
            		return;
            	}            	
            	int taskId = editorTab.getTaskIndex(newName);
            	if (newName.equals(oldName))
            		;	// do nothing
            	else if (taskId >= 0) {
            		Dialogs.showErrorMsg("Cannot change the name since " +
            				"there exists another task with this name.");
            		System.out.println();
            	} else {
            		taskSetData[row][col] = value;    	            
            		trackChange();
            		editorTab.setTaskName(taskName, newName);            		
            		showTaskSet();            		
            	}
            }
            else if (col == 1) {  // task priority changed            
            	try {
            		int priority = Integer.parseInt((String)value);
            		taskSetData[row][col] = value;
            		trackChange();            		
            		editorTab.setPriority(taskName, priority);
            	} catch(Exception e) {
            		Util.log(e);
            		System.out.println("Wrong number format: " + value);
            		Dialogs.showErrorMsg("Priority must be an integer number.");
            	}
            }
            // Normally, one should call fireTableCellUpdated() when 
            // a value is changed.  However, doing so in this demo
            // causes a problem with TableSorter.  The tableChanged()
            // call on TableSorter that results from calling
            // fireTableCellUpdated() causes the indices to be regenerated
            // when they shouldn't be.  Ideally, TableSorter should be
            // given a more intelligent tableChanged() implementation,
            // and then the following line can be uncommented.
            // fireTableCellUpdated(row, col);
        }

        private void trackChange() {
        	Task t = editorTab.getSelectedTask();
        	editorTab.trackGlobalChange(t.getModifyChange());
		}
    }
    
    /*
     * The function is called whenever the user modifies a (task) table entry 
     * We have to apply the change to both job object and the drawingPane
     */
	private boolean handleTableDataChange(int row, int column, TableModel model) {
		if (editorTab.hasActiveTaskSet() == false)	// the user has clicked an empty table
			return false;
		
		boolean valid = true;
		String name = (String)(model.getValueAt(row, 1));
		String WCET = (String)(model.getValueAt(row, 2));
		String dl = (String)(model.getValueAt(row, 3));
		if (column == 1) {	// the job name	
			String s = editorTab.jobNameValidity(name, row);
					// jobPropValidity(name, WCET, deadline, row);	
			if (s.equalsIgnoreCase("valid") == false) {
				Dialogs.showErrorMsg(s);
				valid = false;
			}
		}		
		else if(column == 2)		// the job WCET	
			valid = Util.checkIntVal(WCET, "WCET");
		else if(column == 3)		// the job deadline
			valid = Util.checkIntVal(dl, "Deadline");

		if (valid) {
			Task t = editorTab.getSelectedTask();
			JobType job = t.getJob(row);
			editorTab.trackChange(job.getModifyChange());
			
			job.updateName(name);
			job.updateWCET_DL(str2int(WCET), str2int(dl));			
		}
		return valid;
	}
	
	private int str2int(String str) {
		return  Integer.parseInt( str );
	}

	public void selectFirstTask() {
		if (taskSetTable.getRowCount() > 0)
			taskSetTable.setRowSelectionInterval(0, 0);		
	}
}