package taskeditor;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JProgressBar;
import javax.swing.JToolBar;

import main.Util;

public class ToolBar extends JToolBar implements ActionListener {

	private	EditorTab editTab;
	private	JButton newButton;
	private	JButton openButton;
	private	JButton saveButton;
	private	JButton svAsButton;
	private	JButton undoButton;
	private	JButton redoButton;
	private	JButton randButton;
	private	JButton confButton;
	private	JButton exitButton;
	
	public ToolBar(EditorTab parent, JProgressBar progress) {
		editTab = parent;
		
		ImageIcon iconNew = Util.getIcon("/new.gif");
		ImageIcon iconOpen = Util.getIcon("/open.gif");
		ImageIcon iconSave = Util.getIcon("/save.gif");
		ImageIcon iconSvAs = Util.getIcon("/saveas.gif");
		ImageIcon undoSave = Util.getIcon("/undo.gif");
		ImageIcon redoSave = Util.getIcon("/redo.gif");
		ImageIcon iconRand = Util.getIcon("/rand.png");

		ImageIcon iconConf = Util.getIcon("/conf.png");
		ImageIcon iconExit = Util.getIcon("/exit.png");

		newButton  = new JButton(iconNew);
		openButton = new JButton(iconOpen);
		saveButton = new JButton(iconSave);
		svAsButton = new JButton(iconSvAs);
		undoButton = new JButton(undoSave);
		redoButton = new JButton(redoSave);
		randButton = new JButton(iconRand);
		confButton = new JButton(iconConf);
		exitButton = new JButton(iconExit);
		
		
		// set up the jbutton
		newButton.setToolTipText("Create new task set");
		saveButton.setToolTipText("Save");
		svAsButton.setToolTipText("Save As");
		undoButton.setToolTipText("Undo");
		redoButton.setToolTipText("Redo");
		randButton.setToolTipText("Random task set");
		confButton.setToolTipText("Configuration");
		
		newButton .addActionListener(this);
		openButton.addActionListener(this);
		saveButton.addActionListener(this);
		svAsButton.addActionListener(this);
		undoButton.addActionListener(this);
		redoButton.addActionListener(Util.createUnavailableFeatureListener());
		randButton.addActionListener(this);
		confButton.addActionListener(this);
		exitButton.addActionListener(Util.createExitActionListener());		
		redoButton.setEnabled(false);	// not yet implemented
		
		add(newButton);
		add(openButton);
		add(saveButton);
		add(svAsButton);
		addSeparator();
		add(undoButton);
		add(redoButton);
		addSeparator();
		add(randButton);
		add(Box.createHorizontalStrut(80));
		add(Box.createHorizontalStrut(40));
		add(progress);		
		add(Box.createHorizontalStrut(200));		
		add(Box.createHorizontalGlue());
		add(confButton);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		if (src == newButton)
			editTab.newPressed();
		else if (src == openButton)
			editTab.openPressed();		
		else if (src == saveButton)
			editTab.save();		
		else if (src == svAsButton)
			editTab.saveAs();		
		else if (src == undoButton)
			editTab.undoDispatcher();
		else if (src == randButton)
			editTab.randomButtPressed();
		else if (src == confButton)
			editTab.configPressed();
		else 
			System.out.println("Warning: Unimplemented action button" + e.getSource());		
	}

}
