package taskeditor.listeners;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import main.Edge;
import main.JobType;
import main.Task;

import taskeditor.DrawStatus;
import taskeditor.DrawingPane;
import taskeditor.changetrace.Change;
import util.Dialogs;
import util.GUIUtil;


public class JobIconListener extends MovableListener implements ActionListener {
	private JobType 	clickedNode;
	private JobType 	job; 
	
	public JobIconListener(DrawingPane pane, JobType owner) {
		super(pane, GUIUtil.HAND_CURSOR);
		job = owner;
	}
	
	protected void selected(MouseEvent e) {
        drawPane.jobSelected(job);
        // job.getLabel().setFont(new Font("", Font.BOLD, 14));            
        
        if (drawStatus() == DrawStatus.ARC && GUIUtil.leftClick(e)) {
        	if (drawPane.getNewEdgeSrcJob() == null) { // set the source job of the edge            	        	
        		drawPane.setNewEdgeSrcJob(job);
        		drawPane.setNewEdgeNails( new Vector<Point>() );
        	} else {     // the source of the edge already selected
            	Task curr_task = selectedTask();
            	/*if(curr_task.contains_edge(drawPane.getNewEdgeSrcJob(), job)) // check if there is already an edge between these two jobs
            	{
            		showErrorMsg("There is already an edge between these nodes");
            	}
            	else*/
            	{
            		if (newEdgeSrcJob().equals(job)) {	// the source and target are the same node            		
            			if (newEdgeNails().size() == 0)		// let the user put at least one nail out of the node
            				return;
            		}
            		String interRel = "15"; // default inter-release time // Util.getTextFromUser("Please Enter Inter-Release Time:", "");
                	int InterRelease = Integer.parseInt(interRel);
                	Edge newEdge = new Edge(curr_task.get_edge_Index(), newEdgeSrcJob(), job, InterRelease, newEdgeNails());
                	curr_task.addEdge(newEdge);
                	
                	trackChange(newEdge.getCreateChange());
                	
                	drawPane.setNewEdgeSrcJob(null);
                    drawPane.setNewEdgeNails(null);
                    drawPane.edgeSelected(newEdge);
                    redrawSelectedTask();
            	}
            }
        }
    }

    public void dragged(int dx, int dy) {
    	job.x = job.x + dx;
        job.y = job.y + dy;
        job.moveLabel(dx, dy);	
    }    
    
    public void mouseClicked(MouseEvent e) {
    	if (GUIUtil.rightClick(e) && drawStatus() == DrawStatus.NONE) {
    		JComponent c = (JComponent) e.getSource();
    		int x = e.getPoint().x;
    		int y = e.getPoint().y;
        	clickedNode = job;
        	createPopupMenu().show(c, x, y);        	
        }
    	else if (e.getClickCount() == 2) {
            rename_job(job);
        }
    }
    	
    public JPopupMenu createPopupMenu() {
        JPopupMenu popupMenu = new JPopupMenu();
        JCheckBoxMenuItem mi1 = new JCheckBoxMenuItem(SET_AS_START);
        JMenuItem 		  mi2 = new JMenuItem(RENAME);
        JMenuItem 		  mi3 = new JMenuItem(CODESEG);
        JMenuItem 		  mi4 = new JMenuItem(BRANCH_COND);
        JMenuItem 		  mi5 = new JMenuItem(DELETE_NODE);
        
        mi1.addActionListener(this);
        mi2.addActionListener(this);
        mi3.addActionListener(this);
        mi4.addActionListener(this);
        mi5.addActionListener(this);
        
        boolean isInitialJob = selectedTask().getStartJob() == clickedNode;
        mi1.setSelected(isInitialJob);
        
        popupMenu.add(mi1);
        popupMenu.add(mi2);
        popupMenu.addSeparator();
        popupMenu.add(mi3);            
        popupMenu.add(mi4);        
        popupMenu.addSeparator();
        popupMenu.add(mi5);            
        
        return popupMenu;
      }

    public void actionPerformed(ActionEvent e) {
    	Task t = selectedTask();        
    	if (e.getActionCommand().equals(SET_AS_START)) {
    		t.trackChange(t.getModifyChange());
    		updateStartJob(t);
    		redrawSelectedTask();
    	} else if (e.getActionCommand().equals(RENAME)) {        		
    		rename_job(clickedNode);
    	} else if (e.getActionCommand().equals(CODESEG)) {        		
    		set_job_code(clickedNode);
    	} else if (e.getActionCommand().equals(BRANCH_COND)){        		
    		set_branch_code(clickedNode);
    	} else if (e.getActionCommand().equals(DELETE_NODE)) {
    		Change ch = t.getRemoveGroupChange(clickedNode);
    		drawPane.trackChange(ch);
    		
    		drawPane.removeJob(clickedNode);
    		clickedNode = null;
    		drawPane.updateTaskInfo();
    		redrawSelectedTask();
    	}
	}
    
    private void updateStartJob(Task t) {
    	if (clickedNode == t.getStartJob())
    		t.setStartJob(null);
    	else
    		t.setStartJob(clickedNode);
	}

	private void redrawSelectedTask() {
    	drawPane.redrawSelectedTask();		
	}

	private void set_branch_code(JobType job) {
    	Task t = selectedTask();
		if (t == null)
			return;
		job.getBranchCondFromUser();			
	}

	private void rename_job(JobType job) {
    	String new_name = Dialogs.getTextFromUser("Name:", job.getName()); 
        if (new_name == null) 
        	return;       // user has cancelled
        
        int jobNo = selectedTask().getJobNo(job.getName()); 
    	String valid = drawPane.jobNameValidity(new_name, jobNo);
    	if (valid.compareToIgnoreCase("valid") != 0) {
    		showErrorMsg(valid);
    		rename_job(job);   // recursive call
    		return;
    	}
    	trackModifChange();
		job.updateName(new_name);
		drawPane.updateTaskInfo();
	}

	private void set_job_code(JobType job) {
		String oldCode = job.getCode();
		oldCode = (oldCode == null) ? "" : oldCode;
		String code = GUIUtil.getCodeFromUser(oldCode, "Job Type Code", "Code block:");
		if (code == null)	// user canceled
			return;
		job.setCode(code);
	}
	
	private Task selectedTask() {
		return drawPane.selectedTask();
	}
	
	protected void trackModifChange() {
    	Change ch = job.getModifyChange();
    	trackChange(ch);		
	}
    
    private void trackChange(Change ch) {
    	drawPane.trackChange(ch);		
	}

	private Vector<Point> newEdgeNails() {		
		return drawPane.newEdgeNails();
	}

	private JobType newEdgeSrcJob() {
    	return drawPane.getNewEdgeSrcJob();
    }
    
	private 	static final String SET_AS_START = "Initial Job";
    private 	static final String RENAME 		 = "Rename";
    private 	static final String CODESEG 	 = "Job Code";
    private 	static final String BRANCH_COND	 = "Branch Conditions";
    private 	static final String DELETE_NODE  = "Delete";    	
}
