package taskeditor.listeners;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;

import taskeditor.DrawStatus;
import taskeditor.DrawingPane;
import util.Dialogs;
import util.GUIUtil;

import main.Util;

public abstract class MovableListener extends MouseAdapter implements MouseMotionListener {
	protected int 			startX;
	protected int 			startY;
	protected DrawingPane 	drawPane;
    protected boolean		dragStarted = false;
    private  Cursor         cursor;

    public MovableListener(DrawingPane pane) {
    	this(pane, GUIUtil.MOVE_CURSOR);  // default: MOVE_CURSOR
	}
    
    public MovableListener(DrawingPane pane, Cursor c) {
		drawPane = pane;
		cursor   = c;
	}
	    
    /*  
     *  MouseEvent handlers:
     */	
    public void mouseMoved(MouseEvent e) {    	
    	if (drawStatus() != DrawStatus.ARC) {
    		drawPane.setCursor(cursor);
    	}

    	JComponent c = (JComponent) e.getSource();
    	MouseEvent convertEvent = SwingUtilities.convertMouseEvent(e.getComponent(), e, c.getParent());
    	convertEvent.setSource(this);    	
    	drawPane.dispatchEvent(convertEvent);  // To update the pointer coordination 
    }

    public void mousePressed(MouseEvent e) {
        startX = e.getX();
        startY = e.getY();
        selected(e);
        drawPane.nodeChanged();   // to trigger repaint of the edge with new color
    }    

    public void mouseDragged(MouseEvent e) {   
    	updatePointInfo(e); 
    	
    	if (drawStatus() == DrawStatus.ARC) {
    		return;
    	}

    	if (dragStarted == false) {
	    	trackModifChange();
	    	dragStarted = true;
	    }        
    	
        int dx = e.getX() - startX;
        int dy = e.getY() - startY;

        JComponent c = (JComponent) e.getSource();
        if (drawPane.snaped()) {
        	int oldX = c.getX();
            int oldY = c.getY();
            int newX = oldX + dx;
            int newY = oldY + dy;            
	        newX = Util.discritize(newX, drawPane.gridX);
	        newY = Util.discritize(newY, drawPane.gridY);
	        dx = newX - oldX;
	        dy = newY - oldY;	        
        }
        c.setLocation(c.getX() + dx, c.getY() + dy);
        dragged(dx, dy);
        nodeChanged();    // to trigger painting the edges
    }
    
    public void mouseReleased(MouseEvent e) {
    	dragStarted = false;
    }
    
    /*  
     *  Auxiliary functions
     */
	protected DrawStatus drawStatus() {
    	return drawPane.drawStatus();
	}
	
	private void nodeChanged() {
		drawPane.nodeChanged();
	}	
	
	private void updatePointInfo(MouseEvent e) {
    	Point p = e.getPoint();	  // relative to the icon
    	p.x = p.x + ((JComponent) e.getSource()).getX();
    	p.y = p.y + ((JComponent) e.getSource()).getY();    	
    	drawPane.updatePointInfo(p);
	}
	
	protected void showErrorMsg(String msg) {
		Dialogs.showErrorMsg(msg);
	}
	
	protected abstract void selected(MouseEvent e);
	protected abstract void dragged(int dx, int dy);
	protected abstract void trackModifChange();
}
