package taskeditor.movables;

import java.awt.Point;
import java.util.Vector;

import main.Edge;
import main.Util;

public class Nails {
	
	private Vector<Nail>	nails;
	private	Edge			edge;
		
	public Nails(Vector<Point> points, Edge e) {
		nails = new Vector<Nail>();
		for (int i = 0; i < points.size(); i++) {
			Nail n = new Nail(points.get(i), e);
			nails.add(n);
		}
		edge = e;
	}

	public void addNail(int x, int y) {
		Nail nail = new Nail(x, y, edge);
		Point[] points = getAugmentedPoints();
		for (int i = 0; i < points.length - 1; i++) {
			Point n1 = points[i];
			Point n2 = points[i+1];
			if (nail.isBetween(n1, n2)) {
				nails.add(i, nail);
				return;
			}
		}
		nails.add(nail);	// add at the end otherwise		
	}

	// Returns true if the point is near this edge
	public boolean testPoint(Point p, int margin) {
		Point[] points = getAugmentedPoints();
		for (int i = 0; i < points.length - 1; i++) {
			Point n1 = points[i];
			Point n2 = points[i+1];			
			if ( Util.dotIsNearLine(p, n1.x, n1.y, n2.x, n2.y, margin) )
				return true;						
		}		
		return false;
	}
	
	
	/* Basically, "nails" contains only the points on the edge,
	 * 	excluding the source and dest. job locations.
	 * This function "augments" the list with these points. 
	 */   
	public Point[] getAugmentedPoints() {
		Vector<Point> result = clonePoints();
		if (sourceOrDestNotSet())	
			return toArray(result);
		
		int x0 = edge.source().x; // getIcon().getX()+30;
        int y0 = edge.source().y; //.getIcon().getY()+30;
		
		result.add(0, new Point(x0, y0));
    	Point last = result.lastElement();  // last nail before the destination
    	int xn = (int)last.x; 
    	int yn = (int)last.y;
    	int xz = edge.destin().x; // getLabel().getX()+30;    // the destination coordination
        int yz = edge.destin().y; //.getLabel().getY()+30;
        double arc = Math.sqrt((xz - xn)*(xz - xn) + (yz - yn)*(yz - yn));
        double sin = (arc == 0) ? 0 : (yz - yn) / arc;
        double cosine = (arc == 0) ? 0 : (xz - xn) / arc;            
        Point last_point = new Point((int)(xz - RADIUS*cosine), (int)(yz - RADIUS*sin));
        result.add(last_point); // the coordination of the border point of the destination to which the edge connects    	
        return toArray(result);
	}	

	public boolean sourceOrDestNotSet() {
		if (edge.source() == null || edge.destin() == null)
			return true;
		if (edge.source().getIcon() == null || edge.destin().getIcon() == null)
			return true;
		return false;
	}
	
	public Vector<Point> clonePoints() {
		Vector<Point> cn = new Vector<Point>();	
		for (int i = 0; i < nails.size(); i++)
			cn.add(new Point(nails.get(i).x(), nails.get(i).y()));
		return cn;
	}

	public Nail[] toArray() {
		return nails.toArray(new Nail[]{});
	}
	
	private Point[] toArray(Vector<Point> points) {
		return points.toArray(new Point[]{});
	}

	public int size() {
		return nails.size();
	}

	public Nail get(int i) {
		return nails.get(i);
	}

	public void setEdge(Edge e) {
		edge = e;		
	}
	
	static final private int RADIUS = 22;	// Radius of job icon

}
