package util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.LinkedList;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;

import main.JManager;
import main.Util;


public class Dialogs {

	private	static	JFrame	frame;
	static final 	int 	WARN = JOptionPane.WARNING_MESSAGE;
	static final 	int		INFO = JOptionPane.INFORMATION_MESSAGE;
	static final 	int		QUES = JOptionPane.QUESTION_MESSAGE;
	static final 	int		ERRO = JOptionPane.ERROR_MESSAGE;

	public static void showWarnMsg(String msg) {
		JOptionPane.showMessageDialog(frame, msg, "Warning", WARN);
	}
	
	public static void showInfoMsg(String msg) {
		JOptionPane.showMessageDialog(frame, msg, "Information", INFO);
	}
	
	public static int showOptionMsg(Object msg, String title, String[] options) {		
		return JOptionPane.showOptionDialog(frame,
				msg,
				title,
			    JOptionPane.YES_NO_OPTION,
			    QUES,
			    null,     //do not use a custom Icon
			    options,  //the titles of buttons
			    options[0]); //default button title		
	}
	
	/*
	 * Shows a one-field window to get a value from the user Returns null if
	 * user canceled or closed the dialog box
	 */
	public static String getTextFromUser(String field, String primaryVal) {
		Object value = JOptionPane.showInputDialog (frame, field, primaryVal);

		String result = null;
		if (value != null) // null means the user calcelled the modification
			result = (String) value;
		return result;
	}
	
	public static int showOkCancelPanel(JPanel p, String title) {
		Border empty = GUIUtil.getEmptyBorder(10);
        p.setBorder(empty);
        
        JPanel pnl = new JPanel();
        pnl.add(p);
        pnl.setBorder(GUIUtil.getBlackTitleBorder(title));
        
        int o = JOptionPane.OK_CANCEL_OPTION;
		String t = "Settings";
		String[] opts = new String[]{ "   Ok   ", "  Cancel  " };
        
		return JOptionPane.showOptionDialog (frame, pnl, t, o, QUES, null, opts, "");
	}

	public static void showSyntaxErrDialog(String title,
			String msg, String details) {
		SyntaxErrDialog dlg = new SyntaxErrDialog(frame, title, msg, details); 
		dlg.appear();		
	}
	
	public static void showDefaultSyntaxErrDialog(LinkedList<String> errors) {
		String title = "Syntax Errors";
		String details = "";
		for(String s : errors)
			details += s + "\r\n";

		final String Space = "      "; 
		final String Space4 = Space + Space + Space + Space;
		String msg = "The model contains syntax errors.";		
		msg = Space + msg + Space4 + Space4 + Space;
		Dialogs.showSyntaxErrDialog(title, msg, details);
	}

	/*public static void showErrorMsg(String msg, Component c) {
		showErrorMsg(c, msg);
	}*/
	
	public static void showErrorMsg(String msg) {
		JOptionPane.showMessageDialog(frame, msg, "Error", ERRO);
	}

	public static Color showColorDialog(final Color currentColor) {
		final JColorChooser colorChooser;
        colorChooser = new JColorChooser();
        colorChooser.setColor(currentColor);

        class ColorListener implements ActionListener {
        	Color userColor;
        	
        	public ColorListener(Color c) {
        		userColor = c;
        	}
        	
    		public void actionPerformed(ActionEvent e) {
    			userColor = colorChooser.getColor();
    		}
    		
    		public Color getColor() {
    			return userColor;
    		}
    	}
        
        ColorListener listener = new ColorListener(currentColor);
        JDialog dialog = JColorChooser.createDialog(frame,
                "Pick a Color",
                true,  //modal
                colorChooser,
                listener,  //OK button handler
                null); //no CANCEL button handler
        dialog.setVisible(true);
        return listener.getColor();
	}	

	public static String showOpenDialog(String tit, String path, String ext) {
		JFileChooser dialog = Util.getFileChooser(tit, path, ext);
		int option = dialog.showOpenDialog(frame);
		if (option != JFileChooser.APPROVE_OPTION)
			return null; // User canceled or closed the dialogues.
		
		File file = dialog.getSelectedFile();
		return file.getPath();		
	}
	
	public static Component ancestor(Component cmp) {
		return SwingUtilities.getAncestorOfClass(JFrame.class, cmp);
	}

	public static void setAncestor(JManager jf) {
		frame = jf;
	}
}


class SyntaxErrDialog extends JDialog implements ActionListener {
	boolean detailsShown = false;
	String details;
	
    public SyntaxErrDialog(JFrame parent, String title, String msg, String dtl) {
    	super(parent, title, true);
    	details = dtl;
	    adjustLocation(parent);
	    
	    JPanel myPanel = new JPanel(new GridLayout(3, 1, 10, 0));
		myPanel.add(new JLabel());
        myPanel.add(new JLabel(msg));	  
	    
        JPanel buttonPane = new JPanel();
        JButton button1 = new JButton("OK");
        JButton button2 = new JButton("Details >>");
        buttonPane.add(button1); 
        buttonPane.add(button2);
        button1.addActionListener(this);
        button2.addActionListener(this);    
    
        JPanel fixedP = new JPanel(new BorderLayout());
  	    fixedP.add(myPanel, BorderLayout.NORTH);
        fixedP.add(buttonPane, BorderLayout.CENTER);
    
        getContentPane().add(fixedP, BorderLayout.NORTH);
        //getContentPane().add(new JLabel("   "), BorderLayout.CENTER);
    
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        // setMaximumSize(new Dimension(50, 1000));
        pack();        
    }
    
    public void appear() {
    	setVisible(true);
    }
    
    private void adjustLocation(JFrame parent) {
    	if (parent != null) {
    		Dimension parentSize = parent.getSize();
    		Point p = parent.getLocation(); 
		    setLocation(p.x + parentSize.width / 4, p.y + parentSize.height / 4);
    	}
    }

    public void actionPerformed(ActionEvent e) {
	  String src = ((JButton)e.getSource()).getText();
	  if(src.equalsIgnoreCase("ok"))
	  {
		  setVisible(false);
		  dispose();
	  }
	  else // SHOW DETAILS
	  {
		  Dimension d = getPreferredSize();
		  if(detailsShown == false)
		  {			  
			  JTextArea text = new JTextArea(details);			  	
			  text.setFont(new Font(text.getFont().getName(), Font.PLAIN, 14));
			  JScrollPane sccr =  new JScrollPane(text);
			  d.width = d.width; // + 150;
			  d.height = d.height + 200;
			  setPreferredSize(d);
			  getContentPane().add(sccr, BorderLayout.CENTER);
			  ((JButton)e.getSource()).setText("Details <<");
		  }
		  else {
			  getContentPane().remove(1);	// removing the added textArea
			  d.width = d.width; // - 150;
			  d.height = d.height - 200;
			  setPreferredSize(d);			  
			  ((JButton)e.getSource()).setText("Details >>");
		  }
		  detailsShown = !detailsShown;
		  pack();	
		  repaint();
		  revalidate();
	  }
  }
    
}
