package util;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;

import taskeditor.DrawingPane;
import taskeditor.changetrace.Change;

import main.Edge;
import main.EdgeParam;
import main.Util;
import enums.ActionType;
import enums.SendSemantic;

public class EdgeGUIUtil {
    
	public static JPopupMenu createEdgeMenu(Edge e, DrawingPane pane, Point pnt) {
		return new NailMenu(pane, e, pnt);	    
	}	

	public static EdgeParam getEdgeParamFromUser(EdgeParam param) {
		
		ActionType[] actionTypes = {ActionType.None, ActionType.Send, ActionType.Receive};
		SendSemantic[] sendTypes = {SendSemantic.Blocking, SendSemantic.Non_Blocking};
		
        // label and text fields
		final JLabel l1 = new JLabel( "Inter-Release Time:" );
		final JLabel l2 = new JLabel( "Action Type:" );
		final JLabel l3 = new JLabel( "Action Name:" );
		final JLabel l4 = new JLabel( "Send Semantics:" );
                
        final JTextField tf1 = GUIUtil.formattedField(param.interRel + ""); 
        final JComboBox<ActionType> cb1 = new JComboBox<ActionType>(actionTypes);
        final JTextField tf3 = GUIUtil.formattedField(param.action);
        final JComboBox<SendSemantic> cb4 = new JComboBox<SendSemantic>(sendTypes);
        
        cb1.setSelectedItem(param.aType);
        if (param.aType == ActionType.None) {
        	l3.setEnabled(false);
        	l4.setEnabled(false);
        	tf3.setEnabled(false);
        	cb4.setEnabled(false);
        } else if (param.aType == ActionType.Send) {
        	cb4.setSelectedItem(param.send);
        } else if (param.aType == ActionType.Receive) {
        	cb4.setEnabled(false);
        	// tf3.setEnabled(false);
        }
        cb1.addActionListener(
        		new ActionListener(){
                    public void actionPerformed(ActionEvent e){
                        JComboBox<ActionType> combo = (JComboBox<ActionType>)e.getSource();
                        ActionType type = (ActionType)combo.getSelectedItem();
                        if (type == ActionType.None) {
                        	l3.setEnabled(false);
                        	l4.setEnabled(false);
                            tf3.setEnabled(false);
                            cb4.setEnabled(false);
                        }
                        else {
                        	l3.setEnabled(true);
                            tf3.setEnabled(true);
                            if (type == ActionType.Send) {
                            	cb4.setEnabled(true);
                            	l4.setEnabled(true);
                            }
                            else {
                            	cb4.setEnabled(false);
                            	l4.setEnabled(false);
                            }
                        }
                    }
                }    
        		);
            	 
        // put all together
        JPanel p = new JPanel( new GridLayout(4, 2, 10, 5));
        p.add(l1);
        p.add(tf1);
        p.add(l2);
        p.add(cb1);
        p.add(l3);
        p.add(tf3);
        p.add(l4);
        p.add(cb4);
        
        String title = "Edge Parameters";
        int result = Dialogs.showOkCancelPanel(p, title);
                
        if (result == JOptionPane.OK_OPTION) {
        	EdgeParam res = new EdgeParam();            
        	String in = tf1.getText();
        	try {
        		res.interRel = Integer.parseInt(in);        		
        	} catch (NumberFormatException ex) {
        		Dialogs.showErrorMsg("Inter-release time cannot be negative.");
        		return getEdgeParamFromUser(param);
        	}        	
        	res.aType = (ActionType)cb1.getSelectedItem();
        	if (res.aType != ActionType.None) {
        		res.action = tf3.getText();
        		if (res.action.length() == 0) {
            		Dialogs.showErrorMsg("Action name cannot be empty.");
            		return getEdgeParamFromUser(param);
            	} else if (Util.isWord(res.action) == false) {
            		Dialogs.showErrorMsg("Action cannot contain non-word characters.");
            		return getEdgeParamFromUser(param);
            	}
        		if (res.aType == ActionType.Send) {
        			res.send = (SendSemantic)cb4.getSelectedItem();
        		}
        	}
        	return res;
        } else        
        	return null;
	}
}


/*
 * A customized popup menu
 */
class NailMenu extends JPopupMenu implements ActionListener {		

	final static 	String EDIT 		 = "Edit";
	final static 	String ADD_NAIL 	 = "Add Nail";
    final static	String ADD_CODE 	 = "Add Code";
    final static	String DELETE_EDGE = "Delete Edge";
    
	private Edge 		edge;
	private DrawingPane drawPane;
	private Point 		topLeft;
	
	public NailMenu(DrawingPane dp, Edge e, Point loc) {
		drawPane = dp;
		edge = e;
		topLeft = loc;
		
		JMenuItem mi1 = new JMenuItem(EDIT);
		JMenuItem mi2 = new JMenuItem(ADD_CODE);
	    JMenuItem mi3 = new JMenuItem(ADD_NAIL);
	    JMenuItem mi4 = new JMenuItem(DELETE_EDGE);
	    mi1.addActionListener(this);
	    mi2.addActionListener(this);
	    mi3.addActionListener(this);
	    mi4.addActionListener(this);
	    
	    add(mi1);
	    add(mi2);
	    addSeparator();
	    add(mi3);
	    addSeparator();
	    add(mi4);
	}
	
	public void actionPerformed(ActionEvent e) {
		String cmd = e.getActionCommand();
		if (cmd.equals(EDIT)) {
			trackChange();
			edge.editRequest();
    		drawPane.updateTaskInfo();
		} else if (cmd.equals(ADD_NAIL)) {
    		trackChange();
    		edge.addNail(topLeft.x, topLeft.y);
    	} else if (cmd.equals(ADD_CODE)) {
    		trackChange();
    		getEdgeCode(edge);
		} else if(cmd.equals(DELETE_EDGE)) {
    		drawPane.trackChange(edge.getRemoveChange());
    		drawPane.removeEdge(edge);	        		
		}
    	drawPane.redrawSelectedTask();
	}
	
	private void trackChange() {
		Change ch = edge.getModifyChange();
		drawPane.trackChange(ch);        		
	}
	
	private void getEdgeCode(Edge e) {
		String mssge = "User code:";
		String title = "Transition Code";
		e.setCode(GUIUtil.getCodeFromUser(e.code(), title, mssge));
	}
}