/*
 * This class provides utilities related to graphical objects
 * 	which are going to be shown in the GUI
 * The class is completely stateless
 */

package util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.geom.QuadCurve2D;
import java.io.File;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import javax.swing.border.LineBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;

import main.Edge;
import main.JManager;
import main.Util;

import taskeditor.DrawingPane;


public class GUIUtil {

	public static Border getGrayTitleBorder(String title) {
		Border border = GUIUtil.getTitleBorder(title, Color.gray);
		return border;
	}

	public static Border getBlackTitleBorder(String title) {
		Border border = GUIUtil.getTitleBorder(title, Color.black);
		return border;
	}

	public static Border getTitleBorder(String title, Color color) {
		TitledBorder border = BorderFactory.createTitledBorder(new EtchedBorder(), title);
		border.setTitleColor(color);		
		return border;
	}
	
	public static Border getEmptyBorder(int margin) {
		Border border = BorderFactory.createEmptyBorder(margin, margin, margin, margin);
		return border;
	}
	
	public static Border getRoundedBorder(String title, Color color) {

		LineBorder roundedLineBorder = new LineBorder(Color.black, 1, true);
		TitledBorder roundedBorder = new TitledBorder(roundedLineBorder, title);
		//TitledBorder border = BorderFactory.createTitledBorder( roundedBorder, title );
		//border.setTitleColor(color);
		return roundedBorder;
	}

	public static JScrollPane customScroll(Component parent) {
		JScrollPane scroll = new JScrollPane(parent);
		scroll.getVerticalScrollBar().setUnitIncrement(14);  // set the speed of scroll
		scroll.getHorizontalScrollBar().setUnitIncrement(14);  // set the speed of scroll
		scroll.setAutoscrolls(true); //enable synthetic drag events
		return scroll;
	}

	public static void enable(JComponent text) {
		text.setEnabled(true);
		text.setForeground(Color.BLACK);
	}
	
	public static void disable(JComponent text) {
		text.setEnabled(false);
		text.setForeground(Color.GRAY);
	}

	public static JLabel createTitle(String name) {
		JLabel title = new JLabel(name, SwingConstants.CENTER);
		title.setForeground(Color.white);
        title.setBackground(new Color(0,0,150));
        title.setOpaque(true);
        return title;
	}

	public static JSpinner cearteSpinner(int init, int min, int max) {
		int step = 1;
		SpinnerNumberModel model = new SpinnerNumberModel(init, min, max, step);		 
		return new JSpinner(model);
	}

	// Sets the width and height of the given component
	public static void setSize(JComponent c, int w, int h) {
		c.setSize(w, h);
		c.setPreferredSize(new Dimension(w ,h));		
	}

	public static void color(JComponent com, Color col) {
		com.setBackground(col);
		Component[] components = com.getComponents();
		for(Component child : components)
			if(child instanceof JComponent)
				color((JComponent)child, col);
	}

	public static void alignHeader(JTable table) {
		JTableHeader jtableHeader = table.getTableHeader();
        DefaultTableCellRenderer rend = (DefaultTableCellRenderer) table.getTableHeader().getDefaultRenderer();
        rend.setHorizontalAlignment(JLabel.CENTER);
        jtableHeader.setDefaultRenderer(rend);		
	}

	public static void alignCells(JTable table, int cols) {

		DefaultTableCellRenderer centerRenderer = new DefaultTableCellRenderer();
		centerRenderer.setHorizontalAlignment(SwingConstants.CENTER);
		for (int i = 0; i < cols; i++) 
			table.getColumnModel().getColumn(i).setCellRenderer(centerRenderer);		
	}

	// called from DrawingPane
	public static void drawSampleEdge(Graphics g, Point[] nails, boolean selected) {
    	if(selected == true)
    		g.setColor(Color.blue);
    	else
    		g.setColor(Color.black);
    	
    	final int z = 15; // round corner
        final int s = 13; // arrow length

        QuadCurve2D.Double quad = new QuadCurve2D.Double();

        int ox2=0, oy2=0;
        Point n = null, n2 = null;

        for(int i = 0; i < nails.length - 1; i++) {        	
        	n  =  nails[i];
        	n2  = nails[i+1];
        	int x1 = (int)n.getX();
        	int y1 = (int)n.getY();
        	int x2 = (int)n2.getX();
        	int y2 = (int)n2.getY();
        	int dx = x2-x1;
        	int dy = y2-y1;
        	int l = (int)Math.sqrt(Math.pow(dx,2)+Math.pow(dy,2));          

        	// draw the nail as a rectangle
        	if (selected == true) {
        		g.setColor(Color.green);
        		g.fillRect(x1-3, y1-3, 6, 6);
        		g.setColor(Color.black);
        		g.drawRect(x1-3, y1-3, 6, 6);	     
        		g.setColor(Color.blue);
        	}

        	if (i > 0) {
        		x1 += (l>((z)<<1))?(z)*dx/l:dx>>1;
        		y1 += (l>((z)<<1))?(z)*dy/l:dy>>1;
        		quad.setCurve(new Point(ox2,oy2), new Point((int)n.getX(),
        				(int)n.getY()), new Point(x1,y1));
        		((Graphics2D) g).draw(quad);
        	}
        	if (i < nails.length - 2) {
        		x2 -= (l>((z)<<1))?(z)*dx/l:dx>>1;
        		y2 -= (l>((z)<<1))?(z)*dy/l:dy>>1;
        	}
        	ox2 = x2;
        	oy2 = y2;

        	if (i < nails.length - 2)
        		g.drawLine(x1, y1, x2, y2);
        	else
        		//g.drawLine(x1, y1, x2, y2);
        		Util.drawArrow(g, x1, y1, x2, y2, s);                 
    	}		
	}

	public static void dotPane(Graphics g2d, JPanel pane, int dx, int dy) {
        Color def_col = g2d.getColor();
    	g2d.setColor(Color.gray);
    	Dimension size = pane.getSize();
        Insets insets = pane.getInsets();
        int w = size.width - insets.left - insets.right;
        int h = size.height - insets.top - insets.bottom;

        int width = pane.getWidth();
        int height = pane.getHeight();
        		
       for (int i = 0; i <= width/dx; i++) {        	
            for (int j = 0; j <= height/dy; j++) {                
                int x = Math.abs(i*dx) % w;
                int y = Math.abs(j*dy) % h;
                g2d.drawLine(x, y, x, y);
            }
        }
        g2d.setColor(def_col);        
    }
	
	public static void gridPane(Graphics g2d, JPanel pane) {
        Color def_col = g2d.getColor();
        int darkC = 200;
        // Color c = new Color(darkC, darkC, darkC, 150); 
        Color c = new Color(darkC, darkC, darkC);    	
        // pane.setBackground(c);
        
        g2d.setColor(Color.LIGHT_GRAY);
    	Dimension size = pane.getSize();
        Insets insets = pane.getInsets();
        int w = size.width - insets.left - insets.right;
        int h = size.height - insets.top - insets.bottom;
        
        int width = pane.getWidth();
        int height = pane.getHeight();
        
        // Draw grid:
        for (int i = 0; i <= width/10; i++) {
        	int x = Math.abs(i*10) % w;
            int y1 = 0;
            int y2 = height;
            g2d.drawLine(x, y1, x, y2);
        }
        for (int i = 0; i <= height/10; i++) {
        	int x1 = 0;
            int x2 = width;
            int y = Math.abs(i*10) % w;            
            g2d.drawLine(x1, y, x2, y);
        }
        
        g2d.setColor(def_col);
    }
	
	public static void color(JLabel l, Color c) {
		if(l != null)
			l.setForeground (c);
	}


	public static boolean rightClick(MouseEvent e) {
		// if(e.getButton() == MouseEvent.BUTTON3)
		if (SwingUtilities.isRightMouseButton(e))
			return true;
		return false;
	}        

	public static boolean leftClick(MouseEvent e) {
    	// if(e.getButton() == MouseEvent.BUTTON1)
    	if (SwingUtilities.isLeftMouseButton(e))    			
    		return true;
		return false;
	}
    
	public static boolean doubleClick(MouseEvent e) {
    	if(leftClick(e) && e.getClickCount() == 2)
    		return true;
		return false;
	}

	public static JTextField formattedField(String v) {
		JTextField tf = new JTextField(5);
        tf.setFont(new Font("", 1, 12));
        tf.setText(v);
        return tf;
	}

	public static final Cursor MOVE_CURSOR  = Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR);
	public static final Cursor HAND_CURSOR  = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
	public static final Cursor DFLT_CURSOR  = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
	public static final Cursor CROSS_CURSOR = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
	// for code synthesis
	// using this method, a code block is given by the user
	public static String getCodeFromUser(String currentContent, String title, String msg) {
		
		// Text area for getting the code
		JTextArea textArea = new JTextArea(currentContent, 14, 45);
		textArea.setFont(textArea.getFont().deriveFont(14f));
		
	    textArea.setWrapStyleWord(true);
	    JScrollPane areaScroll = new JScrollPane(textArea);
	
	    // label and browse button
	    JLabel label = new JLabel( msg );
	    JButton browse = Util.createCodeBrowse(textArea);
	    JPanel header = new JPanel( new BorderLayout());
	    header.add(label, BorderLayout.LINE_START);
	    header.add(Box.createHorizontalGlue(), BorderLayout.CENTER);
	    header.add(browse, BorderLayout.LINE_END);
	    
	    // put together
	    JPanel myPanel = new JPanel( new BorderLayout(10, 10));
	    myPanel.add(header, BorderLayout.PAGE_START);
	    myPanel.add(areaScroll, BorderLayout.CENTER);
	
	    // show the dialogue
	    int result = Dialogs.showOkCancelPanel(myPanel, title);
	    
	    // return the code content
	    if (result == JOptionPane.OK_OPTION) 
	    	return textArea.getText();
	    else 
	    	return currentContent;
	}

	public static void useSystemLookAndFeel() {
		try {
			String laf = UIManager.getSystemLookAndFeelClassName();
			UIManager.setLookAndFeel(laf);
			// UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
		} catch (Exception ex) {
			String msg = "Problem occured when setting system look and feel.";
			System.out.println(msg);
			// Dialogs.showErrorMsg(msg);
		}		
	}
	

	public static void setLookAndFeel(String name) {
		String msg = "This Look&Feel (i.e., " + name + ") either is not supported\n" +
				"by your system or its use is forbidden.";		
		try {
		    for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
		        if (name.equals(info.getName())) {
		            UIManager.setLookAndFeel(info.getClassName());
		            return;
		        }
		    }
		    Util.warning(msg);
		} catch (Exception ex) {
			Util.warning(msg);
			// Dialogs.showErrorMsg(msg);
		}
				
	}
}
